@@ -16,7 +16,9 @@ from .history import (
     CountHistory,
     CountHistoryBase,
     CsvHistory,
+    CsvAmiciHistory,
     Hdf5History,
+    Hdf5AmiciHistory,
     NoHistory,
     HistoryBase,
     HistoryOptions,
