@@ -1,6 +1,7 @@
 """Abstract engine base class."""
+
 import abc
-from typing import List
+from typing import Any
 
 from .task import Task
 
@@ -12,7 +13,9 @@ class Engine(abc.ABC):
         pass
 
     @abc.abstractmethod
-    def execute(self, tasks: List[Task], progress_bar: bool = True):
+    def execute(
+        self, tasks: list[Task], progress_bar: bool = None
+    ) -> list[Any]:
         """Execute tasks.
 
         Parameters
