@@ -1,12 +1,13 @@
 """Engines with multi-node parallelization."""
+
 import logging
-from typing import List
+from typing import Any
 
 import cloudpickle as pickle
 from mpi4py import MPI
 from mpi4py.futures import MPIPoolExecutor
-from tqdm import tqdm
 
+from ..util import tqdm
 from .base import Engine
 from .task import Task
 
@@ -31,27 +32,32 @@ class MPIPoolEngine(Engine):
     def __init__(self):
         super().__init__()
 
-    def execute(self, tasks: List[Task], progress_bar: bool = True):
+    def execute(
+        self, tasks: list[Task], progress_bar: bool = None
+    ) -> list[Any]:
         """
         Pickle tasks and distribute work to workers.
 
         Parameters
         ----------
         tasks:
-            List of tasks to execute.
+            List of :class:`pypesto.engine.Task` to execute.
         progress_bar:
             Whether to display a progress bar.
+
+        Returns
+        -------
+        A list of results.
         """
         pickled_tasks = [pickle.dumps(task) for task in tasks]
 
         n_procs = MPI.COMM_WORLD.Get_size()  # Size of communicator
         logger.info(
-            f"Performing parallel task execution on {n_procs-1} "
-            f"workers with one manager."
+            f"Parallelizing on {n_procs - 1} workers with one manager."
         )
 
         with MPIPoolExecutor() as executor:
             results = executor.map(
-                work, tqdm(pickled_tasks, disable=not progress_bar)
+                work, tqdm(pickled_tasks, enable=progress_bar)
             )
         return results
