@@ -1,12 +1,13 @@
 """Engines with multi-process parallelization."""
+
 import logging
+import multiprocessing
 import os
-from multiprocessing import Pool
-from typing import List
+from typing import Any, Union
 
 import cloudpickle as pickle
-from tqdm import tqdm
 
+from ..util import tqdm
 from .base import Engine
 from .task import Task
 
@@ -30,43 +31,59 @@ class MultiProcessEngine(Engine):
         Defaults to the number of CPUs available on the system according to
         `os.cpu_count()`.
         The effectively used number of processes will be the minimum of
-        `n_procs` and the number of tasks submitted.
+        `n_procs` and the number of tasks submitted. Defaults to ``None``.
+    method:
+        Start method, any of "fork", "spawn", "forkserver", or None,
+        giving the system specific default context. Defaults to ``None``.
     """
 
-    def __init__(self, n_procs: int = None):
+    def __init__(
+        self,
+        n_procs: Union[int, None] = None,
+        method: Union[str, None] = None,
+    ):
         super().__init__()
 
         if n_procs is None:
             n_procs = os.cpu_count()
-            logger.warning(
-                f"Engine set up to use up to {n_procs} processes in total. "
-                f"The number was automatically determined and might not be "
-                f"appropriate on some systems."
+            logger.info(
+                f"Engine will use up to {n_procs} processes (= CPU count)."
             )
         self.n_procs: int = n_procs
+        self.method: str = method
 
-    def execute(self, tasks: List[Task], progress_bar: bool = True):
+    def execute(
+        self, tasks: list[Task], progress_bar: bool = None
+    ) -> list[Any]:
         """Pickle tasks and distribute work over parallel processes.
 
         Parameters
         ----------
         tasks:
-            List of tasks to execute.
+            List of :class:`pypesto.engine.Task` to execute.
         progress_bar:
             Whether to display a progress bar.
+
+        Returns
+        -------
+        A list of results.
         """
         n_tasks = len(tasks)
 
         pickled_tasks = [pickle.dumps(task) for task in tasks]
 
         n_procs = min(self.n_procs, n_tasks)
-        logger.info(
-            f"Performing parallel task execution on {n_procs} " f"processes."
-        )
+        logger.debug(f"Parallelizing on {n_procs} processes.")
 
-        with Pool(processes=n_procs) as pool:
-            results = pool.map(
-                work, tqdm(pickled_tasks, disable=not progress_bar)
+        ctx = multiprocessing.get_context(method=self.method)
+
+        with ctx.Pool(processes=n_procs) as pool:
+            results = list(
+                tqdm(
+                    pool.imap(work, pickled_tasks),
+                    total=len(pickled_tasks),
+                    enable=progress_bar,
+                ),
             )
 
         return results
