@@ -1,8 +1,8 @@
 """Engines without parallelization."""
-from typing import List
 
-from tqdm import tqdm
+from typing import Any
 
+from ..util import tqdm
 from .base import Engine
 from .task import Task
 
@@ -11,13 +11,15 @@ class SingleCoreEngine(Engine):
     """
     Dummy engine for sequential execution on one core.
 
-    Note that the objective itself may be multithreaded.
+    .. note:: The objective itself may be multithreaded.
     """
 
     def __init__(self):
         super().__init__()
 
-    def execute(self, tasks: List[Task], progress_bar: bool = True):
+    def execute(
+        self, tasks: list[Task], progress_bar: bool = None
+    ) -> list[Any]:
         """Execute all tasks in a simple for loop sequentially.
 
         Parameters
@@ -26,9 +28,16 @@ class SingleCoreEngine(Engine):
             List of tasks to execute.
         progress_bar:
             Whether to display a progress bar.
+
+        Returns
+        -------
+        A list of results.
         """
         results = []
-        for task in tqdm(tasks, disable=not progress_bar):
+        for task in tqdm(
+            tasks,
+            enable=progress_bar,
+        ):
             results.append(task.execute())
 
         return results
