@@ -1,5 +1,7 @@
 """Abstract Task class."""
+
 import abc
+from typing import Any
 
 
 class Task(abc.ABC):
@@ -7,7 +9,7 @@ class Task(abc.ABC):
     Abstract Task class.
 
     A task is one of a list of independent execution tasks that are
-    submitted to the execution engine to be executed using the execute()
+    submitted to the execution engine to be executed using the :func:`execute`
     method, commonly in parallel.
     """
 
@@ -15,5 +17,5 @@ class Task(abc.ABC):
         pass
 
     @abc.abstractmethod
-    def execute(self):
+    def execute(self) -> Any:
         """Execute the task and return its results."""
