@@ -16,7 +16,12 @@ from .dimension_reduction import (
     get_umap_representation_parameters,
     get_umap_representation_predictions,
 )
-from .ensemble import Ensemble, EnsemblePrediction, get_percentile_label
+from .ensemble import (
+    Ensemble,
+    EnsemblePrediction,
+    calculate_cutoff,
+    get_percentile_label,
+)
 from .util import (
     read_ensemble_prediction_from_h5,
     read_from_csv,
