@@ -1,4 +1,4 @@
-from typing import Callable, Tuple, Union
+from collections.abc import Callable
 
 import numpy as np
 
@@ -11,7 +11,7 @@ def get_umap_representation_parameters(
     n_components: int = 2,
     normalize_data: bool = False,
     **kwargs,
-) -> Tuple:
+) -> tuple:
     """
     UMAP of parameter ensemble.
 
@@ -46,12 +46,12 @@ def get_umap_representation_parameters(
 
 
 def get_umap_representation_predictions(
-    ens: Union[Ensemble, EnsemblePrediction],
+    ens: Ensemble | EnsemblePrediction,
     prediction_index: int = 0,
     n_components: int = 2,
     normalize_data: bool = False,
     **kwargs,
-) -> Tuple:
+) -> tuple:
     """
     UMAP of ensemble prediction.
 
@@ -96,8 +96,8 @@ def get_pca_representation_parameters(
     ens: Ensemble,
     n_components: int = 2,
     rescale_data: bool = True,
-    rescaler: Union[Callable, None] = None,
-) -> Tuple:
+    rescaler: Callable | None = None,
+) -> tuple:
     """
     PCA of parameter ensemble.
 
@@ -134,12 +134,12 @@ def get_pca_representation_parameters(
 
 
 def get_pca_representation_predictions(
-    ens: Union[Ensemble, EnsemblePrediction],
+    ens: Ensemble | EnsemblePrediction,
     prediction_index: int = 0,
     n_components: int = 2,
     rescale_data: bool = True,
-    rescaler: Union[Callable, None] = None,
-) -> Tuple:
+    rescaler: Callable | None = None,
+) -> tuple:
     """
     PCA of ensemble prediction.
 
@@ -188,7 +188,7 @@ def _get_umap_representation_lowlevel(
     n_components: int = 2,
     normalize_data: bool = False,
     **kwargs,
-) -> Tuple:
+) -> tuple:
     """
     Low level UMAP of parameter ensemble.
 
@@ -238,8 +238,8 @@ def _get_pca_representation_lowlevel(
     dataset: np.ndarray,
     n_components: int = 2,
     rescale_data: bool = True,
-    rescaler: Union[Callable, None] = None,
-) -> Tuple:
+    rescaler: Callable | None = None,
+) -> tuple:
     """
     Low level PCA of parameter ensemble.
 
