@@ -1,26 +1,42 @@
 """Helper methods for hierarchical optimization with PEtab."""
 
-from typing import Dict, Literal, Tuple
+import warnings
+from typing import Literal
 
 import pandas as pd
-import petab
+import petab.v1 as petab
 import sympy as sp
 from more_itertools import one
-from petab.C import LIN
-from petab.C import LOWER_BOUND as PETAB_LOWER_BOUND
-from petab.C import (
+from petab.v1.C import (
+    ESTIMATE,
+    LIN,
+    NOISE_PARAMETERS,
     OBSERVABLE_ID,
+    OBSERVABLE_PARAMETERS,
     OBSERVABLE_TRANSFORMATION,
     PARAMETER_SEPARATOR,
 )
-from petab.C import UPPER_BOUND as PETAB_UPPER_BOUND
-from petab.observables import get_formula_placeholders
+from petab.v1.C import LOWER_BOUND as PETAB_LOWER_BOUND
+from petab.v1.C import UPPER_BOUND as PETAB_UPPER_BOUND
+from petab.v1.observables import get_formula_placeholders
 
-from ..C import INNER_PARAMETER_BOUNDS
+from ..C import (
+    CENSORING_BOUNDS,
+    CENSORING_TYPES,
+    INNER_PARAMETER_BOUNDS,
+    INTERVAL_CENSORED,
+    LEFT_CENSORED,
+    MEASUREMENT_CATEGORY,
+    MEASUREMENT_TYPE,
+    ORDINAL,
+    PARAMETER_TYPE,
+    RELATIVE,
+    RIGHT_CENSORED,
+    SEMIQUANTITATIVE,
+    InnerParameterType,
+)
 from ..C import LOWER_BOUND as PYPESTO_LOWER_BOUND
-from ..C import PARAMETER_TYPE
 from ..C import UPPER_BOUND as PYPESTO_UPPER_BOUND
-from ..C import InnerParameterType
 
 
 def correct_parameter_df_bounds(parameter_df: pd.DataFrame) -> pd.DataFrame:
@@ -39,6 +55,11 @@ def correct_parameter_df_bounds(parameter_df: pd.DataFrame) -> pd.DataFrame:
     def correct_row(row: pd.Series) -> pd.Series:
         if pd.isna(row[PARAMETER_TYPE]):
             return row
+        if row[PARAMETER_TYPE] in [
+            InnerParameterType.SCALING,
+            InnerParameterType.OFFSET,
+        ]:
+            return row
         bounds = INNER_PARAMETER_BOUNDS[row[PARAMETER_TYPE]]
         row[PETAB_LOWER_BOUND] = bounds[PYPESTO_LOWER_BOUND]
         row[PETAB_UPPER_BOUND] = bounds[PYPESTO_UPPER_BOUND]
@@ -55,40 +76,63 @@ def validate_hierarchical_petab_problem(petab_problem: petab.Problem) -> None:
     petab_problem:
         The PEtab problem.
     """
-    if PARAMETER_TYPE not in petab_problem.parameter_df:
-        # not a hierarchical optimization problem
-        return
-
-    # ensure we only have linear parameter scale
-    inner_parameter_table = petab_problem.parameter_df[
-        petab_problem.parameter_df[PARAMETER_TYPE].isin(
-            [
-                InnerParameterType.OFFSET,
-                InnerParameterType.SIGMA,
-                InnerParameterType.SCALING,
-            ]
-        )
-    ]
-    if (
-        petab.PARAMETER_SCALE in inner_parameter_table
-        and not (
-            inner_parameter_table[petab.PARAMETER_SCALE].isna()
-            | (inner_parameter_table[petab.PARAMETER_SCALE] == petab.LIN)
-        ).all()
-    ):
-        sub_df = inner_parameter_table.loc[
-            :, [PARAMETER_TYPE, petab.PARAMETER_SCALE]
+    if PARAMETER_TYPE in petab_problem.parameter_df:
+        # ensure we only have linear parameter scale
+        inner_parameter_table = petab_problem.parameter_df[
+            petab_problem.parameter_df[PARAMETER_TYPE].isin(
+                [
+                    InnerParameterType.OFFSET,
+                    InnerParameterType.SIGMA,
+                    InnerParameterType.SCALING,
+                ]
+            )
         ]
-        raise NotImplementedError(
-            "Only parameterScale=lin supported for parameters of the inner "
-            f"subproblem.\n{sub_df}"
+        if (
+            petab.PARAMETER_SCALE in inner_parameter_table
+            and not (
+                inner_parameter_table[petab.PARAMETER_SCALE].isna()
+                | (inner_parameter_table[petab.PARAMETER_SCALE] == petab.LIN)
+                | (
+                    inner_parameter_table[PARAMETER_TYPE]
+                    != InnerParameterType.SIGMA
+                )
+            ).all()
+        ):
+            sub_df = inner_parameter_table.loc[
+                :, [PARAMETER_TYPE, petab.PARAMETER_SCALE]
+            ]
+            raise NotImplementedError(
+                "LOG and LOG10 parameter scale of inner parameters is not supported "
+                "for sigma parameters. Inner parameter table:\n"
+                f"{sub_df}"
+            )
+        elif (
+            petab.PARAMETER_SCALE in inner_parameter_table
+            and not (
+                inner_parameter_table[petab.PARAMETER_SCALE].isna()
+                | (inner_parameter_table[petab.PARAMETER_SCALE] == petab.LIN)
+            ).all()
+        ):
+            sub_df = inner_parameter_table.loc[
+                :, [PARAMETER_TYPE, petab.PARAMETER_SCALE]
+            ]
+            warnings.warn(
+                f"LOG and LOG10 parameter scale of inner parameters is used only "
+                f"for their visualization, and does not affect their optimization. "
+                f"Inner parameter table:\n{sub_df}",
+                stacklevel=1,
+            )
+
+        inner_parameter_df = validate_measurement_formulae(
+            petab_problem=petab_problem
         )
 
-    inner_parameter_df = validate_measurement_formulae(
-        petab_problem=petab_problem
-    )
+        validate_inner_parameter_pairings(
+            inner_parameter_df=inner_parameter_df
+        )
 
-    validate_inner_parameter_pairings(inner_parameter_df=inner_parameter_df)
+    if MEASUREMENT_TYPE in petab_problem.measurement_df:
+        validate_observable_data_types(petab_problem=petab_problem)
 
 
 def validate_inner_parameter_pairings(
@@ -173,7 +217,7 @@ def validate_inner_parameter_pairings(
 
 def get_inner_parameters(
     petab_problem: petab.Problem,
-) -> Dict[str, InnerParameterType]:
+) -> dict[str, InnerParameterType]:
     """Get information about the inner parameters.
 
     Parameters
@@ -262,8 +306,8 @@ def validate_measurement_formulae(
 def _validate_measurement_specific_observable_formula(
     measurement: pd.Series,
     petab_problem: petab.Problem,
-    inner_parameters: Dict[str, InnerParameterType],
-) -> Tuple[InnerParameterType, InnerParameterType]:
+    inner_parameters: dict[str, InnerParameterType],
+) -> tuple[InnerParameterType, InnerParameterType]:
     """Check whether a measurement observable formula is valid.
 
     Parameters
@@ -281,7 +325,7 @@ def _validate_measurement_specific_observable_formula(
     """
     formula, formula_inner_parameters = _get_symbolic_formula_from_measurement(
         measurement=measurement,
-        formula_type='observable',
+        formula_type="observable",
         petab_problem=petab_problem,
         inner_parameters=inner_parameters,
     )
@@ -347,8 +391,8 @@ def _validate_measurement_specific_observable_formula(
 def _validate_measurement_specific_noise_formula(
     measurement: pd.Series,
     petab_problem: petab.Problem,
-    inner_parameters: Dict[str, InnerParameterType],
-) -> Tuple[InnerParameterType, InnerParameterType]:
+    inner_parameters: dict[str, InnerParameterType],
+) -> tuple[InnerParameterType, InnerParameterType]:
     """Check whether a measurement noise formula is valid.
 
     Parameters
@@ -366,7 +410,7 @@ def _validate_measurement_specific_noise_formula(
     """
     formula, formula_inner_parameters = _get_symbolic_formula_from_measurement(
         measurement=measurement,
-        formula_type='noise',
+        formula_type="noise",
         petab_problem=petab_problem,
         inner_parameters=inner_parameters,
     )
@@ -397,10 +441,10 @@ def _validate_measurement_specific_noise_formula(
 
 def _get_symbolic_formula_from_measurement(
     measurement: pd.Series,
-    formula_type: Literal['observable', 'noise'],
+    formula_type: Literal["observable", "noise"],
     petab_problem: petab.Problem,
-    inner_parameters: Dict[str, InnerParameterType],
-) -> Tuple[sp.Expr, Dict[sp.Symbol, InnerParameterType]]:
+    inner_parameters: dict[str, InnerParameterType],
+) -> tuple[sp.Expr, dict[sp.Symbol, InnerParameterType]]:
     """Get a symbolic representation of a formula, with overrides overridden.
 
     Also performs some checks to ensure only valid numbers and types of inner
@@ -424,7 +468,7 @@ def _get_symbolic_formula_from_measurement(
     observable_id = measurement[OBSERVABLE_ID]
 
     formula_string = petab_problem.observable_df.loc[
-        observable_id, formula_type + 'Formula'
+        observable_id, formula_type + "Formula"
     ]
     symbolic_formula = sp.sympify(formula_string)
 
@@ -433,8 +477,20 @@ def _get_symbolic_formula_from_measurement(
         observable_id=observable_id,
         override_type=formula_type,
     )
+    # Search for placeholders in the formula that are not allowed to be
+    # overridden by inner parameters -- noise inner parameters should not
+    # be in observable formulas, and vice versa.
+    disallowed_formula_type = (
+        "noise" if formula_type == "observable" else "observable"
+    )
+    disallowed_formula_placeholders = get_formula_placeholders(
+        formula_string=formula_string,
+        observable_id=observable_id,
+        override_type=disallowed_formula_type,
+    )
+
     if formula_placeholders:
-        overrides = measurement[formula_type + 'Parameters']
+        overrides = measurement[formula_type + "Parameters"]
         overrides = (
             overrides.split(PARAMETER_SEPARATOR)
             if isinstance(overrides, str)
@@ -443,16 +499,30 @@ def _get_symbolic_formula_from_measurement(
         subs = dict(zip(formula_placeholders, overrides))
         symbolic_formula = symbolic_formula.subs(subs)
 
+    if disallowed_formula_placeholders:
+        disallowed_overrides = measurement[
+            disallowed_formula_type + "Parameters"
+        ]
+        disallowed_overrides = (
+            disallowed_overrides.split(PARAMETER_SEPARATOR)
+            if isinstance(disallowed_overrides, str)
+            else [disallowed_overrides]
+        )
+        disallowed_subs = dict(
+            zip(disallowed_formula_placeholders, disallowed_overrides)
+        )
+        symbolic_formula = symbolic_formula.subs(disallowed_subs)
+
     symbolic_formula_inner_parameters = {
         sp.Symbol(inner_parameter_id): inner_parameter_type
         for inner_parameter_id, inner_parameter_type in inner_parameters.items()
         if sp.Symbol(inner_parameter_id) in symbolic_formula.free_symbols
     }
 
-    if formula_type == 'noise':
+    if formula_type == "noise":
         max_parameters = 1
         expected_inner_parameter_types = [InnerParameterType.SIGMA]
-    elif formula_type == 'observable':
+    elif formula_type == "observable":
         max_parameters = 2
         expected_inner_parameter_types = [
             InnerParameterType.OFFSET,
@@ -498,3 +568,151 @@ def _get_symbolic_formula_from_measurement(
             )
 
     return symbolic_formula, symbolic_formula_inner_parameters
+
+
+def validate_observable_data_types(petab_problem: petab.Problem) -> None:
+    """Check whether the data types of observables are valid."""
+
+    supported_data_types = [
+        SEMIQUANTITATIVE,
+        RELATIVE,
+        ORDINAL,
+    ] + CENSORING_TYPES
+
+    # Get observables across data types
+    observables_by_data_type = {}
+    meas_df = petab_problem.measurement_df
+    if MEASUREMENT_TYPE in meas_df.columns:
+        petab_data_types = meas_df[meas_df[MEASUREMENT_TYPE].notna()][
+            MEASUREMENT_TYPE
+        ].unique()
+        for data_type in petab_data_types:
+            observables_by_data_type[data_type] = set(
+                meas_df.loc[
+                    meas_df[MEASUREMENT_TYPE] == data_type, OBSERVABLE_ID
+                ]
+            )
+        # Check whether all data types are supported
+        if not set(petab_data_types).issubset(supported_data_types):
+            raise ValueError(
+                f"Unsupported data type(s): `{set(petab_data_types) - set(supported_data_types)}`."
+            )
+
+    # For relative data, we search for observable parameters in the parameter table
+    # and their corresponding observables in the measurement table.
+    relative_observables = set()
+    if PARAMETER_TYPE in petab_problem.parameter_df.columns:
+        par_df = petab_problem.parameter_df[
+            petab_problem.parameter_df[PARAMETER_TYPE].notna()
+        ]
+        meas_df_w_obs_pars = meas_df[meas_df[OBSERVABLE_PARAMETERS].notna()]
+        for par_id, row in par_df.iterrows():
+            if not row[ESTIMATE]:
+                continue
+            if row[PARAMETER_TYPE] in [
+                InnerParameterType.SCALING,
+                InnerParameterType.OFFSET,
+            ]:
+                for _, row in meas_df_w_obs_pars.iterrows():
+                    if par_id in row[OBSERVABLE_PARAMETERS].split(
+                        PARAMETER_SEPARATOR
+                    ):
+                        relative_observables.add(row[OBSERVABLE_ID])
+
+            elif row[PARAMETER_TYPE] == InnerParameterType.SIGMA:
+                corresponding_obs = set(
+                    meas_df[meas_df[NOISE_PARAMETERS] == par_id][OBSERVABLE_ID]
+                )
+                # If a sigma parameter belongs to a semi-quantiative
+                # observable, it is not a relative inner parameter.
+                if SEMIQUANTITATIVE in observables_by_data_type and (
+                    corresponding_obs
+                    & observables_by_data_type[SEMIQUANTITATIVE]
+                ):
+                    continue
+                relative_observables.update(corresponding_obs)
+
+    observables_by_data_type[RELATIVE] = relative_observables
+
+    # Check whether there is any overlap across data types
+    for data_type, observables in observables_by_data_type.items():
+        for (
+            other_data_type,
+            other_observables,
+        ) in observables_by_data_type.items():
+            if data_type == other_data_type or (
+                data_type in CENSORING_TYPES
+                and other_data_type in CENSORING_TYPES
+            ):
+                continue
+            if observables & other_observables:
+                raise ValueError(
+                    "The same observable is associated with multiple data "
+                    f"types. Observable(s): `{observables & other_observables}`. "
+                    f"Data types: `{data_type}`, `{other_data_type}`."
+                )
+
+    # Validate ordinal measurement specification
+    if ORDINAL in observables_by_data_type:
+        meas_df_w_ordinals = meas_df[meas_df[MEASUREMENT_TYPE] == ORDINAL]
+        if MEASUREMENT_CATEGORY not in meas_df_w_ordinals.columns:
+            raise ValueError(
+                "Measurement category must be specified for ordinal "
+                "measurements."
+            )
+        for _, row in meas_df_w_ordinals.iterrows():
+            try:
+                category = float(row[MEASUREMENT_CATEGORY])
+                if category != int(category):
+                    raise ValueError
+            except ValueError as e:
+                raise ValueError(
+                    "Measurement category for ordinal measurement must be "
+                    "an integer."
+                ) from e
+
+    # Validate censored measurement specification
+    if set(CENSORING_TYPES) & set(observables_by_data_type.keys()):
+        meas_df_w_censored = meas_df[
+            meas_df[MEASUREMENT_TYPE].isin(CENSORING_TYPES)
+        ]
+        if CENSORING_BOUNDS not in meas_df_w_censored.columns:
+            raise ValueError(
+                "Censoring bounds must be specified for censored measurements."
+            )
+        for _, row in meas_df_w_censored.iterrows():
+            if (
+                row[MEASUREMENT_TYPE] == LEFT_CENSORED
+                or row[MEASUREMENT_TYPE] == RIGHT_CENSORED
+            ):
+                try:
+                    float(row[CENSORING_BOUNDS])
+                except ValueError as e:
+                    raise ValueError(
+                        f"Censoring bound(s) for a {row[MEASUREMENT_TYPE]}"
+                        " measurement must be of type float. Failure at "
+                        f"bound: `{row[CENSORING_BOUNDS]}`."
+                    ) from e
+            elif row[MEASUREMENT_TYPE] == INTERVAL_CENSORED:
+                bounds = row[CENSORING_BOUNDS].split(PARAMETER_SEPARATOR)
+                if len(bounds) != 2:
+                    raise ValueError(
+                        f"Censoring bounds for a {INTERVAL_CENSORED} measurement"
+                        f" must be two floats separated by the separator {PARAMETER_SEPARATOR}."
+                        f" Found {len(bounds)} bounds: `{bounds}`."
+                    )
+                try:
+                    float(bounds[0])
+                    float(bounds[1])
+                except ValueError as e:
+                    raise ValueError(
+                        f"Censoring bound(s) for a {row[MEASUREMENT_TYPE]}"
+                        " measurement must be of type float. Failure at "
+                        f"bounds: `{bounds}`."
+                    ) from e
+                if float(bounds[0]) > float(bounds[1]):
+                    raise ValueError(
+                        f"Censoring bounds for a {INTERVAL_CENSORED}"
+                        " measurement must be increasing. Failure at "
+                        f"bounds: `{bounds}`."
+                    )
