@@ -3,7 +3,8 @@
 import numbers
 import time
 from abc import ABC, abstractmethod
-from typing import Dict, Sequence, Tuple, Union
+from collections.abc import Sequence
+from typing import Union
 
 import numpy as np
 
@@ -38,7 +39,7 @@ class HistoryBase(ABC):
     # all possible history entries
     ALL_KEYS = (X, *RESULT_KEYS, TIME)
 
-    def __init__(self, options: HistoryOptions = None):
+    def __init__(self, options: Union[HistoryOptions, None] = None):
         if options is None:
             options = HistoryOptions()
         options = HistoryOptions.assert_instance(options)
@@ -48,7 +49,7 @@ class HistoryBase(ABC):
     def update(
         self,
         x: np.ndarray,
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: ModeType,
         result: ResultDict,
     ) -> None:
@@ -70,8 +71,8 @@ class HistoryBase(ABC):
 
     def finalize(
         self,
-        message: str = None,
-        exitflag: str = None,
+        message: Union[str, None] = None,
+        exitflag: Union[str, None] = None,
     ) -> None:
         """
         Finalize history. Called after a run. Default: Do nothing.
@@ -118,6 +119,16 @@ class HistoryBase(ABC):
     def start_time(self) -> float:
         """Return start time."""
 
+    @property
+    @abstractmethod
+    def message(self) -> str:
+        """Return message."""
+
+    @property
+    @abstractmethod
+    def exitflag(self) -> str:
+        """Return exitflag."""
+
     @abstractmethod
     def get_x_trace(
         self,
@@ -239,7 +250,7 @@ class HistoryBase(ABC):
         trim: bool = False,
     ) -> Union[Sequence[float], float]:
         """
-        Cumulative execution times.
+        Cumulative execution times [s].
 
         Takes as parameter an index or indices and returns corresponding trace
         values. If only a single value is requested, the list is flattened.
@@ -271,7 +282,7 @@ class NoHistory(HistoryBase):
     def update(  # noqa: D102
         self,
         x: np.ndarray,
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: ModeType,
         result: ResultDict,
     ) -> None:
@@ -304,6 +315,14 @@ class NoHistory(HistoryBase):
     def start_time(self) -> float:  # noqa: D102
         raise NotImplementedError()
 
+    @property
+    def message(self) -> float:  # noqa: D102
+        raise NotImplementedError()
+
+    @property
+    def exitflag(self) -> float:  # noqa: D102
+        raise NotImplementedError()
+
     def get_x_trace(  # noqa: D102
         self, ix: Union[int, Sequence[int], None] = None, trim: bool = False
     ) -> Union[Sequence[np.ndarray], np.ndarray]:
@@ -346,7 +365,7 @@ class CountHistoryBase(HistoryBase):
     Needs a separate implementation of trace.
     """
 
-    def __init__(self, options: Union[HistoryOptions, Dict] = None):
+    def __init__(self, options: Union[HistoryOptions, dict] = None):
         super().__init__(options)
         self._n_fval: int = 0
         self._n_grad: int = 0
@@ -354,11 +373,13 @@ class CountHistoryBase(HistoryBase):
         self._n_res: int = 0
         self._n_sres: int = 0
         self._start_time: float = time.time()
+        self._exitflag = ""
+        self._message = ""
 
     def update(  # noqa: D102
         self,
         x: np.ndarray,
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: ModeType,
         result: ResultDict,
     ) -> None:
@@ -366,7 +387,7 @@ class CountHistoryBase(HistoryBase):
 
     def _update_counts(
         self,
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: ModeType,
     ):
         """Update the counters."""
@@ -407,6 +428,22 @@ class CountHistoryBase(HistoryBase):
     def start_time(self) -> float:  # noqa: D102
         return self._start_time
 
+    @property
+    def message(self) -> str:  # noqa: D102
+        return self._message
+
+    @property
+    def exitflag(self) -> str:  # noqa: D102
+        return self._exitflag
+
+    def finalize(  # noqa: D102
+        self,
+        message: str = None,
+        exitflag: str = None,
+    ) -> None:  # noqa: D102
+        self._message = message
+        self._exitflag = exitflag
+
 
 class CountHistory(CountHistoryBase):
     """History that can only count, other functions cannot be invoked."""
@@ -463,8 +500,7 @@ def add_fun_from_res(result: ResultDict) -> ResultDict:
 
     Returns
     -------
-    full_result:
-        Result dicionary, adding whatever is possible to calculate.
+    Result dictionary, adding whatever is possible to calculate.
     """
     result = result.copy()
 
@@ -493,15 +529,14 @@ def reduce_result_via_options(
 
     Returns
     -------
-    result:
-        Result reduced to what is intended to be stored in history.
+    Result reduced to what is intended to be stored in history.
     """
     result = result.copy()
 
     # apply options to result
     for key in HistoryBase.RESULT_KEYS:
         if result.get(key) is None or not options.get(
-            f'trace_record_{key}', True
+            f"trace_record_{key}", True
         ):
             result[key] = np.nan
 
