@@ -8,8 +8,10 @@ from .amici import AmiciObjective
 from .base import ObjectiveBase
 from .finite_difference import FD, FDDelta
 from .function import Objective
+from .petab import PetabSimulatorObjective
 from .priors import (
     NegLogParameterPriors,
     NegLogPriors,
     get_parameter_prior_dict,
 )
+from .roadrunner import RoadRunnerObjective
