@@ -1,9 +1,21 @@
+import inspect
+import warnings
+from collections.abc import Sequence
 from copy import deepcopy
-from typing import Sequence, Tuple
+from typing import Any
 
 import numpy as np
 
-from ..C import FVAL, GRAD, HESS, HESSP, RDATAS, RES, SRES, ModeType
+from ..C import (
+    FVAL,
+    GRAD,
+    HESS,
+    HESSP,
+    RDATAS,
+    RES,
+    SRES,
+    ModeType,
+)
 from .base import ObjectiveBase, ResultDict
 
 
@@ -30,19 +42,19 @@ class AggregatedObjective(ObjectiveBase):
         # input typechecks
         if not isinstance(objectives, Sequence):
             raise TypeError(
-                f'Objectives must be a Sequence, ' f'was {type(objectives)}.'
+                f"Objectives must be a Sequence, was {type(objectives)}."
             )
 
         if not all(
             isinstance(objective, ObjectiveBase) for objective in objectives
         ):
             raise TypeError(
-                'Objectives must only contain elements of type'
-                'pypesto.Objective'
+                "Objectives must only contain elements of type"
+                "pypesto.Objective"
             )
 
         if not objectives:
-            raise ValueError('Length of objectives must be at least one')
+            raise ValueError("Length of objectives must be at least one")
 
         self._objectives = objectives
 
@@ -54,7 +66,7 @@ class AggregatedObjective(ObjectiveBase):
             objectives=[deepcopy(objective) for objective in self._objectives],
             x_names=deepcopy(self.x_names),
         )
-        for key in set(self.__dict__.keys()) - {'_objectives', 'x_names'}:
+        for key in set(self.__dict__.keys()) - {"_objectives", "x_names"}:
             other.__dict__[key] = deepcopy(self.__dict__[key])
 
         return other
@@ -67,7 +79,7 @@ class AggregatedObjective(ObjectiveBase):
 
     def check_sensi_orders(
         self,
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: ModeType,
     ) -> bool:
         """See `ObjectiveBase` documentation."""
@@ -79,8 +91,10 @@ class AggregatedObjective(ObjectiveBase):
     def call_unprocessed(
         self,
         x: np.ndarray,
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: ModeType,
+        kwargs_list: Sequence[dict[str, Any]] = None,
+        return_dict: bool = False,
         **kwargs,
     ) -> ResultDict:
         """
@@ -88,11 +102,44 @@ class AggregatedObjective(ObjectiveBase):
 
         Main method to overwrite from the base class. It handles and
         delegates the actual objective evaluation.
+
+        Parameters
+        ----------
+        kwargs_list:
+            Objective-specific keyword arguments, where the dictionaries are
+            ordered by the objectives.
         """
+        if kwargs_list is None:
+            kwargs_list = [{}] * len(self._objectives)
+        elif len(kwargs_list) != len(self._objectives):
+            raise ValueError(
+                "The length of `kwargs_list` must match the number of "
+                "objectives you are aggregating."
+            )
+        for objective_, objective_kwargs in zip(self._objectives, kwargs_list):
+            if (
+                "return_dict"
+                in inspect.signature(objective_.call_unprocessed).parameters
+            ):
+                objective_kwargs["return_dict"] = return_dict
+            else:
+                warnings.warn(
+                    "Please add `return_dict` to the argument list of your "
+                    "objective's `call_unprocessed` method. "
+                    f"Current objective: `{type(objective_)}`.",
+                    DeprecationWarning,
+                    stacklevel=1,
+                )
         return aggregate_results(
             [
-                objective.call_unprocessed(x, sensi_orders, mode, **kwargs)
-                for objective in self._objectives
+                objective.call_unprocessed(
+                    x,
+                    sensi_orders,
+                    mode,
+                    **kwargs,
+                    **cur_kwargs,
+                )
+                for objective, cur_kwargs in zip(self._objectives, kwargs_list)
             ]
         )
 
@@ -105,7 +152,7 @@ class AggregatedObjective(ObjectiveBase):
         """Return basic information of the objective configuration."""
         info = super().get_config()
         for n_obj, obj in enumerate(self._objectives):
-            info[f'objective_{n_obj}'] = obj.get_config()
+            info[f"objective_{n_obj}"] = obj.get_config()
         return info
 
 
