@@ -1,16 +1,19 @@
 import copy
+import inspect
 import logging
+import warnings
 from abc import ABC, abstractmethod
-from typing import Dict, Iterable, List, Optional, Sequence, Tuple, Union
+from collections.abc import Iterable, Sequence
+from typing import Optional, Union
 
 import numpy as np
 import pandas as pd
 
 from ..C import FVAL, GRAD, HESS, MODE_FUN, MODE_RES, RES, SRES, ModeType
-from ..history import NoHistory
+from ..history import NoHistory, create_history
 from .pre_post_process import FixedParametersProcessor, PrePostProcessor
 
-ResultDict = Dict[str, Union[float, np.ndarray, Dict]]
+ResultDict = dict[str, Union[float, np.ndarray, dict]]
 
 logger = logging.getLogger(__name__)
 
@@ -55,13 +58,6 @@ class ObjectiveBase(ABC):
         self.pre_post_processor = PrePostProcessor()
         self.history = NoHistory()
 
-    def __deepcopy__(self, memodict=None) -> 'ObjectiveBase':
-        """Create deepcopy of objective object."""
-        other = type(self)()  # maintain type for derived classes
-        for attr, val in self.__dict__.items():
-            other.__dict__[attr] = copy.deepcopy(val)
-        return other
-
     # The following has_ properties can be used to find out what values
     # the objective supports.
     @property
@@ -96,7 +92,7 @@ class ObjectiveBase(ABC):
         return self.check_sensi_orders((1,), MODE_RES)
 
     @property
-    def x_names(self) -> Union[List[str], None]:
+    def x_names(self) -> Union[list[str], None]:
         """Parameter names."""
         if self._x_names is None:
             return self._x_names
@@ -119,14 +115,18 @@ class ObjectiveBase(ABC):
         By default does nothing.
         """
 
+    def create_history(self, id, x_names, options):
+        """See `history.generate.create_history` documentation."""
+        return create_history(id, x_names, options)
+
     def __call__(
         self,
         x: np.ndarray,
-        sensi_orders: Tuple[int, ...] = (0,),
+        sensi_orders: tuple[int, ...] = (0,),
         mode: ModeType = MODE_FUN,
         return_dict: bool = False,
         **kwargs,
-    ) -> Union[float, np.ndarray, Tuple, ResultDict]:
+    ) -> Union[float, np.ndarray, tuple, ResultDict]:
         """
         Obtain arbitrary sensitivities.
 
@@ -168,7 +168,7 @@ class ObjectiveBase(ABC):
         # check input
         if not self.check_mode(mode):
             raise ValueError(
-                f"This Objective cannot be called with mode" f"={mode}."
+                f"This Objective cannot be called with mode={mode}."
             )
         if not self.check_sensi_orders(sensi_orders, mode):
             raise ValueError(
@@ -180,6 +180,19 @@ class ObjectiveBase(ABC):
         x_full = self.pre_post_processor.preprocess(x=x)
 
         # compute result
+        if (
+            "return_dict"
+            in inspect.signature(self.call_unprocessed).parameters
+        ):
+            kwargs["return_dict"] = return_dict
+        else:
+            warnings.warn(
+                "Please add `return_dict` to the argument list of your "
+                "objective's `call_unprocessed` method.",
+                DeprecationWarning,
+                stacklevel=1,
+            )
+
         result = self.call_unprocessed(
             x=x_full, sensi_orders=sensi_orders, mode=mode, **kwargs
         )
@@ -204,8 +217,9 @@ class ObjectiveBase(ABC):
     def call_unprocessed(
         self,
         x: np.ndarray,
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: ModeType,
+        return_dict: bool,
         **kwargs,
     ) -> ResultDict:
         """
@@ -219,6 +233,10 @@ class ObjectiveBase(ABC):
             Specifies which sensitivities to compute, e.g. (0,1) -> fval, grad.
         mode:
             Whether to compute function values or residuals.
+        return_dict:
+            Whether the user requested additional information. Objectives can
+            use this to determine whether to e.g. return "full" or "minimal"
+            information.
 
         Returns
         -------
@@ -256,12 +274,12 @@ class ObjectiveBase(ABC):
 
         Return it as a dictionary.
         """
-        info = {'type': self.__class__.__name__}
+        info = {"type": self.__class__.__name__}
         return info
 
     def check_sensi_orders(
         self,
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: ModeType,
     ) -> bool:
         """
@@ -313,10 +331,10 @@ class ObjectiveBase(ABC):
 
     @staticmethod
     def output_to_tuple(
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: ModeType,
         **kwargs: Union[float, np.ndarray],
-    ) -> Tuple:
+    ) -> tuple:
         """
         Return values as requested by the caller.
 
@@ -414,7 +432,7 @@ class ObjectiveBase(ABC):
         self,
         *args,
         multi_eps: Optional[Iterable] = None,
-        label: str = 'rel_err',
+        label: str = "rel_err",
         **kwargs,
     ):
         """
@@ -435,10 +453,10 @@ class ObjectiveBase(ABC):
             Valid options are the column labels of the dataframe returned by
             the `ObjectiveBase.check_grad` method.
         """
-        if 'eps' in kwargs:
+        if "eps" in kwargs:
             raise KeyError(
-                'Please use the `multi_eps` (not the `eps`) argument with '
-                '`check_grad_multi_eps` to specify step sizes.'
+                "Please use the `multi_eps` (not the `eps`) argument with "
+                "`check_grad_multi_eps` to specify step sizes."
             )
 
         if multi_eps is None:
@@ -452,7 +470,7 @@ class ObjectiveBase(ABC):
         # the step size (`eps`) that produced the smallest error (`label`).
         combined_result = None
         for eps, result in results.items():
-            result['eps'] = eps
+            result["eps"] = eps
             if combined_result is None:
                 combined_result = result
                 continue
@@ -570,14 +588,14 @@ class ObjectiveBase(ABC):
             # log for dimension ix
             if verbosity > 1:
                 logger.info(
-                    f'index:    {ix}\n'
-                    f'grad:     {grad_ix}\n'
-                    f'fd_f:     {fd_f_ix}\n'
-                    f'fd_b:     {fd_b_ix}\n'
-                    f'fd_c:     {fd_c_ix}\n'
-                    f'fd_err:   {fd_err_ix}\n'
-                    f'abs_err:  {abs_err_ix}\n'
-                    f'rel_err:  {rel_err_ix}\n'
+                    f"index:    {ix}\n"
+                    f"grad:     {grad_ix}\n"
+                    f"fd_f:     {fd_f_ix}\n"
+                    f"fd_b:     {fd_b_ix}\n"
+                    f"fd_c:     {fd_c_ix}\n"
+                    f"fd_err:   {fd_err_ix}\n"
+                    f"abs_err:  {abs_err_ix}\n"
+                    f"rel_err:  {rel_err_ix}\n"
                 )
 
             # append to lists
@@ -596,24 +614,24 @@ class ObjectiveBase(ABC):
 
         # create data dictionary for dataframe
         data = {
-            'grad': grad_list,
-            'fd_f': fd_f_list,
-            'fd_b': fd_b_list,
-            'fd_c': fd_c_list,
-            'fd_err': fd_err_list,
-            'abs_err': abs_err_list,
-            'rel_err': rel_err_list,
+            "grad": grad_list,
+            "fd_f": fd_f_list,
+            "fd_b": fd_b_list,
+            "fd_c": fd_c_list,
+            "fd_err": fd_err_list,
+            "abs_err": abs_err_list,
+            "rel_err": rel_err_list,
         }
 
         # update data dictionary if detailed output is requested
         if detailed:
             prefix_data = {
-                'fval': [fval] * len(x_indices),
-                'fval_p': fval_p_list,
-                'fval_m': fval_m_list,
+                "fval": [fval] * len(x_indices),
+                "fval_p": fval_p_list,
+                "fval_m": fval_m_list,
             }
-            std_str = '(grad-fd_c)/std({fd_f,fd_b,fd_c})'
-            mean_str = '|grad-fd_c|/mean(|fd_f-fd_b|,|fd_f-fd_c|,|fd_b-fd_c|)'
+            std_str = "(grad-fd_c)/std({fd_f,fd_b,fd_c})"
+            mean_str = "|grad-fd_c|/mean(|fd_f-fd_b|,|fd_f-fd_c|,|fd_b-fd_c|)"
             postfix_data = {
                 std_str: std_check_list,
                 mean_str: mean_check_list,
@@ -624,7 +642,7 @@ class ObjectiveBase(ABC):
         result = pd.DataFrame(
             data=data,
             index=[
-                self.x_names[ix] if self.x_names is not None else f'x_{ix}'
+                self.x_names[ix] if self.x_names is not None else f"x_{ix}"
                 for ix in x_indices
             ],
         )
@@ -697,15 +715,13 @@ class ObjectiveBase(ABC):
                 return False
 
         return all(
-            [
-                any(
-                    [
-                        np.all(
-                            (mode_df.rel_err.values < rtol)
-                            | (mode_df.abs_err.values < atol)
-                        ),
-                    ]
-                )
-                for mode_df in dfs
-            ]
+            any(
+                [
+                    np.all(
+                        (mode_df.rel_err.values < rtol)
+                        | (mode_df.abs_err.values < atol)
+                    ),
+                ]
+            )
+            for mode_df in dfs
         )
