@@ -1,6 +1,9 @@
-from copy import deepcopy
-from typing import Callable, Dict, List, Sequence, Tuple, Union
+import logging
+import math
+from collections.abc import Sequence
+from typing import Callable, Union
 
+import cloudpickle
 import numpy as np
 
 from .. import C
@@ -8,6 +11,8 @@ from .aggregated import AggregatedObjective
 from .base import ResultDict
 from .function import ObjectiveBase
 
+logger = logging.getLogger(__name__)
+
 
 class NegLogPriors(AggregatedObjective):
     """
@@ -46,7 +51,7 @@ class NegLogParameterPriors(ObjectiveBase):
 
     def __init__(
         self,
-        prior_list: List[Dict],
+        prior_list: list[dict],
         x_names: Sequence[str] = None,
     ):
         """
@@ -63,16 +68,20 @@ class NegLogParameterPriors(ObjectiveBase):
         self.prior_list = prior_list
         super().__init__(x_names)
 
-    def __deepcopy__(self, memodict=None):
-        """Create deepcopy of object."""
-        other = NegLogParameterPriors(deepcopy(self.prior_list))
-        return other
+    def __getstate__(self):
+        """Get state using cloudpickle."""
+        return cloudpickle.dumps(self.__dict__)
+
+    def __setstate__(self, state):
+        """Set state using cloudpickle."""
+        self.__dict__.update(cloudpickle.loads(state))
 
     def call_unprocessed(
         self,
         x: np.ndarray,
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: C.ModeType,
+        return_dict: bool,
         **kwargs,
     ) -> ResultDict:
         """
@@ -96,7 +105,7 @@ class NegLogParameterPriors(ObjectiveBase):
                 elif order == 2:
                     res[C.HESS] = self.hessian_neg_log_density(x)
                 else:
-                    raise ValueError(f'Invalid sensi order {order}.')
+                    raise ValueError(f"Invalid sensi order {order}.")
 
         if mode == C.MODE_RES:
             for order in sensi_orders:
@@ -105,13 +114,13 @@ class NegLogParameterPriors(ObjectiveBase):
                 elif order == 1:
                     res[C.SRES] = self.residual_jacobian(x)
                 else:
-                    raise ValueError(f'Invalid sensi order {order}.')
+                    raise ValueError(f"Invalid sensi order {order}.")
 
         return res
 
     def check_sensi_orders(
         self,
-        sensi_orders: Tuple[int, ...],
+        sensi_orders: tuple[int, ...],
         mode: C.ModeType,
     ) -> bool:
         """See `ObjectiveBase` documentation."""
@@ -123,20 +132,20 @@ class NegLogParameterPriors(ObjectiveBase):
             for order in sensi_orders:
                 if order == 0:
                     return all(
-                        prior.get('residual', None) is not None
+                        prior.get("residual", None) is not None
                         for prior in self.prior_list
                     )
                 elif order == 1:
                     return all(
-                        prior.get('residual_dx', None) is not None
+                        prior.get("residual_dx", None) is not None
                         for prior in self.prior_list
                     )
                 else:
                     return False
         else:
             raise ValueError(
-                f'Invalid input: Expected mode {C.MODE_FUN} or '
-                f'{C.MODE_RES}, received {mode} instead.'
+                f"Invalid input: Expected mode {C.MODE_FUN} or "
+                f"{C.MODE_RES}, received {mode} instead."
             )
 
         return True
@@ -147,20 +156,20 @@ class NegLogParameterPriors(ObjectiveBase):
             return True
         elif mode == C.MODE_RES:
             return all(
-                prior.get('residual', None) is not None
+                prior.get("residual", None) is not None
                 for prior in self.prior_list
             )
         else:
             raise ValueError(
-                f'Invalid input: Expected mode {C.MODE_FUN} or '
-                f'{C.MODE_RES}, received {mode} instead.'
+                f"Invalid input: Expected mode {C.MODE_FUN} or "
+                f"{C.MODE_RES}, received {mode} instead."
             )
 
     def neg_log_density(self, x):
         """Evaluate the negative log-density at x."""
         density_val = 0
         for prior in self.prior_list:
-            density_val -= prior['density_fun'](x[prior['index']])
+            density_val -= prior["density_fun"](x[prior["index"]])
 
         return density_val
 
@@ -169,7 +178,7 @@ class NegLogParameterPriors(ObjectiveBase):
         grad = np.zeros_like(x)
 
         for prior in self.prior_list:
-            grad[prior['index']] -= prior['density_dx'](x[prior['index']])
+            grad[prior["index"]] -= prior["density_dx"](x[prior["index"]])
 
         return grad
 
@@ -178,8 +187,8 @@ class NegLogParameterPriors(ObjectiveBase):
         hessian = np.zeros((len(x), len(x)))
 
         for prior in self.prior_list:
-            hessian[prior['index'], prior['index']] -= prior['density_ddx'](
-                x[prior['index']]
+            hessian[prior["index"], prior["index"]] -= prior["density_ddx"](
+                x[prior["index"]]
             )
 
         return hessian
@@ -189,8 +198,8 @@ class NegLogParameterPriors(ObjectiveBase):
         h_dot_p = np.zeros_like(p)
 
         for prior in self.prior_list:
-            h_dot_p[prior['index']] -= (
-                prior['density_ddx'](x[prior['index']]) * p[prior['index']]
+            h_dot_p[prior["index"]] -= (
+                prior["density_ddx"](x[prior["index"]]) * p[prior["index"]]
             )
 
         return h_dot_p
@@ -198,7 +207,7 @@ class NegLogParameterPriors(ObjectiveBase):
     def residual(self, x):
         """Evaluate the residual representation of the prior at x."""
         return np.asarray(
-            [prior['residual'](x[prior['index']]) for prior in self.prior_list]
+            [prior["residual"](x[prior["index"]]) for prior in self.prior_list]
         )
 
     def residual_jacobian(self, x):
@@ -210,8 +219,8 @@ class NegLogParameterPriors(ObjectiveBase):
         """
         sres = np.zeros((len(self.prior_list), len(x)))
         for iprior, prior in enumerate(self.prior_list):
-            sres[iprior, prior['index']] = prior['residual_dx'](
-                x[prior['index']]
+            sres[iprior, prior["index"]] = prior["residual_dx"](
+                x[prior["index"]]
             )
 
         return sres
@@ -235,24 +244,26 @@ def get_parameter_prior_dict(
         "parameterScaleUniform", "parameterScaleNormal",
         "parameterScaleLaplace"}
     prior_parameters:
-        Parameters of the priors. Parameters are defined in linear scale.
+        Parameters of the priors. Parameters are defined in the parameter
+        scale if the `prior_type` starts with ``parameterScale``, otherwise in
+        the linear scale.
     parameter_scale:
         scale in which the parameter is defined (since a parameter can be
         log-transformed, while the prior is always defined in the linear
-        space, unless it starts with "parameterScale")
+        space, unless it starts with ``parameterScale``).
     """
     log_f, d_log_f_dx, dd_log_f_ddx, res, d_res_dx = _prior_densities(
         prior_type, prior_parameters
     )
 
-    if parameter_scale == C.LIN or prior_type.startswith('parameterScale'):
+    if parameter_scale == C.LIN or prior_type.startswith("parameterScale"):
         return {
-            'index': index,
-            'density_fun': log_f,
-            'density_dx': d_log_f_dx,
-            'density_ddx': dd_log_f_ddx,
-            'residual': res,
-            'residual_dx': d_res_dx,
+            "index": index,
+            "density_fun": log_f,
+            "density_dx": d_log_f_dx,
+            "density_ddx": dd_log_f_ddx,
+            "residual": res,
+            "residual_dx": d_res_dx,
         }
 
     elif parameter_scale == C.LOG:
@@ -272,27 +283,31 @@ def get_parameter_prior_dict(
                 + np.exp(x_log) * dd_log_f_ddx(np.exp(x_log))
             )
 
-        res_log = None
         if res is not None:
 
             def res_log(x_log):
                 """Residual-prior for log-parameters."""
                 return res(np.exp(x_log))
 
-        d_res_log = None
+        else:
+            res_log = None
+
         if d_res_dx is not None:
 
             def d_res_log(x_log):
                 """Residual-prior for log-parameters."""
                 return d_res_dx(np.exp(x_log)) * np.exp(x_log)
 
+        else:
+            d_res_log = None
+
         return {
-            'index': index,
-            'density_fun': log_f_log,
-            'density_dx': d_log_f_log,
-            'density_ddx': dd_log_f_log,
-            'residual': res_log,
-            'residual_dx': d_res_log,
+            "index": index,
+            "density_fun": log_f_log,
+            "density_dx": d_log_f_log,
+            "density_ddx": dd_log_f_log,
+            "residual": res_log,
+            "residual_dx": d_res_log,
         }
 
     elif parameter_scale == C.LOG10:
@@ -332,12 +347,12 @@ def get_parameter_prior_dict(
                 return d_res_dx(10**x_log10) * log10 * 10**x_log10
 
         return {
-            'index': index,
-            'density_fun': log_f_log10,
-            'density_dx': d_log_f_log10,
-            'density_ddx': dd_log_f_log10,
-            'residual': res_log,
-            'residual_dx': d_res_log,
+            "index": index,
+            "density_fun": log_f_log10,
+            "density_dx": d_log_f_log10,
+            "density_ddx": dd_log_f_log10,
+            "residual": res_log,
+            "residual_dx": d_res_log,
         }
 
     else:
@@ -486,6 +501,11 @@ def _prior_densities(
             return np.sqrt(abs(x - mean) / scale)
 
         def d_res_dx(x):
+            if x == mean:
+                logger.warning(
+                    "x == mean in d_res_dx of Laplace prior. Returning NaN."
+                )
+                return math.nan
             return 1 / 2 * (x - mean) / np.sqrt(scale * abs(x - mean) ** 3)
 
         return log_f, d_log_f_dx, dd_log_f_ddx, res, d_res_dx
@@ -508,9 +528,7 @@ def _prior_densities(
             return -1 / x - (np.log(x) - mean) / (sigma**2 * x)
 
         def dd_log_f_ddx(x):
-            return 1 / (x**2) - (1 - np.log(x) + mean) / (
-                sigma**2 * x**2
-            )
+            return 1 / (x**2) - (1 - np.log(x) + mean) / (sigma**2 * x**2)
 
         return log_f, d_log_f_dx, dd_log_f_ddx, None, None
 
@@ -519,7 +537,7 @@ def _prior_densities(
         raise NotImplementedError
     else:
         raise ValueError(
-            f'NegLogPriors of type {prior_type} are currently ' 'not supported'
+            f"NegLogPriors of type {prior_type} are currently not supported"
         )
 
 
