@@ -6,7 +6,13 @@ Optimize
 Multistart optimization with support for various optimizers.
 """
 
-from .ess import CESSOptimizer, ESSOptimizer, SacessOptimizer
+from .ess import (
+    ESSOptimizer,
+    SacessFidesFactory,
+    SacessOptimizer,
+    SacessOptions,
+    get_default_ess_options,
+)
 from .load import (
     fill_result_from_history,
     optimization_result_from_history,
@@ -15,7 +21,7 @@ from .load import (
 )
 from .optimize import minimize
 from .optimizer import (
-    CmaesOptimizer,
+    CmaOptimizer,
     DlibOptimizer,
     FidesOptimizer,
     IpoptOptimizer,
