@@ -5,23 +5,20 @@ import logging
 import re
 import time
 import warnings
-from typing import TYPE_CHECKING, Dict, Optional
+from functools import wraps
+from typing import TYPE_CHECKING
 
 import numpy as np
 import scipy.optimize
 
-from ..C import FVAL, GRAD, MODE_FUN, MODE_RES
-from ..history import (
-    HistoryOptions,
-    NoHistory,
-    OptimizerHistory,
-    create_history,
-)
+from ..C import FVAL, GRAD, INNER_PARAMETERS, MODE_FUN, MODE_RES, SPLINE_KNOTS
+from ..history import HistoryOptions, NoHistory, OptimizerHistory
 from ..objective import Objective
-from ..problem import Problem
+from ..problem import HierarchicalProblem, Problem
 from ..result import OptimizerResult
 from .load import fill_result_from_history
 from .options import OptimizeOptions
+from .util import check_finite_bounds
 
 if TYPE_CHECKING:
     import fides
@@ -39,12 +36,55 @@ class OptimizerImportError(ImportError):
         )
 
 
+def hierarchical_decorator(minimize):
+    """Add inner parameters to the optimizer result.
+
+    Default decorator for the :meth:`Optimizer.minimize` method.
+    """
+
+    @wraps(minimize)
+    def wrapped_minimize(
+        self,
+        problem: Problem,
+        x0: np.ndarray,
+        id: str,
+        history_options: HistoryOptions = None,
+        optimize_options: OptimizeOptions = None,
+    ):
+        # perform the actual optimization
+        result = minimize(
+            self,
+            problem=problem,
+            x0=x0,
+            id=id,
+            history_options=history_options,
+            optimize_options=optimize_options,
+        )
+
+        if isinstance(problem, HierarchicalProblem) and result.x is not None:
+            # Call the objective to obtain inner parameters of
+            # the optimal outer optimization parameters
+            return_dict = problem.objective(
+                result.x,
+                return_dict=True,
+            )
+            if INNER_PARAMETERS in return_dict:
+                result[INNER_PARAMETERS] = return_dict[INNER_PARAMETERS]
+            if SPLINE_KNOTS in return_dict:
+                result[SPLINE_KNOTS] = return_dict[SPLINE_KNOTS]
+
+        return result
+
+    return wrapped_minimize
+
+
 def history_decorator(minimize):
     """Initialize and extract information stored in the history.
 
-    Default decorator for the minimize() method.
+    Default decorator for the :meth:`Optimizer.minimize` method.
     """
 
+    @wraps(minimize)
     def wrapped_minimize(
         self,
         problem: Problem,
@@ -62,7 +102,7 @@ def history_decorator(minimize):
         objective.initialize()
 
         # initialize the history
-        history = create_history(
+        history = objective.create_history(
             id=id,
             x_names=[problem.x_names[ix] for ix in problem.x_free_indices],
             options=history_options,
@@ -92,10 +132,19 @@ def history_decorator(minimize):
                 message=result.message, exitflag=result.exitflag
             )
         except Exception as err:
-            if optimize_options.allow_failed_starts:
-                logger.error(f'start {id} failed: {err}')
+            if optimize_options and optimize_options.allow_failed_starts:
+                import sys
+                import traceback
+
+                trace = "\n".join(traceback.format_exception(*sys.exc_info()))
+
+                logger.error(f"start {id} failed:\n{trace}")
                 result = OptimizerResult(
-                    x0=x0, exitflag=-1, message=str(err), id=id
+                    x0=x0,
+                    exitflag=-1,
+                    message=str(err),
+                    id=id,
+                    optimizer=str(self),
                 )
             else:
                 raise
@@ -118,11 +167,12 @@ def history_decorator(minimize):
 def time_decorator(minimize):
     """Measure time of optimization.
 
-    Default decorator for the minimize() method to take time.
+    Default decorator for the :meth:`Optimizer.minimize` method to take time.
     Currently, the method time.time() is used, which measures
     the wall-clock time.
     """
 
+    @wraps(minimize)
     def wrapped_minimize(
         self,
         problem: Problem,
@@ -150,10 +200,11 @@ def time_decorator(minimize):
 def fix_decorator(minimize):
     """Include also fixed parameters in the result arrays of minimize().
 
-    Default decorator for the minimize() method (nans will be inserted in the
-    derivatives).
+    Default decorator for the :meth:`Optimizer.minimize` method (nans will be
+    inserted in the derivatives).
     """
 
+    @wraps(minimize)
     def wrapped_minimize(
         self,
         problem: Problem,
@@ -185,6 +236,34 @@ def fix_decorator(minimize):
     return wrapped_minimize
 
 
+def minimize_decorator_collection(minimize):
+    """Collect all decorators for the minimize() method."""
+
+    @wraps(minimize)
+    @fix_decorator
+    @time_decorator
+    @hierarchical_decorator
+    @history_decorator
+    def wrapped_minimize(
+        self,
+        problem: Problem,
+        x0: np.ndarray,
+        id: str,
+        history_options: HistoryOptions = None,
+        optimize_options: OptimizeOptions = None,
+    ):
+        return minimize(
+            self,
+            problem=problem,
+            x0=x0,
+            id=id,
+            history_options=history_options,
+            optimize_options=optimize_options,
+        )
+
+    return wrapped_minimize
+
+
 class Optimizer(abc.ABC):
     """
     Optimizer base class, not functional on its own.
@@ -197,9 +276,7 @@ class Optimizer(abc.ABC):
         """Initialize base class."""
 
     @abc.abstractmethod
-    @fix_decorator
-    @time_decorator
-    @history_decorator
+    @minimize_decorator_collection
     def minimize(
         self,
         problem: Problem,
@@ -234,30 +311,27 @@ class Optimizer(abc.ABC):
         """Create default options specific for the optimizer."""
         return None
 
-
-def check_finite_bounds(lb, ub):
-    """Raise if bounds are not finite."""
-    if not np.isfinite(lb).all() or not np.isfinite(ub).all():
-        raise ValueError(
-            'Selected optimizer cannot work with unconstrained '
-            'optimization problems.'
-        )
+    def check_x0_support(self, x_guesses: np.ndarray = None) -> bool:
+        """Check whether optimizer supports x0, return boolean."""
+        return True
 
 
 class ScipyOptimizer(Optimizer):
     """
     Use the SciPy optimizers.
 
-    Find details on the optimizer and configuration options at:
-    https://docs.scipy.org/doc/scipy/reference/generated/scipy.\
-        optimize.minimize.html#scipy.optimize.minimize
-    """
+    Find details on the optimizer and configuration options at: :func:`scipy.optimize.minimize`.
+
+    .. note::
+        Least-squares optimizers may face errors in case of non-continuous
+        differentiable objective functions (e.g. Laplace priors).
+    """  # noqa
 
     def __init__(
         self,
-        method: str = 'L-BFGS-B',
+        method: str = "L-BFGS-B",
         tol: float = None,
-        options: Dict = None,
+        options: dict = None,
     ):
         super().__init__()
 
@@ -277,9 +351,7 @@ class ScipyOptimizer(Optimizer):
             rep += f" options={self.options}"
         return rep + ">"
 
-    @fix_decorator
-    @time_decorator
-    @history_decorator
+    @minimize_decorator_collection
     def minimize(
         self,
         problem: Problem,
@@ -309,18 +381,18 @@ class ScipyOptimizer(Optimizer):
             bounds = (lb, ub)
 
             fun = objective.get_res
-            jac = objective.get_sres if objective.has_sres else '2-point'
+            jac = objective.get_sres if objective.has_sres else "2-point"
             # TODO: pass jac computing methods in options
 
             if self.options is not None:
                 ls_options = self.options.copy()
-                ls_options['verbose'] = (
+                ls_options["verbose"] = (
                     2
-                    if 'disp' in ls_options.keys() and ls_options['disp']
+                    if "disp" in ls_options.keys() and ls_options["disp"]
                     else 0
                 )
-                ls_options.pop('disp', None)
-                ls_options['max_nfev'] = ls_options.pop('maxfun', None)
+                ls_options.pop("disp", None)
+                ls_options["max_nfev"] = ls_options.pop("maxfun", None)
             else:
                 ls_options = {}
 
@@ -332,15 +404,15 @@ class ScipyOptimizer(Optimizer):
                 jac=jac,
                 bounds=bounds,
                 tr_solver=ls_options.pop(
-                    'tr_solver', 'lsmr' if len(x0) > 1 else 'exact'
+                    "tr_solver", "lsmr" if len(x0) > 1 else "exact"
                 ),
-                loss='linear',
+                loss="linear",
                 ftol=tol,
                 **ls_options,
             )
             # extract fval/grad from result, note that fval is not available
             # from least squares solvers
-            grad = getattr(res, 'grad', None)
+            grad = getattr(res, "grad", None)
             fval = None
         else:
             # is an fval based optimization method
@@ -358,38 +430,38 @@ class ScipyOptimizer(Optimizer):
             #        'dogleg', 'trust-ncg']
             # TODO: is it more efficient to have tuple as output of fun?
             method_supports_grad = self.method.lower() in [
-                'cg',
-                'bfgs',
-                'newton-cg',
-                'l-bfgs-b',
-                'tnc',
-                'slsqp',
-                'dogleg',
-                'trust-ncg',
-                'trust-krylov',
-                'trust-exact',
-                'trust-constr',
+                "cg",
+                "bfgs",
+                "newton-cg",
+                "l-bfgs-b",
+                "tnc",
+                "slsqp",
+                "dogleg",
+                "trust-ncg",
+                "trust-krylov",
+                "trust-exact",
+                "trust-constr",
             ]
             method_supports_hess = self.method.lower() in [
-                'newton-cg',
-                'dogleg',
-                'trust-ncg',
-                'trust-krylov',
-                'trust-exact',
-                'trust-constr',
+                "newton-cg",
+                "dogleg",
+                "trust-ncg",
+                "trust-krylov",
+                "trust-exact",
+                "trust-constr",
             ]
             method_supports_hessp = self.method.lower() in [
-                'newton-cg',
-                'trust-ncg',
-                'trust-krylov',
-                'trust-constr',
+                "newton-cg",
+                "trust-ncg",
+                "trust-krylov",
+                "trust-constr",
             ]
             # switch off passing over functions if not applicable (e.g.
             # NegLogParameterPrior) since grad/hess attributes do not exist
             if not isinstance(objective, Objective):
-                if not hasattr(objective, 'grad'):
+                if not hasattr(objective, "grad"):
                     objective.grad = False
-                if not hasattr(objective, 'hess'):
+                if not hasattr(objective, "hess"):
                     objective.hess = False
             # Todo Resolve warning by implementing saving of hess temporarily
             #  in objective and pass to scipy seperately
@@ -400,7 +472,8 @@ class ScipyOptimizer(Optimizer):
                     "for each evaluation of hess, fun will be "
                     "evaluated again. This can lead to increased "
                     "computation times. If possible, separate fun "
-                    "and hess."
+                    "and hess.",
+                    stacklevel=2,
                 )
             if objective.grad is True:
 
@@ -443,7 +516,7 @@ class ScipyOptimizer(Optimizer):
                 tol=self.tol,
             )
             # extract fval/grad from result
-            grad = getattr(res, 'jac', None)
+            grad = getattr(res, "jac", None)
             fval = res.fun
 
         # fill in everything known, although some parts will be overwritten
@@ -451,39 +524,55 @@ class ScipyOptimizer(Optimizer):
             x=np.array(res.x),
             fval=fval,
             grad=grad,
-            hess=getattr(res, 'hess', None),
+            hess=getattr(res, "hess", None),
             exitflag=res.status,
             message=res.message,
+            optimizer=str(self),
         )
 
         return optimizer_result
 
     def is_least_squares(self):
         """Check whether optimizer is a least squares optimizer."""
-        return re.match(r'(?i)^(ls_)', self.method)
+        return re.match(r"(?i)^(ls_)", self.method)
 
     def get_default_options(self):
         """Create default options specific for the optimizer."""
+        options = {"disp": False}
         if self.is_least_squares():
-            options = {'max_nfev': 1000, 'disp': False}
-        else:
-            options = {'maxfun': 1000, 'disp': False}
+            options["max_nfev"] = 1000
+        elif self.method.lower() in ("l-bfgs-b", "tnc"):
+            options["maxfun"] = 1000
+        elif self.method.lower() in ("nelder-mead", "powell"):
+            options["maxfev"] = 1000
+
+        if self.method.lower() == "l-bfgs-b":
+            # Deprecated since scipy 1.15.0
+            del options["disp"]
+
         return options
 
 
 class IpoptOptimizer(Optimizer):
-    """Use IpOpt (https://pypi.org/project/ipopt/) for optimization."""
+    """Use Ipopt (https://pypi.org/project/cyipopt/) for optimization."""
 
-    def __init__(self, options: Dict = None):
+    def __init__(self, options: dict = None):
         """
         Initialize.
 
         Parameters
         ----------
         options:
-            Options are directly passed on to `cyipopt.minimize_ipopt`.
+            Options are directly passed on to `cyipopt.minimize_ipopt`, except
+            for the `approx_grad` option, which is handled separately.
+
+            For a list of available options, see the Ipopt documentation
+            (https://coin-or.github.io/Ipopt/OPTIONS.html).
         """
         super().__init__()
+        self.approx_grad = False
+        if (options is not None) and "approx_grad" in options:
+            self.approx_grad = options.pop("approx_grad")
         self.options = options
 
     def __repr__(self) -> str:
@@ -493,9 +582,7 @@ class IpoptOptimizer(Optimizer):
             rep += f" options={self.options}"
         return rep + ">"
 
-    @fix_decorator
-    @time_decorator
-    @history_decorator
+    @minimize_decorator_collection
     def minimize(
         self,
         problem: Problem,
@@ -508,17 +595,27 @@ class IpoptOptimizer(Optimizer):
         try:
             import cyipopt
         except ImportError:
-            raise OptimizerImportError("ipopt")
+            raise OptimizerImportError("ipopt") from None
 
         objective = problem.objective
 
         bounds = np.array([problem.lb, problem.ub]).T
 
+        if self.approx_grad:
+            jac = None
+        elif objective.has_grad:
+            jac = objective.get_grad
+        else:
+            raise ValueError(
+                "For Ipopt, the objective must either be able to return "
+                "gradients or the `approx_grad` must be set to True."
+            )
+
         ret = cyipopt.minimize_ipopt(
             fun=objective.get_fval,
             x0=x0,
             method=None,  # ipopt does not use this argument for anything
-            jac=objective.get_grad,
+            jac=jac,
             hess=None,  # ipopt does not support Hessian yet
             hessp=None,  # ipopt does not support Hessian vector product yet
             bounds=bounds,
@@ -528,7 +625,10 @@ class IpoptOptimizer(Optimizer):
 
         # the ipopt return object is a scipy.optimize.OptimizeResult
         return OptimizerResult(
-            x=ret.x, exitflag=ret.status, message=ret.message
+            x=ret.x,
+            exitflag=ret.status,
+            message=ret.message,
+            optimizer=str(self),
         )
 
     def is_least_squares(self):
@@ -539,14 +639,14 @@ class IpoptOptimizer(Optimizer):
 class DlibOptimizer(Optimizer):
     """Use the Dlib toolbox for optimization."""
 
-    def __init__(self, options: Dict = None):
+    def __init__(self, options: dict = None):
         super().__init__()
 
         self.options = options
         if self.options is None:
             self.options = DlibOptimizer.get_default_options(self)
-        elif 'maxiter' not in self.options:
-            raise KeyError('Dlib options are missing the key word ' 'maxiter.')
+        elif "maxiter" not in self.options:
+            raise KeyError("Dlib options are missing the keyword maxiter.")
 
     def __repr__(self) -> str:
         rep = f"<{self.__class__.__name__}"
@@ -555,9 +655,7 @@ class DlibOptimizer(Optimizer):
             rep += f" options={self.options}"
         return rep + ">"
 
-    @fix_decorator
-    @time_decorator
-    @history_decorator
+    @minimize_decorator_collection
     def minimize(
         self,
         problem: Problem,
@@ -575,7 +673,7 @@ class DlibOptimizer(Optimizer):
         try:
             import dlib
         except ImportError:
-            raise OptimizerImportError("dlib")
+            raise OptimizerImportError("dlib") from None
 
         if not objective.has_fun:
             raise ValueError(
@@ -591,11 +689,11 @@ class DlibOptimizer(Optimizer):
             get_fval_vararg,
             list(lb),
             list(ub),
-            int(self.options['maxiter']),
+            int(self.options["maxiter"]),
             0.002,
         )
 
-        optimizer_result = OptimizerResult()
+        optimizer_result = OptimizerResult(optimizer=str(self))
 
         return optimizer_result
 
@@ -605,17 +703,23 @@ class DlibOptimizer(Optimizer):
 
     def get_default_options(self):
         """Create default options specific for the optimizer."""
-        return {'maxiter': 10000}
+        return {"maxiter": 10000}
+
+    def check_x0_support(self, x_guesses: np.ndarray = None) -> bool:
+        """Check whether optimizer supports x0."""
+        if x_guesses is not None and x_guesses.size > 0:
+            logger.warning("The Dlib optimizer does not support x0.")
+        return False
 
 
 class PyswarmOptimizer(Optimizer):
     """Global optimization using pyswarm."""
 
-    def __init__(self, options: Dict = None):
+    def __init__(self, options: dict = None):
         super().__init__()
 
         if options is None:
-            options = {'maxiter': 200}
+            options = {"maxiter": 200}
         self.options = options
 
     def __repr__(self) -> str:
@@ -625,9 +729,7 @@ class PyswarmOptimizer(Optimizer):
             rep += f" options={self.options}"
         return rep + ">"
 
-    @fix_decorator
-    @time_decorator
-    @history_decorator
+    @minimize_decorator_collection
     def minimize(
         self,
         problem: Problem,
@@ -643,7 +745,7 @@ class PyswarmOptimizer(Optimizer):
         try:
             import pyswarm
         except ImportError:
-            raise OptimizerImportError("pyswarm")
+            raise OptimizerImportError("pyswarm") from None
 
         check_finite_bounds(lb, ub)
 
@@ -651,7 +753,9 @@ class PyswarmOptimizer(Optimizer):
             problem.objective.get_fval, lb, ub, **self.options
         )
 
-        optimizer_result = OptimizerResult(x=np.array(xopt), fval=fopt)
+        optimizer_result = OptimizerResult(
+            x=np.array(xopt), fval=fopt, optimizer=str(self)
+        )
 
         return optimizer_result
 
@@ -659,8 +763,14 @@ class PyswarmOptimizer(Optimizer):
         """Check whether optimizer is a least squares optimizer."""
         return False
 
+    def check_x0_support(self, x_guesses: np.ndarray = None) -> bool:
+        """Check whether optimizer supports x0."""
+        if x_guesses is not None and x_guesses.size > 0:
+            logger.warning("The pyswarm optimizer does not support x0.")
+        return False
 
-class CmaesOptimizer(Optimizer):
+
+class CmaOptimizer(Optimizer):
     """
     Global optimization using covariance matrix adaptation evolutionary search.
 
@@ -668,7 +778,7 @@ class CmaesOptimizer(Optimizer):
     (https://github.com/CMA-ES/pycma).
     """
 
-    def __init__(self, par_sigma0: float = 0.25, options: Dict = None):
+    def __init__(self, par_sigma0: float = 0.25, options: dict = None):
         """
         Initialize.
 
@@ -684,7 +794,7 @@ class CmaesOptimizer(Optimizer):
         super().__init__()
 
         if options is None:
-            options = {'maxiter': 10000}
+            options = {"maxiter": 10000}
         self.options = options
         self.par_sigma0 = par_sigma0
 
@@ -695,9 +805,7 @@ class CmaesOptimizer(Optimizer):
             rep += f" options={self.options}"
         return rep + ">"
 
-    @fix_decorator
-    @time_decorator
-    @history_decorator
+    @minimize_decorator_collection
     def minimize(
         self,
         problem: Problem,
@@ -713,12 +821,12 @@ class CmaesOptimizer(Optimizer):
         check_finite_bounds(lb, ub)
 
         sigma0 = self.par_sigma0 * np.median(ub - lb)
-        self.options['bounds'] = [lb, ub]
+        self.options["bounds"] = [lb, ub]
 
         try:
             import cma
         except ImportError:
-            raise OptimizerImportError("cma")
+            raise OptimizerImportError("cma") from None
 
         result = (
             cma.CMAEvolutionStrategy(
@@ -731,7 +839,7 @@ class CmaesOptimizer(Optimizer):
         )
 
         optimizer_result = OptimizerResult(
-            x=np.array(result[0]), fval=result[1]
+            x=np.array(result[0]), fval=result[1], optimizer=str(self)
         )
 
         return optimizer_result
@@ -741,36 +849,47 @@ class CmaesOptimizer(Optimizer):
         return False
 
 
+class CmaesOptimizer(CmaOptimizer):
+    """Deprecated, use CmaOptimizer instead."""
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        warnings.warn(
+            "`CmaesOptimizer` has been renamed to `CmaOptimizer`, "
+            "please update your code.",
+            DeprecationWarning,
+            stacklevel=1,
+        )
+
+
 class ScipyDifferentialEvolutionOptimizer(Optimizer):
     """
     Global optimization using scipy's differential evolution optimizer.
 
-    Package homepage: https://docs.scipy.org/doc/scipy/reference/generated\
-        /scipy.optimize.differential_evolution.html
+    See: :func:`scipy.optimize.differential_evolution`.
 
     Parameters
     ----------
     options:
         Optimizer options that are directly passed on to scipy's optimizer.
 
-
     Examples
     --------
     Arguments that can be passed to options:
 
     maxiter:
-        used to calculate the maximal number of funcion evaluations by
-        maxfevals = (maxiter + 1) * popsize * len(x)
+        used to calculate the maximal number of function evaluations by
+        ``maxfevals = (maxiter + 1) * popsize * len(x)``
         Default: 100
     popsize:
         population size, default value 15
     """
 
-    def __init__(self, options: Dict = None):
+    def __init__(self, options: dict = None):
         super().__init__()
 
         if options is None:
-            options = {'maxiter': 100}
+            options = {"maxiter": 100}
         self.options = options
 
     def __repr__(self) -> str:
@@ -780,9 +899,7 @@ class ScipyDifferentialEvolutionOptimizer(Optimizer):
             rep += f" options={self.options}"
         return rep + ">"
 
-    @fix_decorator
-    @time_decorator
-    @history_decorator
+    @minimize_decorator_collection
     def minimize(
         self,
         problem: Problem,
@@ -791,15 +908,18 @@ class ScipyDifferentialEvolutionOptimizer(Optimizer):
         history_options: HistoryOptions = None,
         optimize_options: OptimizeOptions = None,
     ) -> OptimizerResult:
-        """Perform optimization. Parameters: see `Optimizer` documentation."""
+        """Perform optimization.
+
+        See :meth:`Optimizer.minimize`.
+        """
         bounds = list(zip(problem.lb, problem.ub))
 
         result = scipy.optimize.differential_evolution(
-            problem.objective.get_fval, bounds, **self.options
+            problem.objective.get_fval, bounds, x0=x0, **self.options
         )
 
         optimizer_result = OptimizerResult(
-            x=np.array(result.x), fval=result.fun
+            x=np.array(result.x), fval=result.fun, optimizer=str(self)
         )
 
         return optimizer_result
@@ -836,10 +956,10 @@ class PyswarmsOptimizer(Optimizer):
         Default: 1000
     """
 
-    def __init__(self, par_popsize: float = 10, options: Dict = None):
+    def __init__(self, par_popsize: float = 10, options: dict = None):
         super().__init__()
 
-        all_options = {'maxiter': 1000, 'c1': 0.5, 'c2': 0.3, 'w': 0.9}
+        all_options = {"maxiter": 1000, "c1": 0.5, "c2": 0.3, "w": 0.9}
         if options is None:
             options = {}
         all_options.update(options)
@@ -853,9 +973,7 @@ class PyswarmsOptimizer(Optimizer):
             rep += f" options={self.options}"
         return rep + ">"
 
-    @fix_decorator
-    @time_decorator
-    @history_decorator
+    @minimize_decorator_collection
     def minimize(
         self,
         problem: Problem,
@@ -871,7 +989,7 @@ class PyswarmsOptimizer(Optimizer):
         try:
             import pyswarms
         except ImportError:
-            raise OptimizerImportError("pyswarms")
+            raise OptimizerImportError("pyswarms") from None
 
         # check for finite values for the bounds
         if np.isfinite(lb).all() is np.False_:
@@ -888,6 +1006,7 @@ class PyswarmsOptimizer(Optimizer):
             dimensions=len(x0),
             options=self.options,
             bounds=(lb, ub),
+            # init_pos=x0,  # TODO: is a problem if x0 is not of the swarm size
         )
 
         def successively_working_fval(swarm: np.ndarray) -> np.ndarray:
@@ -911,13 +1030,14 @@ class PyswarmsOptimizer(Optimizer):
 
         cost, pos = optimizer.optimize(
             successively_working_fval,
-            iters=self.options['maxiter'],
+            iters=self.options["maxiter"],
             verbose=False,
         )
 
         optimizer_result = OptimizerResult(
             x=pos,
             fval=float(cost),
+            optimizer=str(self),
         )
 
         return optimizer_result
@@ -926,6 +1046,12 @@ class PyswarmsOptimizer(Optimizer):
         """Check whether optimizer is a least squares optimizer."""
         return False
 
+    def check_x0_support(self, x_guesses: np.ndarray = None) -> bool:
+        """Check whether optimizer supports x0."""
+        if x_guesses is not None and x_guesses.size > 0:
+            logger.warning("The pyswarms optimizer does not support x0.")
+        return False
+
 
 class NLoptOptimizer(Optimizer):
     """
@@ -938,8 +1064,8 @@ class NLoptOptimizer(Optimizer):
         self,
         method=None,
         local_method=None,
-        options: Dict = None,
-        local_options: Dict = None,
+        options: dict = None,
+        local_options: dict = None,
     ):
         """
         Initialize.
@@ -962,8 +1088,8 @@ class NLoptOptimizer(Optimizer):
 
         if options is None:
             options = {}
-        elif 'maxiter' in options:
-            options['maxeval'] = options.pop('maxiter')
+        elif "maxiter" in options:
+            options["maxeval"] = options.pop("maxiter")
         if local_options is None:
             local_options = {}
         self.options = options
@@ -972,7 +1098,7 @@ class NLoptOptimizer(Optimizer):
         try:
             import nlopt
         except ImportError:
-            raise OptimizerImportError("nlopt")
+            raise OptimizerImportError("nlopt") from None
 
         if method is None:
             method = nlopt.LD_LBFGS
@@ -992,7 +1118,7 @@ class NLoptOptimizer(Optimizer):
         if local_method is not None and method not in needs_local_method:
             raise ValueError(
                 f'Method "{method}" does not allow a local '
-                f'method. Please set `local_method` to None.'
+                f"method. Please set `local_method` to None."
             )
 
         self.local_methods = [
@@ -1041,7 +1167,7 @@ class NLoptOptimizer(Optimizer):
         if method not in methods:
             raise ValueError(
                 f'"{method}" is not a valid method. Valid '
-                f'methods are: {methods}'
+                f"methods are: {methods}"
             )
 
         self.method = method
@@ -1049,7 +1175,7 @@ class NLoptOptimizer(Optimizer):
         if local_method is not None and local_method not in self.local_methods:
             raise ValueError(
                 f'"{local_method}" is not a valid method. Valid '
-                f'methods are: {self.local_methods}'
+                f"methods are: {self.local_methods}"
             )
 
         self.local_method = local_method
@@ -1062,12 +1188,10 @@ class NLoptOptimizer(Optimizer):
         if self.options is not None:
             rep += f" options={self.options}"
         if self.local_options is not None:
-            rep += f" local_options={self.local_methods}"
+            rep += f" local_options={self.local_options}"
         return rep + ">"
 
-    @fix_decorator
-    @time_decorator
-    @history_decorator
+    @minimize_decorator_collection
     def minimize(
         self,
         problem: Problem,
@@ -1082,15 +1206,15 @@ class NLoptOptimizer(Optimizer):
         opt = nlopt.opt(self.method, problem.dim)
 
         valid_options = [
-            'ftol_abs',
-            'ftol_rel',
-            'xtol_abs',
-            'xtol_rel',
-            'stopval',
-            'x_weights',
-            'maxeval',
-            'maxtime',
-            'initial_step',
+            "ftol_abs",
+            "ftol_rel",
+            "xtol_abs",
+            "xtol_rel",
+            "stopval",
+            "x_weights",
+            "maxeval",
+            "maxtime",
+            "initial_step",
         ]
 
         def set_options(o, options):
@@ -1098,9 +1222,9 @@ class NLoptOptimizer(Optimizer):
                 if option not in valid_options:
                     raise ValueError(
                         f'"{option}" is not a valid option. Valid '
-                        f'options are: {valid_options}'
+                        f"options are: {valid_options}"
                     )
-                getattr(o, f'set_{option}')(value)
+                getattr(o, f"set_{option}")(value)
 
         if self.local_method is not None:
             local_opt = nlopt.opt(self.local_method, problem.dim)
@@ -1128,7 +1252,7 @@ class NLoptOptimizer(Optimizer):
         set_options(opt, self.options)
         try:
             result = opt.optimize(x0)
-            msg = 'Finished Successfully.'
+            msg = "Finished Successfully."
         except (
             nlopt.RoundoffLimited,
             nlopt.ForcedStop,
@@ -1144,6 +1268,7 @@ class NLoptOptimizer(Optimizer):
             fval=opt.last_optimum_value(),
             message=msg,
             exitflag=opt.last_optimize_result(),
+            optimizer=str(self),
         )
 
         return optimizer_result
@@ -1152,6 +1277,30 @@ class NLoptOptimizer(Optimizer):
         """Check whether optimizer is a least squares optimizer."""
         return False
 
+    def check_x0_support(self, x_guesses: np.ndarray = None) -> bool:
+        """Check whether optimizer supports multiple initial guesses."""
+        import nlopt
+
+        if self.method in (
+            nlopt.GN_AGS,
+            nlopt.GD_STOGO,
+            nlopt.GD_STOGO_RAND,
+            nlopt.GN_ORIG_DIRECT,
+            nlopt.GN_ORIG_DIRECT_L,
+            nlopt.GN_DIRECT,
+            nlopt.GN_DIRECT_L,
+            nlopt.GN_DIRECT_L_NOSCAL,
+            nlopt.GN_DIRECT_L_RAND,
+            nlopt.GN_DIRECT_L_RAND_NOSCAL,
+        ):
+            if x_guesses is not None and x_guesses.size > 0:
+                logger.warning(
+                    f"The NLopt optimizer method {self.method} does "
+                    "not support x0."
+                )
+            return False
+        return True
+
 
 class FidesOptimizer(Optimizer):
     """
@@ -1162,11 +1311,10 @@ class FidesOptimizer(Optimizer):
 
     def __init__(
         self,
-        hessian_update: Optional[
-            fides.hessian_approximation.HessianApproximation
-        ] = 'default',
-        options: Optional[Dict] = None,
-        verbose: Optional[int] = logging.INFO,
+        hessian_update: None
+        | (fides.hessian_approximation.HessianApproximation) = "default",
+        options: dict | None = None,
+        verbose: int | None = logging.INFO,
     ):
         """
         Initialize.
@@ -1174,10 +1322,11 @@ class FidesOptimizer(Optimizer):
         Parameters
         ----------
         options:
-            Optimizer options.
+            Optimizer options. See :meth:`fides.minimize.Optimizer.minimize`
+            and :class:`fides.constants.Options` for details.
         hessian_update:
-            Hessian update strategy. If this is None, a hybrid approximation
-            that switches from the problem.objective provided Hessian (
+            Hessian update strategy. If this is ``None``, a hybrid approximation
+            that switches from the ``problem.objective`` provided Hessian (
             approximation) to a BFGS approximation will be used.
         """
         super().__init__()
@@ -1185,20 +1334,20 @@ class FidesOptimizer(Optimizer):
         try:
             import fides
         except ImportError:
-            raise OptimizerImportError("fides")
+            raise OptimizerImportError("fides") from None
 
         if (
             (hessian_update is not None)
-            and (hessian_update != 'default')
+            and (hessian_update != "default")
             and not isinstance(
                 hessian_update,
                 fides.hessian_approximation.HessianApproximation,
             )
         ):
             raise ValueError(
-                'Incompatible type for hessian update. '
-                'Must be a HessianApproximation, '
-                f'was {type(hessian_update)}.'
+                "Incompatible type for hessian update. "
+                "Must be a HessianApproximation, "
+                f"was {type(hessian_update)}."
             )
 
         if options is None:
@@ -1212,12 +1361,11 @@ class FidesOptimizer(Optimizer):
         rep = f"<{self.__class__.__name__} "
         # print everything that is customized
         if self.hessian_update is not None:
-            if self.hessian_update == 'default':
+            if self.hessian_update == "default":
                 rep += f" hessian_update={self.hessian_update}"
             else:
                 rep += (
-                    f" hessian_update="
-                    f"{self.hessian_update.__class__.__name__}"
+                    f" hessian_update={self.hessian_update.__class__.__name__}"
                 )
         if self.verbose is not None:
             rep += f" verbose={self.verbose}"
@@ -1225,9 +1373,7 @@ class FidesOptimizer(Optimizer):
             rep += f" options={self.options}"
         return rep + ">"
 
-    @fix_decorator
-    @time_decorator
-    @history_decorator
+    @minimize_decorator_collection
     def minimize(
         self,
         problem: Problem,
@@ -1239,13 +1385,14 @@ class FidesOptimizer(Optimizer):
         """Perform optimization. Parameters: see `Optimizer` documentation."""
         import fides
 
-        if self.hessian_update == 'default':
+        if self.hessian_update == "default":
             if not problem.objective.has_hess:
                 warnings.warn(
-                    'Fides is using BFGS as hessian approximation, '
-                    'as the problem does not provide a Hessian. '
-                    'Specify a Hessian to use a more efficient '
-                    'hybrid approximation scheme.'
+                    "Fides is using BFGS as hessian approximation, "
+                    "as the problem does not provide a Hessian. "
+                    "Specify a Hessian to use a more efficient "
+                    "hybrid approximation scheme.",
+                    stacklevel=1,
                 )
                 _hessian_update = fides.BFGS()
             else:
@@ -1259,13 +1406,13 @@ class FidesOptimizer(Optimizer):
             else False
         )
 
-        args = {'mode': MODE_RES if resfun else MODE_FUN}
+        args = {"mode": MODE_RES if resfun else MODE_FUN}
 
         if not problem.objective.has_grad:
             raise ValueError(
-                'Fides cannot be applied to problems '
-                'with objectives that do not support '
-                'gradient evaluation.'
+                "Fides cannot be applied to problems "
+                "with objectives that do not support "
+                "gradient evaluation."
             )
 
         if _hessian_update is None or (
@@ -1273,13 +1420,13 @@ class FidesOptimizer(Optimizer):
         ):
             if not problem.objective.has_hess:
                 raise ValueError(
-                    'Specified hessian update scheme cannot be '
-                    'used with objectives that do not support '
-                    'Hessian computation.'
+                    "Specified hessian update scheme cannot be "
+                    "used with objectives that do not support "
+                    "Hessian computation."
                 )
-            args['sensi_orders'] = (0, 1, 2)
+            args["sensi_orders"] = (0, 1, 2)
         else:
-            args['sensi_orders'] = (0, 1)
+            args["sensi_orders"] = (0, 1)
 
         opt = fides.Optimizer(
             fun=problem.objective,
@@ -1305,6 +1452,7 @@ class FidesOptimizer(Optimizer):
             hess=opt.hess,
             message=msg,
             exitflag=opt.exitflag,
+            optimizer=str(self),
         )
 
         return optimizer_result
