@@ -2,10 +2,11 @@
 
 import logging
 import os
+from collections.abc import Iterable
 from pathlib import Path
-from typing import Iterable, List
 
 import h5py
+import numpy as np
 
 from .. import C
 from ..engine import Engine, SingleCoreEngine
@@ -60,9 +61,16 @@ def preprocess_hdf5_history(
         return False
 
     # create directory with same name as original file stem
-    template_path = (
-        path.parent / path.stem / (path.stem + "_{id}" + path.suffix)
-    )
+    if "{id}" in path.stem:
+        template_path = (
+            path.parent
+            / path.stem.replace("{id}", "")
+            / (path.stem + path.suffix)
+        )
+    else:
+        template_path = (
+            path.parent / path.stem / (path.stem + "_{id}" + path.suffix)
+        )
     template_path.parent.mkdir(parents=True, exist_ok=True)
     # set history file to template path
     history_options.storage_file = str(template_path)
@@ -71,7 +79,7 @@ def preprocess_hdf5_history(
 
 
 def postprocess_hdf5_history(
-    ret: List[OptimizerResult],
+    ret: list[OptimizerResult],
     storage_file: str,
     history_options: HistoryOptions,
 ) -> None:
@@ -91,14 +99,16 @@ def postprocess_hdf5_history(
         History options used in the optimization.
     """
     # create hdf5 file that gathers the others within history group
-    with h5py.File(storage_file, mode='w') as f:
+    if "{id}" in storage_file:
+        storage_file = storage_file.replace("{id}", "")
+    with h5py.File(storage_file, mode="w") as f:
         # create file and group
         f.require_group("history")
         # append links to each single result file
         for result in ret:
-            id = result['id']
-            f[f'history/{id}'] = h5py.ExternalLink(
-                result['history'].file, f'history/{id}'
+            id = result["id"]
+            f[f"history/{id}"] = h5py.ExternalLink(
+                result["history"].file, f"history/{id}"
             )
 
     # reset storage file (undo preprocessing changes)
@@ -152,17 +162,16 @@ def assign_ids(
         Number of starts of the optimizer.
     """
     used_ids = set()
+    n_used = 0
     if result is not None:
         used_ids = set(result.optimize_result.id)
-        N_used = len(used_ids)
+        n_used = len(used_ids)
     if ids is None:
         i = 0
-        ids = [str(j) for j in range(i * n_starts, (i + 1) * n_starts)]
+        ids = [str(j) for j in range(0, n_starts)]
         while not used_ids.isdisjoint(ids):
             i += 1
-            ids = [
-                str(j) for j in range(N_used * i, N_used * (i + 1) + n_starts)
-            ]
+            ids = [str(j) for j in range(n_used * i, n_used * i + n_starts)]
     if len(ids) != n_starts:
         raise AssertionError("Number of starts and ids must coincide.")
     if not used_ids.isdisjoint(ids):
@@ -170,3 +179,12 @@ def assign_ids(
             "Manually assigned ids must differ from existing ones."
         )
     return ids
+
+
+def check_finite_bounds(lb, ub):
+    """Raise if bounds are not finite."""
+    if not np.isfinite(lb).all() or not np.isfinite(ub).all():
+        raise ValueError(
+            "Selected optimizer cannot work with unconstrained "
+            "optimization problems."
+        )
