@@ -4,10 +4,10 @@ PEtab
 
 pyPESTO support for the PEtab data format.
 """
+
 import warnings
 
 from .importer import PetabImporter
-from .pysb_importer import PetabImporterPysb
 
 # PEtab and amici are optional dependencies
 
@@ -18,13 +18,10 @@ except ImportError:
     warnings.warn(
         "PEtab import requires an installation of petab "
         "(https://github.com/PEtab-dev/PEtab). "
-        "Install via `pip3 install petab`."
+        "Install via `pip3 install petab`.",
+        stacklevel=1,
     )
 try:
     import amici
 except ImportError:
-    warnings.warn(
-        "PEtab import requires an installation of amici "
-        "(https://github.com/AMICI-dev/AMICI). "
-        "Install via `pip3 install amici`."
-    )
+    amici = None
