@@ -1,15 +1,28 @@
 import copy
-from typing import Iterable, List, Optional, SupportsFloat, SupportsInt, Union
+import logging
+import sys
+from collections.abc import Iterable
+from typing import (
+    Callable,
+    Optional,
+    SupportsFloat,
+    SupportsInt,
+    Union,
+)
 
 import numpy as np
 import pandas as pd
 
 from ..objective import ObjectiveBase
 from ..objective.priors import NegLogParameterPriors
+from ..startpoint import StartpointMethod, to_startpoint_method, uniform
+from ..version import __version__
 
 SupportsFloatIterableOrValue = Union[Iterable[SupportsFloat], SupportsFloat]
 SupportsIntIterableOrValue = Union[Iterable[SupportsInt], SupportsInt]
 
+logger = logging.getLogger(__name__)
+
 
 class Problem:
     """
@@ -57,6 +70,9 @@ class Problem:
     copy_objective:
         Whethter to generate a deep copy of the objective function before
         potential modification the problem class performs on it.
+    startpoint_method:
+        Method for how to choose start points. ``False`` means the optimizer
+        does not require start points, e.g. for the ``PyswarmOptimizer``.
 
     Notes
     -----
@@ -75,8 +91,8 @@ class Problem:
     def __init__(
         self,
         objective: ObjectiveBase,
-        lb: Union[np.ndarray, List[float]],
-        ub: Union[np.ndarray, List[float]],
+        lb: Union[np.ndarray, list[float]],
+        ub: Union[np.ndarray, list[float]],
         dim_full: Optional[int] = None,
         x_fixed_indices: Optional[SupportsIntIterableOrValue] = None,
         x_fixed_vals: Optional[SupportsFloatIterableOrValue] = None,
@@ -84,9 +100,10 @@ class Problem:
         x_names: Optional[Iterable[str]] = None,
         x_scales: Optional[Iterable[str]] = None,
         x_priors_defs: Optional[NegLogParameterPriors] = None,
-        lb_init: Union[np.ndarray, List[float], None] = None,
-        ub_init: Union[np.ndarray, List[float], None] = None,
+        lb_init: Union[np.ndarray, list[float], None] = None,
+        ub_init: Union[np.ndarray, list[float], None] = None,
         copy_objective: bool = True,
+        startpoint_method: Union[StartpointMethod, Callable, bool] = None,
     ):
         if copy_objective:
             objective = copy.deepcopy(objective)
@@ -107,9 +124,9 @@ class Problem:
 
         if x_fixed_indices is None:
             x_fixed_indices = []
-        x_fixed_indices = _make_iterable_if_value(x_fixed_indices, 'int')
-        self.x_fixed_indices: List[int] = [
-            _type_conversion_with_check(idx, ix, 'fixed indices', 'int')
+        x_fixed_indices = _make_iterable_if_value(x_fixed_indices, "int")
+        self.x_fixed_indices: list[int] = [
+            _type_conversion_with_check(idx, ix, "fixed indices", "int")
             for idx, ix in enumerate(x_fixed_indices)
         ]
 
@@ -117,9 +134,9 @@ class Problem:
         # or remove values during profile computation
         if x_fixed_vals is None:
             x_fixed_vals = []
-        x_fixed_vals = _make_iterable_if_value(x_fixed_vals, 'float')
-        self.x_fixed_vals: List[float] = [
-            _type_conversion_with_check(idx, x, 'fixed values', 'float')
+        x_fixed_vals = _make_iterable_if_value(x_fixed_vals, "float")
+        self.x_fixed_vals: list[float] = [
+            _type_conversion_with_check(idx, x, "fixed values", "float")
             for idx, x in enumerate(x_fixed_vals)
         ]
 
@@ -130,18 +147,28 @@ class Problem:
         if x_names is None and objective.x_names is not None:
             x_names = objective.x_names
         elif x_names is None:
-            x_names = [f'x{j}' for j in range(0, self.dim_full)]
+            x_names = [f"x{j}" for j in range(0, self.dim_full)]
         if len(set(x_names)) != len(x_names):
             raise ValueError("Parameter names x_names must be unique")
-        self.x_names: List[str] = list(x_names)
+        self.x_names: list[str] = list(x_names)
 
         if x_scales is None:
-            x_scales = ['lin'] * self.dim_full
+            x_scales = ["lin"] * self.dim_full
         self.x_scales = x_scales
 
         self.x_priors = x_priors_defs
 
         self.normalize()
+        self._check_x_guesses()
+
+        # startpoint method
+        if startpoint_method is None:
+            startpoint_method = uniform
+        # convert startpoint method to class instance
+        self.startpoint_method = to_startpoint_method(startpoint_method)
+        # save python and pypesto version
+        self.python_version = ".".join(map(str, sys.version_info[:3]))
+        self.pypesto_version = __version__
 
     @property
     def lb(self) -> np.ndarray:
@@ -174,7 +201,7 @@ class Problem:
         return self.dim_full - len(self.x_fixed_indices)
 
     @property
-    def x_free_indices(self) -> List[int]:
+    def x_free_indices(self) -> list[int]:
         """Return non fixed parameters."""
         return sorted(set(range(0, self.dim_full)) - set(self.x_fixed_indices))
 
@@ -185,7 +212,7 @@ class Problem:
         Reduce all vectors to dimension dim and have the objective accept
         vectors of dimension dim.
         """
-        for attr in ['lb_full', 'lb_init_full', 'ub_full', 'ub_init_full']:
+        for attr in ["lb_full", "lb_init_full", "ub_full", "ub_init_full"]:
             value = self.__getattribute__(attr)
             if value.size == 1:
                 self.__setattr__(attr, value * np.ones(self.dim_full))
@@ -215,6 +242,15 @@ class Problem:
             x_fixed_vals=self.x_fixed_vals,
         )
 
+        # make prior aware of fixed parameters (for sampling etc.)
+        if self.x_priors is not None:
+            self.x_priors.update_from_problem(
+                dim_full=self.dim_full,
+                x_free_indices=self.x_free_indices,
+                x_fixed_indices=self.x_fixed_indices,
+                x_fixed_vals=self.x_fixed_vals,
+            )
+
         # sanity checks
         if len(self.x_scales) != self.dim_full:
             raise AssertionError("x_scales dimension invalid.")
@@ -225,11 +261,25 @@ class Problem:
                 "x_fixed_indices and x_fixed_vals must have the same length."
             )
         if np.isnan(self.lb).any():
-            raise ValueError('lb must not contain nan values')
+            raise ValueError("lb must not contain nan values")
         if np.isnan(self.ub).any():
-            raise ValueError('ub must not contain nan values')
+            raise ValueError("ub must not contain nan values")
         if np.any(self.lb >= self.ub):
-            raise ValueError('lb<ub not fulfilled.')
+            raise ValueError("lb<ub not fulfilled.")
+
+    def _check_x_guesses(self):
+        """Check whether the supplied x_guesses adhere to the bounds."""
+        if self.x_guesses.size == 0:
+            return
+        adheres_ub = self.x_guesses <= self.ub
+        adheres_lb = self.x_guesses >= self.lb
+        adheres_bounds = adheres_ub & adheres_lb
+        # if any bounds are violated, log a warning
+        if not adheres_bounds.all():
+            logger.warning(
+                "Some initial guesses supplied violate the bounds "
+                "set for this problem."
+            )
 
     def set_x_guesses(self, x_guesses: Iterable[float]):
         """
@@ -242,9 +292,10 @@ class Problem:
         x_guesses_full = np.array(x_guesses)
         if x_guesses_full.shape[1] != self.dim_full:
             raise ValueError(
-                'The dimension of individual x_guesses must be ' 'dim_full.'
+                "The dimension of individual x_guesses must be dim_full."
             )
         self.x_guesses_full = x_guesses_full
+        self._check_x_guesses()
 
     def fix_parameters(
         self,
@@ -252,8 +303,8 @@ class Problem:
         parameter_vals: SupportsFloatIterableOrValue,
     ) -> None:
         """Fix specified parameters to specified values."""
-        parameter_indices = _make_iterable_if_value(parameter_indices, 'int')
-        parameter_vals = _make_iterable_if_value(parameter_vals, 'float')
+        parameter_indices = _make_iterable_if_value(parameter_indices, "int")
+        parameter_vals = _make_iterable_if_value(parameter_vals, "float")
 
         # first clean to-be-fixed indices to avoid redundancies
         for iter_index, (x_index, x_value) in enumerate(
@@ -262,10 +313,10 @@ class Problem:
             # check if parameter was already fixed, otherwise add it to the
             # fixed parameters
             index = _type_conversion_with_check(
-                iter_index, x_index, 'indices', 'int'
+                iter_index, x_index, "indices", "int"
             )
             val = _type_conversion_with_check(
-                iter_index, x_value, 'values', 'float'
+                iter_index, x_value, "values", "float"
             )
             if index in self.x_fixed_indices:
                 self.x_fixed_vals[self.x_fixed_indices.index(index)] = val
@@ -280,12 +331,12 @@ class Problem:
     ) -> None:
         """Free specified parameters."""
         # check and adapt input
-        parameter_indices = _make_iterable_if_value(parameter_indices, 'int')
+        parameter_indices = _make_iterable_if_value(parameter_indices, "int")
 
         # first clean to-be-freed indices
         for iter_index, x_index in enumerate(parameter_indices):
             index = _type_conversion_with_check(
-                iter_index, x_index, 'indices', 'int'
+                iter_index, x_index, "indices", "int"
             )
             if index in self.x_fixed_indices:
                 fixed_x_index = self.x_fixed_indices.index(index)
@@ -295,7 +346,10 @@ class Problem:
         self.normalize()
 
     def get_full_vector(
-        self, x: Union[np.ndarray, None], x_fixed_vals: Iterable[float] = None
+        self,
+        x: Union[np.ndarray, None],
+        x_fixed_vals: Iterable[float] = None,
+        x_is_grad: bool = False,
     ) -> Union[np.ndarray, None]:
         """
         Map vector from dim to dim_full. Usually used for x, grad.
@@ -305,9 +359,9 @@ class Problem:
         x: array_like, shape=(dim,)
             The vector in dimension dim.
         x_fixed_vals: array_like, ndim=1, optional
-            The values to be used for the fixed indices. If None, then nans are
-            inserted. Usually, None will be used for grad and
-            problem.x_fixed_vals for x.
+            The values to be used for the fixed indices. If None and x_is_grad=False, problem.x_fixed_vals is used; for x_is_grad=True, nans are inserted.
+        x_is_grad: bool
+            If true, x is treated as gradients.
         """
         if x is None:
             return None
@@ -325,6 +379,9 @@ class Problem:
         x_full[..., self.x_free_indices] = x
         if x_fixed_vals is not None:
             x_full[..., self.x_fixed_indices] = x_fixed_vals
+            return x_full
+        if not x_is_grad:
+            x_full[..., self.x_fixed_indices] = self.x_fixed_vals
         return x_full
 
     def get_full_matrix(
@@ -356,7 +413,7 @@ class Problem:
     def get_reduced_vector(
         self,
         x_full: Union[np.ndarray, None],
-        x_indices: Optional[List[int]] = None,
+        x_indices: Optional[list[int]] = None,
     ) -> Union[np.ndarray, None]:
         """
         Keep only those elements, which indices are specified in x_indices.
@@ -432,20 +489,53 @@ class Problem:
             pd.DataFrame(
                 index=self.x_names,
                 data={
-                    'free': [
+                    "free": [
                         idx in self.x_free_indices
                         for idx in range(self.dim_full)
                     ],
-                    'lb_full': self.lb_full,
-                    'ub_full': self.ub_full,
+                    "lb_full": [
+                        self.lb_full[idx]
+                        if idx in self.x_free_indices
+                        else "-"
+                        for idx in range(self.dim_full)
+                    ],
+                    "ub_full": [
+                        self.ub_full[idx]
+                        if idx in self.x_free_indices
+                        else "-"
+                        for idx in range(self.dim_full)
+                    ],
+                    "scale": self.x_scales,
+                    "fixedVal (scaled)": [
+                        self.get_full_vector(np.zeros(self.dim))[idx]
+                        if idx in self.x_fixed_indices
+                        else "-"
+                        for idx in range(self.dim_full)
+                    ],
                 },
             )
         )
 
+    def get_startpoints(self, n_starts: int) -> np.ndarray:
+        """
+        Sample startpoints from method.
+
+        Parameters
+        ----------
+        n_starts:
+            Number of start points.
+
+        Returns
+        -------
+        xs:
+            Start points, shape (n_starts, dim).
+        """
+        return self.startpoint_method(n_starts, self)
+
 
 _convtypes = {
-    'float': {'attr': '__float__', 'conv': float},
-    'int': {'attr': '__int__', 'conv': int},
+    "float": {"attr": "__float__", "conv": float},
+    "int": {"attr": "__int__", "conv": int},
 }
 
 
@@ -461,24 +551,24 @@ def _type_conversion_with_check(
     Raises and appropriate error if not possible.
     """
     if convtype not in _convtypes:
-        raise ValueError(f'Unsupported type {convtype}')
+        raise ValueError(f"Unsupported type {convtype}")
 
-    can_convert = hasattr(value, _convtypes[convtype]['attr'])
+    can_convert = hasattr(value, _convtypes[convtype]["attr"])
     # this may fail for weird custom ypes that can be converted to int but
     # not float, but we probably don't want those as indiced anyways
-    lossless_conversion = not convtype == 'int' or (
-        hasattr(value, _convtypes['float']['attr'])
+    lossless_conversion = not convtype == "int" or (
+        hasattr(value, _convtypes["float"]["attr"])
         and (float(value) - int(value) == 0.0)
     )
 
     if not can_convert or not lossless_conversion:
         raise ValueError(
-            f'All {valuename} must support lossless conversion to {convtype}. '
-            f'Found type {type(value)} at index {index}, which cannot '
-            f'be converted to {convtype}.'
+            f"All {valuename} must support lossless conversion to {convtype}. "
+            f"Found type {type(value)} at index {index}, which cannot "
+            f"be converted to {convtype}."
         )
 
-    return _convtypes[convtype]['conv'](value)
+    return _convtypes[convtype]["conv"](value)
 
 
 def _make_iterable_if_value(
@@ -487,9 +577,9 @@ def _make_iterable_if_value(
 ) -> Union[Iterable[SupportsFloat], Iterable[SupportsInt]]:
     """Convert scalar values to iterables for scalar input, may update type."""
     if convtype not in _convtypes:
-        raise ValueError(f'Unsupported type {convtype}')
+        raise ValueError(f"Unsupported type {convtype}")
 
-    if not hasattr(value, '__iter__'):
-        return [_type_conversion_with_check(0, value, 'values', convtype)]
+    if not hasattr(value, "__iter__"):
+        return [_type_conversion_with_check(0, value, "values", convtype)]
     else:
         return value
