@@ -36,6 +36,15 @@ class Result:
         self.profile_result = profile_result or ProfileResult()
         self.sample_result = sample_result or SampleResult()
 
-    def summary(self) -> str:
-        """Get summary of the object."""
-        return self.optimize_result.summary()
+    def summary(self, full: bool = False, show_hess: bool = True) -> str:
+        """
+        Get summary of the object.
+
+        Parameters
+        ----------
+        full:
+            If True, print full vectors including fixed parameters.
+        show_hess:
+            If True, display the Hessian of the OptimizeResult.
+        """
+        return self.optimize_result.summary(full=full, show_hess=show_hess)
