@@ -1,5 +1,4 @@
 import numbers
-from typing import Dict, Tuple
 
 import numpy as np
 
@@ -8,9 +7,56 @@ from .metropolis import MetropolisSampler
 
 
 class AdaptiveMetropolisSampler(MetropolisSampler):
-    """Metropolis-Hastings sampler with adaptive proposal covariance."""
+    """Metropolis-Hastings sampler with adaptive proposal covariance.
 
-    def __init__(self, options: Dict = None):
+    A core problem of the standard Metropolis-Hastings sampler is
+    its fixed proposal distribution, which must be manually tuned.
+    This sampler adapts the proposal distribution during the sampling
+    process based on previous samples.
+    It adapts the correlation structure and the scaling factor of the
+    proposal distribution.
+    For both parts, there exist a variety of methods, see
+
+    * Ballnus et al. 2017.
+      Comprehensive benchmarking of Markov chain Monte Carlo methods for
+      dynamical systems
+      (https://doi.org/10.1186/s12918-017-0433-1)
+    * Andrieu et al. 2008.
+      A tutorial on adaptive MCMC
+      (https://doi.org/10.1007/s11222-008-9110-y)
+
+    for a review.
+
+    Here, we approximate the covariance matrix via a weighted average of
+    current and earlier samples,
+    with a decay factor determining the relative contribution of the
+    current sample and earlier ones to the weighted average of mean and
+    covariance.
+    The scaling factor we aim to converge to a fixed target acceptance rate
+    of 0.234, as suggested by theoretical results.
+    The implementation is based on:
+
+    * Lacki et al. 2015.
+      State-dependent swap strategies and automatic reduction of number of
+      temperatures in adaptive parallel tempering algorithm
+      (https://doi.org/10.1007/s11222-015-9579-0)
+    * Miasojedow et al. 2013.
+      An adaptive parallel tempering algorithm
+      (https://doi.org/10.1080/10618600.2013.778779)
+
+    In turn, these are based on adaptive MCMC as discussed in:
+
+    * Haario et al. 2001.
+      An adaptive Metropolis algorithm
+      (https://doi.org/10.2307/3318737)
+
+    For reference matlab implementations see:
+
+    * https://github.com/ICB-DCM/PESTO/blob/master/private/performPT.m
+    * https://github.com/ICB-DCM/PESTO/blob/master/private/updateStatistics.m
+    """
+
+    def __init__(self, options: dict = None):
         super().__init__(options)
         self._cov = None
         self._mean_hist = None
@@ -24,35 +70,35 @@ class AdaptiveMetropolisSampler(MetropolisSampler):
             # controls adaptation degeneration velocity of the proposals
             # in [0, 1], with 0 -> no adaptation, i.e. classical
             # Metropolis-Hastings
-            'decay_constant': 0.51,
+            "decay_constant": 0.51,
             # number of samples before adaptation decreases significantly.
             # a higher value reduces the impact of early adaptation
-            'threshold_sample': 1,
+            "threshold_sample": 1,
             # regularization factor for ill-conditioned cov matrices of
             # the adapted proposal density. regularization might happen if the
             # eigenvalues of the cov matrix strongly differ in order
             # of magnitude. in this case, the algorithm adds a small
             # diag matrix to the cov matrix with elements of this factor
-            'reg_factor': 1e-6,
+            "reg_factor": 1e-6,
             # initial covariance matrix. defaults to a unit matrix
-            'cov0': None,
+            "cov0": None,
             # target acceptance rate
-            'target_acceptance_rate': 0.234,
+            "target_acceptance_rate": 0.234,
             # show progress
-            'show_progress': True,
+            "show_progress": None,
         }
 
     def initialize(self, problem: Problem, x0: np.ndarray):
         """Initialize the sampler."""
         super().initialize(problem, x0)
 
-        if self.options['cov0'] is not None:
-            cov0 = self.options['cov0']
+        if self.options["cov0"] is not None:
+            cov0 = self.options["cov0"]
             if isinstance(cov0, numbers.Real):
                 cov0 = float(cov0) * np.eye(len(x0))
         else:
             cov0 = np.eye(len(x0))
-        self._cov = regularize_covariance(cov0, self.options['reg_factor'])
+        self._cov = regularize_covariance(cov0, self.options["reg_factor"])
         self._mean_hist = self.trace_x[-1]
         self._cov_hist = self._cov
         self._cov_scale = 1.0
@@ -65,10 +111,10 @@ class AdaptiveMetropolisSampler(MetropolisSampler):
         self, x: np.ndarray, lpost: float, log_p_acc: float, n_sample_cur: int
     ):
         # parse options
-        decay_constant = self.options['decay_constant']
-        threshold_sample = self.options['threshold_sample']
-        reg_factor = self.options['reg_factor']
-        target_acceptance_rate = self.options['target_acceptance_rate']
+        decay_constant = self.options["decay_constant"]
+        threshold_sample = self.options["threshold_sample"]
+        reg_factor = self.options["reg_factor"]
+        target_acceptance_rate = self.options["target_acceptance_rate"]
 
         # compute historical mean and covariance
         self._mean_hist, self._cov_hist = update_history_statistics(
@@ -79,7 +125,7 @@ class AdaptiveMetropolisSampler(MetropolisSampler):
             decay_constant=decay_constant,
         )
 
-        # compute covariance scaling factor
+        # compute covariance scaling factor based on the target acceptance rate
         self._cov_scale *= np.exp(
             (np.exp(log_p_acc) - target_acceptance_rate)
             / np.power(n_sample_cur + 1, decay_constant)
@@ -99,9 +145,12 @@ def update_history_statistics(
     x_new: np.ndarray,
     n_cur_sample: int,
     decay_constant: float,
-) -> Tuple[np.ndarray, np.ndarray]:
-    """
-    Update sampling statistics.
+) -> tuple[np.ndarray, np.ndarray]:
+    """Update sampling mean and covariance matrix via weighted average.
+
+    Update sampling mean and covariance matrix based on the previous
+    estimate and the most recent sample via a weighted average,
+    with gradually decaying update rate.
 
     Parameters
     ----------
