@@ -1,23 +1,44 @@
 """AdaptiveParallelTemperingSampler class."""
-from typing import Dict, Sequence
+
+from collections.abc import Sequence
 
 import numpy as np
 
+from ..C import EXPONENTIAL_DECAY
 from .parallel_tempering import ParallelTemperingSampler
 
 
 class AdaptiveParallelTemperingSampler(ParallelTemperingSampler):
-    """Parallel tempering sampler with adaptive temperature adaptation."""
+    """Parallel tempering sampler with adaptive temperature adaptation.
+
+    Compared to the base class, this sampler adapts the temperatures
+    during the sampling process.
+    This both simplifies the setup as it avoids manual tuning,
+    and improves the performance as the temperatures are adapted to the
+    current state of the chains.
+
+    This implementation is based on:
+
+    * Vousden et al. 2016.
+      Dynamic temperature selection for parallel tempering in Markov chain
+      Monte Carlo simulations
+      (https://doi.org/10.1093/mnras/stv2422),
+
+    via a matlab reference implementation
+    (https://github.com/ICB-DCM/PESTO/blob/master/private/performPT.m).
+    """
 
     @classmethod
-    def default_options(cls) -> Dict:
+    def default_options(cls) -> dict:
         """Get default options for sampler."""
         options = super().default_options()
         # scaling factor for temperature adaptation
-        options['eta'] = 100
+        options["eta"] = 100
         # controls the adaptation degeneration velocity of the temperature
         # adaption.
-        options['nu'] = 1e3
+        options["nu"] = 1e3
+        # initial temperature schedule as in Vousden et. al. 2016.
+        options["beta_init"] = EXPONENTIAL_DECAY
 
         return options
 
@@ -27,8 +48,8 @@ class AdaptiveParallelTemperingSampler(ParallelTemperingSampler):
             return
 
         # parameters
-        nu = self.options['nu']
-        eta = self.options['eta']
+        nu = self.options["nu"]
+        eta = self.options["eta"]
         betas = self.betas
 
         # booleans to integer array
