@@ -3,7 +3,6 @@
 from __future__ import annotations
 
 import logging
-from typing import List, Union
 
 import numpy as np
 
@@ -46,7 +45,7 @@ class EmceeSampler(Sampler):
         try:
             import emcee
         except ImportError:
-            raise SamplerImportError("emcee")
+            raise SamplerImportError("emcee") from None
 
         super().__init__()
         self.nwalkers: int = nwalkers
@@ -60,9 +59,9 @@ class EmceeSampler(Sampler):
         self.run_args: dict = run_args
 
         # set in initialize
-        self.problem: Union[Problem, None] = None
-        self.sampler: Union[emcee.EnsembleSampler, None] = None
-        self.state: Union[emcee.State, None] = None
+        self.problem: Problem | None = None
+        self.sampler: emcee.EnsembleSampler | None = None
+        self.state: emcee.State | None = None
 
     def get_epsilon_ball_initial_state(
         self,
@@ -123,7 +122,7 @@ class EmceeSampler(Sampler):
     def initialize(
         self,
         problem: Problem,
-        x0: Union[np.ndarray, List[np.ndarray]],
+        x0: np.ndarray | list[np.ndarray],
     ) -> None:
         """Initialize the sampler.
 
@@ -150,7 +149,7 @@ class EmceeSampler(Sampler):
         lb = self.problem.lb
         ub = self.problem.ub
 
-        # parameter dimenstion
+        # parameter dimension
         ndim = len(self.problem.x_free_indices)
 
         def log_prob(x):
