@@ -1,19 +1,43 @@
-from typing import Dict, Sequence, Union
+from collections.abc import Sequence
+from typing import Union
 
 import numpy as np
-from tqdm import tqdm
 
 from ..history import NoHistory
 from ..objective import NegLogPriors, ObjectiveBase
 from ..problem import Problem
 from ..result import McmcPtResult
+from ..util import tqdm
 from .sampler import InternalSample, InternalSampler
 
 
 class MetropolisSampler(InternalSampler):
-    """Simple Metropolis-Hastings sampler with fixed proposal variance."""
+    """Simple Metropolis-Hastings sampler with fixed proposal variance.
 
-    def __init__(self, options: Dict = None):
+    The Metropolis-Hastings sampler is a Markov chain Monte Carlo (MCMC)
+    method generating a sequence of samples from a probability
+    distribution.
+
+    This class implements a simple Metropolis algorithm with fixed
+    symmetric Gaussian proposal distribution.
+
+    For the underlying original publication, see:
+
+    * Metropolis et al. 1953.
+      Equation of State Calculations by Fast Computing Machines
+      (https://doi.org/10.1063/1.1699114)
+    * Hastings 1970.
+      Monte Carlo sampling methods using Markov chains and their
+      applications
+      (https://doi.org/10.1093/biomet/57.1.97)
+
+    For reference matlab implementations see:
+
+    * https://en.wikipedia.org/wiki/Metropolis%E2%80%93Hastings_algorithm
+    * https://github.com/ICB-DCM/PESTO/blob/master/private/performPT.m
+    """
+
+    def __init__(self, options: dict = None):
         super().__init__(options)
         self.problem: Union[Problem, None] = None
         self.neglogpost: Union[ObjectiveBase, None] = None
@@ -27,8 +51,8 @@ class MetropolisSampler(InternalSampler):
     def default_options(cls):
         """Return the default options for the sampler."""
         return {
-            'std': 1.0,  # the proposal standard deviation
-            'show_progress': True,  # whether to show the progress
+            "std": 1.0,  # the proposal standard deviation
+            "show_progress": None,  # whether to show the progress
         }
 
     def initialize(self, problem: Problem, x0: np.ndarray):
@@ -50,10 +74,10 @@ class MetropolisSampler(InternalSampler):
         lpost = -self.trace_neglogpost[-1]
         lprior = -self.trace_neglogprior[-1]
 
-        show_progress = self.options['show_progress']
+        show_progress = self.options.get("show_progress", None)
 
         # loop over iterations
-        for _ in tqdm(range(int(n_samples)), disable=not show_progress):
+        for _ in tqdm(range(int(n_samples)), enable=show_progress):
             # perform step
             x, lpost, lprior = self._perform_step(
                 x=x, lpost=lpost, lprior=lprior, beta=beta
@@ -75,7 +99,7 @@ class MetropolisSampler(InternalSampler):
         temper_lpost:
             Whether to temperate the posterior or only the likelihood.
         """
-        self.options['show_progress'] = False
+        self.options["show_progress"] = False
         self.temper_lpost = temper_lpost
 
     def _perform_step(
@@ -105,6 +129,14 @@ class MetropolisSampler(InternalSampler):
         # compute log prior
         lprior_new = -self.neglogprior(x_new)
 
+        # if lpost_new is -inf, x_new will not be accepted
+        if lpost_new == -np.inf:
+            # update proposal
+            self._update_proposal(
+                x, lpost, -np.inf, len(self.trace_neglogpost) + 1
+            )
+            return x, lpost, lprior
+
         if not self.temper_lpost:
             # extract current log likelihood value
             llh = lpost - lprior
@@ -112,6 +144,9 @@ class MetropolisSampler(InternalSampler):
             llh_new = lpost_new - lprior_new
             # log acceptance probability (temper log likelihood)
             log_p_acc = min(beta * (llh_new - llh) + (lprior_new - lprior), 0)
+            # catch the nan case which occurs if both lpost_new and lprior_new are -inf
+            if np.isnan(log_p_acc):
+                log_p_acc = -np.inf
         else:
             # log acceptance probability (temper log posterior)
             log_p_acc = min(beta * (lpost_new - lpost), 0)
@@ -135,7 +170,7 @@ class MetropolisSampler(InternalSampler):
 
     def _propose_parameter(self, x: np.ndarray):
         """Propose a step."""
-        x_new: np.ndarray = x + self.options['std'] * np.random.randn(len(x))
+        x_new: np.ndarray = x + self.options["std"] * np.random.randn(len(x))
         return x_new
 
     def _update_proposal(
