@@ -1,7 +1,7 @@
 """A set of helper functions."""
+
 import logging
 import os
-from typing import Tuple
 
 import numpy as np
 
@@ -16,7 +16,7 @@ def calculate_ci_mcmc_sample(
     result: Result,
     ci_level: float = 0.95,
     exclude_burn_in: bool = True,
-) -> Tuple[np.ndarray, np.ndarray]:
+) -> tuple[np.ndarray, np.ndarray]:
     """Calculate parameter credibility intervals based on MCMC samples.
 
     Parameters
@@ -25,6 +25,8 @@ def calculate_ci_mcmc_sample(
         The pyPESTO result object with filled sample result.
     ci_level:
         Lower tail probability, defaults to 95% interval.
+    exclude_burn_in:
+        Whether to exclude the burn-in samples.
 
     Returns
     -------
@@ -50,7 +52,7 @@ def calculate_ci_mcmc_sample(
 def calculate_ci_mcmc_sample_prediction(
     simulated_values: np.ndarray,
     ci_level: float = 0.95,
-) -> Tuple[np.ndarray, np.ndarray]:
+) -> tuple[np.ndarray, np.ndarray]:
     """Calculate prediction credibility intervals based on MCMC samples.
 
     Parameters
@@ -73,7 +75,7 @@ def calculate_ci(
     values: np.ndarray,
     ci_level: float,
     **kwargs,
-) -> Tuple[np.ndarray, np.ndarray]:
+) -> tuple[np.ndarray, np.ndarray]:
     """Calculate confidence/credibility levels using percentiles.
 
     Parameters
