@@ -2,11 +2,13 @@
 Model Selection
 ===============
 
-Perform model selection with a PEtab Select problem.
+Perform model selection with a
+`PEtab Select <https://petab-select.readthedocs.io/>`_ problem.
 """
 
 from . import postprocessors
-from .misc import model_to_pypesto_problem
+from .misc import SacessMinimizeMethod, model_to_pypesto_problem
+from .model_problem import ModelProblem
 from .problem import Problem
 
 try:
@@ -19,5 +21,6 @@ except ImportError:
     warnings.warn(
         "pyPESTO's model selection methods require an installation of PEtab "
         "Select (https://github.com/PEtab-dev/petab_select). Install via "
-        "`pip3 install petab-select` or `pip3 install pypesto[select]`."
+        "`pip3 install petab-select` or `pip3 install pypesto[select]`.",
+        stacklevel=1,
     )
