@@ -1,30 +1,35 @@
 """Functionality related to using a PEtab Select model selection method."""
+
 import logging
 from dataclasses import dataclass
 from enum import Enum
-from typing import Any, Callable, Dict, List, Optional, Tuple, Union
+from typing import Any, Callable
 
 import numpy as np
 import petab_select
 from petab_select import (
+    CANDIDATE_SPACE,
+    MODELS,
+    PREDECESSOR_MODEL,
+    UNCALIBRATED_MODELS,
     VIRTUAL_INITIAL_MODEL,
     CandidateSpace,
     Criterion,
     Method,
     Model,
+    Models,
 )
 
-from ..C import TYPE_POSTPROCESSOR
 from ..problem import Problem
-from .model_problem import ModelProblem
+from .model_problem import TYPE_POSTPROCESSOR, ModelProblem
 
 
 class MethodSignalProceed(str, Enum):
     """Indicators for how a model selection method should proceed."""
 
     # TODO move to PEtab Select?
-    STOP = 'stop'
-    CONTINUE = 'continue'
+    STOP = "stop"
+    CONTINUE = "continue"
 
 
 @dataclass
@@ -56,13 +61,13 @@ class MethodLogger:
     level:
         The logging level.
     logger:
-        A logger from the `logging` module.
+        A logger from the :mod:`logging` module.
     """
 
     column_width: int = 12
     column_sep: str = " | "
 
-    def __init__(self, level: str = 'info'):
+    def __init__(self, level: str = "info"):
         self.logger = logging.getLogger(__name__)
         self.level = level
 
@@ -84,7 +89,7 @@ class MethodLogger:
     def new_selection(self) -> None:
         """Start logging a new model selection."""
         padding = 20
-        self.log('-' * padding + 'New Selection' + '-' * padding)
+        self.log("-" * padding + "New Selection" + "-" * padding)
         columns = {
             "Predecessor model subspace:ID": "model0",
             "Model subspace:ID": "model",
@@ -139,12 +144,11 @@ class MethodLogger:
 
             Returns
             -------
-            str
-                The ID.
+            The ID.
             """
-            model_subspace_id = model.model_subspace_id or ''
+            model_subspace_id = model.model_subspace_id or ""
             original_model_id = model.model_id or model.get_hash()
-            model_id = model_subspace_id + ':' + original_model_id
+            model_id = model_subspace_id + ":" + original_model_id
             return model_id
 
         def float_to_str(value: float, precision: int = 3) -> str:
@@ -194,51 +198,50 @@ class MethodCaller:
     petab_select_problem:
         The PEtab Select problem.
     candidate_space:
-        A `petab_select.CandidateSpace`, used to generate candidate models.
+        A :class:`petab_select.CandidateSpace`, used to generate candidate models.
     criterion:
         The criterion by which models will be compared.
     criterion_threshold:
-        The minimum improvement in criterion that a test model must have to
+        The minimum improvement in `criterion` that a test model must have to
         be selected. The comparison is made according to the method. For
         example, in `ForwardSelector`, test models are compared to the
         previously selected model.
     calibrated_models:
-        The calibrated models of the model selection, as a `dict` where keys
-        are model hashes and values are models.
+        All calibrated models of the model selection.
     limit:
         Limit the number of calibrated models. NB: the number of accepted
         models may (likely) be fewer.
     logger:
-        A `MethodLogger`, used to log results.
-    minimize_options:
-        A dictionary that will be passed to `pypesto.minimize` as keyword
-        arguments for model optimization.
-    model_postprocessor:
-        A method that is applied to each model after calibration.
-    objective_customizer:
-        A method that is applied to the pyPESTO objective after the
-        objective is initialized, before calibration.
+        A :class:`MethodLogger`, used to log results.
     predecessor_model:
         Specify the predecessor (initial) model for the model selection
-        algorithm. If `None`, then the algorithm will generate an
+        algorithm. If ``None``, then the algorithm will generate an initial
         predecessor model if required.
+    user_calibrated_models:
+        Supply calibration results for models yourself, as a list of models.
+        If a model with the same hash is encountered in the current model
+        selection run, and the user-supplied calibrated model has the
+        `criterion` value set, the model will not be calibrated again.
     select_first_improvement:
-        If `True`, model selection will terminate as soon as a better model
+        If ``True``, model selection will terminate as soon as a better model
         is found. If `False`, all candidate models will be tested.
     startpoint_latest_mle:
-        If `True`, one of the startpoints in the multistart optimization
+        If ``True``, one of the startpoints in the multistart optimization
         will be the MLE of the latest model.
     """
 
     def __init__(
         self,
         petab_select_problem: petab_select.Problem,
-        calibrated_models: Dict[str, Model],
+        calibrated_models: Models,
         # Arguments/attributes that can simply take the default value here.
         criterion_threshold: float = 0.0,
         limit: int = np.inf,
-        minimize_options: Dict = None,
+        # TODO deprecated
+        minimize_options: dict = None,
+        # TODO deprecated
         model_postprocessor: TYPE_POSTPROCESSOR = None,
+        # TODO deprecated
         objective_customizer: Callable = None,
         select_first_improvement: bool = False,
         startpoint_latest_mle: bool = True,
@@ -248,7 +251,10 @@ class MethodCaller:
         # TODO misleading, `Method` here is simply an Enum, not a callable...
         method: Method = None,
         predecessor_model: Model = None,
+        # TODO deprecated
         model_to_pypesto_problem_method: Callable[[Any], Problem] = None,
+        model_problem_options: dict = None,
+        user_calibrated_models: list[Model] = None,
     ):
         """Arguments are used in every `__call__`, unless overridden."""
         self.petab_select_problem = petab_select_problem
@@ -256,16 +262,41 @@ class MethodCaller:
 
         self.criterion_threshold = criterion_threshold
         self.limit = limit
-        self.minimize_options = minimize_options
-        self.model_postprocessor = model_postprocessor
-        self.objective_customizer = objective_customizer
         self.predecessor_model = predecessor_model
         self.select_first_improvement = select_first_improvement
         self.startpoint_latest_mle = startpoint_latest_mle
-        self.model_to_pypesto_problem_method = model_to_pypesto_problem_method
+
+        self.user_calibrated_models = Models()
+        if user_calibrated_models is not None:
+            self.user_calibrated_models = user_calibrated_models
 
         self.logger = MethodLogger()
 
+        # TODO deprecated
+        old_model_problem_options = {}
+        for key, value in [
+            ("postprocessor", model_postprocessor),
+            (
+                "model_to_pypesto_problem_method",
+                model_to_pypesto_problem_method,
+            ),
+            ("minimize_options", minimize_options),
+            ("objective_customizer", objective_customizer),
+        ]:
+            if value is not None:
+                old_model_problem_options[key] = value
+                self.logger.log(
+                    f"Specifying `{key}` as an individual argument is "
+                    "deprecated. Please instead specify it within some "
+                    "`model_problem_options` dictionary, e.g. "
+                    f'`model_problem_options={{"{key}": ...}}`.',
+                    level="warning",
+                )
+        self.model_problem_options = {}
+        self.model_problem_options |= old_model_problem_options
+        if model_problem_options is not None:
+            self.model_problem_options |= model_problem_options
+
         self.criterion = criterion
         if self.criterion is None:
             self.criterion = self.petab_select_problem.criterion
@@ -274,24 +305,26 @@ class MethodCaller:
         if candidate_space is not None and method is not None:
             self.logger.log(
                 (
-                    'Both `candidate_space` and `method` were provided. '
-                    'Please only provide one. The method will be ignored here.'
+                    "Both `candidate_space` and `method` were provided. "
+                    "Please only provide one. The method will be ignored here."
                 ),
-                level='warning',
+                level="warning",
             )
         # Get method.
         self.method = (
             method
             if method is not None
-            else candidate_space.method
-            if candidate_space is not None
-            else self.petab_select_problem.method
+            else (
+                candidate_space.method
+                if candidate_space is not None
+                else self.petab_select_problem.method
+            )
         )
         # Require either a candidate space or a method.
         if candidate_space is None and self.method is None:
             raise ValueError(
-                'Please provide one of either `candidate_space` or `method`, '
-                'or specify the `method` in the PEtab Select problem.'
+                "Please provide one of either `candidate_space` or `method`, "
+                "or specify the `method` in the PEtab Select problem."
             )
         # Use candidate space if provided.
         if candidate_space is not None:
@@ -300,98 +333,97 @@ class MethodCaller:
                 candidate_space.set_predecessor_model(predecessor_model)
         # Else generate one based on the PEtab Select problem.
         else:
-            self.candidate_space = (
-                self.petab_select_problem.new_candidate_space(
-                    method=self.method,
-                    predecessor_model=self.predecessor_model,
+            if predecessor_model is not None:
+                self.candidate_space = (
+                    self.petab_select_problem.new_candidate_space(
+                        method=self.method,
+                        predecessor_model=self.predecessor_model,
+                    )
+                )
+            else:
+                self.candidate_space = (
+                    self.petab_select_problem.new_candidate_space(
+                        method=self.method,
+                    )
                 )
-            )
         # May have changed from `None` to `petab_select.VIRTUAL_INITIAL_MODEL`
         self.predecessor_model = self.candidate_space.get_predecessor_model()
 
-    def __call__(
-        self,
-        predecessor_model: Optional[Union[Model, None]] = None,
-        newly_calibrated_models: Optional[Dict[str, Model]] = None,
-    ) -> Tuple[List[Model], Dict[str, Model]]:
+    def __call__(self) -> tuple[Model, Models]:
         """Run a single iteration of the model selection method.
 
         A single iteration here refers to calibration of all candidate models.
         For example, given a predecessor model with 3 estimated parameters,
         with the forward method, a single iteration would involve calibration
         of all models that have both: the same 3 estimated parameters; and 1
-        additional estimated paramenter.
-
-        The input `newly_calibrated_models` is from the previous iteration. The
-        output `newly_calibrated_models` is from the current iteration.
-
-        Parameters
-        ----------
-        predecessor_model:
-            The model that will be used for comparison. Example 1: the
-            initial model of a forward method. Example 2: all models found
-            with a brute force method should be better than this model.
-        newly_calibrated_models:
-            The newly calibrated models from the previous iteration.
+        additional estimated parameter.
 
         Returns
         -------
-        tuple
-            A 2-tuple, with the following values:
+        A 2-tuple, with the following values:
 
-               1. the predecessor model for the newly calibrated models; and
-               2. the newly calibrated models, as a `dict` where keys are model
-                  hashes and values are models.
+           1. the predecessor model for the newly calibrated models; and
+           2. the newly calibrated models.
         """
         # All calibrated models in this iteration (see second return value).
         self.logger.new_selection()
 
-        if predecessor_model is None:
-            # May still be `None` (e.g. brute force method)
-            predecessor_model = self.predecessor_model
-
-        candidate_space = petab_select.ui.candidates(
+        iteration = petab_select.ui.start_iteration(
             problem=self.petab_select_problem,
             candidate_space=self.candidate_space,
             limit=self.limit,
-            calibrated_models=self.calibrated_models,
-            newly_calibrated_models=newly_calibrated_models,
-            excluded_model_hashes=self.calibrated_models.keys(),
-            # FIXME meaning changes here, i.e. predecessor model is used as previous
-            #       predecessor model, since PEtab Select now gets the next predecessor
-            #       model if the candidate space has models.
-            previous_predecessor_model=predecessor_model,
             criterion=self.criterion,
+            user_calibrated_models=self.user_calibrated_models,
         )
-        predecessor_model = self.candidate_space.predecessor_model
 
-        if not candidate_space.models:
+        if not iteration[UNCALIBRATED_MODELS]:
             raise StopIteration("No valid models found.")
 
         # TODO parallelize calibration (maybe not sensible if
         #      `self.select_first_improvement`)
-        newly_calibrated_models = {}
-        for candidate_model in candidate_space.models:
-            # autoruns calibration
-            self.new_model_problem(model=candidate_model)
-            newly_calibrated_models[
-                candidate_model.get_hash()
-            ] = candidate_model
+        calibrated_models = Models()
+        for model in iteration[UNCALIBRATED_MODELS]:
+            if (
+                model.get_criterion(
+                    criterion=self.criterion,
+                    compute=True,
+                    raise_on_failure=False,
+                )
+                is not None
+            ):
+                self.logger.log(
+                    message=(
+                        "Unexpected calibration result already available for "
+                        f"model: `{model.get_hash()}`. Skipping "
+                        "calibration."
+                    ),
+                    level="warning",
+                )
+            else:
+                self.new_model_problem(model=model)
+
+            calibrated_models.append(model)
             method_signal = self.handle_calibrated_model(
-                model=candidate_model,
-                predecessor_model=predecessor_model,
+                model=model,
+                predecessor_model=iteration[PREDECESSOR_MODEL],
             )
             if method_signal.proceed == MethodSignalProceed.STOP:
                 break
 
-        self.calibrated_models.update(newly_calibrated_models)
+        iteration_results = petab_select.ui.end_iteration(
+            problem=self.petab_select_problem,
+            candidate_space=iteration[CANDIDATE_SPACE],
+            calibrated_models=calibrated_models,
+        )
 
-        return predecessor_model, newly_calibrated_models
+        self.calibrated_models += iteration_results[MODELS]
+
+        return iteration_results[MODELS]
 
     def handle_calibrated_model(
         self,
         model: Model,
-        predecessor_model: Optional[Model],
+        predecessor_model: Model,
     ) -> MethodSignal:
         """Handle the model selection method, given a new calibrated model.
 
@@ -404,8 +436,7 @@ class MethodCaller:
 
         Returns
         -------
-        MethodSignal
-            A `MethodSignal` that describes the result.
+        A :class:`MethodSignal` that describes the result.
         """
         # Use the predecessor model from `__init__` if an iteration-specific
         # predecessor model was not supplied to `__call__`.
@@ -421,8 +452,7 @@ class MethodCaller:
 
         # Reject the model if it doesn't improve on the predecessor model.
         if (
-            predecessor_model is not None
-            and predecessor_model != VIRTUAL_INITIAL_MODEL
+            predecessor_model.hash != VIRTUAL_INITIAL_MODEL.hash
             and not self.model1_gt_model0(
                 model1=model, model0=predecessor_model
             )
@@ -463,9 +493,8 @@ class MethodCaller:
 
         Returns
         -------
-        bool
-            `True`, if `model1` is superior to `model0` by the criterion,
-            else `False`.
+        ``True``, if `model1` is superior to `model0` by the criterion,
+        else ``False``.
         """
         if self.criterion in [
             Criterion.AIC,
@@ -507,8 +536,7 @@ class MethodCaller:
 
         Returns
         -------
-        ModelProblem
-            The model selection problem.
+        The model selection problem.
         """
         x_guess = None
         if (
@@ -518,12 +546,14 @@ class MethodCaller:
             predecessor_model = self.calibrated_models[
                 model.predecessor_model_hash
             ]
-            if str(model.petab_yaml) != str(predecessor_model.petab_yaml):
+            if str(model.model_subspace_petab_yaml) != str(
+                predecessor_model.model_subspace_petab_yaml
+            ):
                 raise NotImplementedError(
-                    'The PEtab YAML files differ between the model and its '
-                    'predecessor model. This may imply different (fixed union '
-                    'estimated) parameter sets. Support for this is not yet '
-                    'implemented.'
+                    "The PEtab YAML files differ between the model and its "
+                    "predecessor model. This may imply different (fixed union "
+                    "estimated) parameter sets. Support for this is not yet "
+                    "implemented."
                 )
             x_guess = {
                 **predecessor_model.parameters,
@@ -536,8 +566,5 @@ class MethodCaller:
             valid=valid,
             autorun=autorun,
             x_guess=x_guess,
-            minimize_options=self.minimize_options,
-            objective_customizer=self.objective_customizer,
-            postprocessor=self.model_postprocessor,
-            model_to_pypesto_problem_method=self.model_to_pypesto_problem_method,
+            **self.model_problem_options,
         )
