@@ -2,6 +2,8 @@
 
 import ast
 import logging
+from pathlib import Path
+from typing import Union
 
 import h5py
 import numpy as np
@@ -18,7 +20,7 @@ def read_hdf5_profile(
     f: h5py.File,
     profile_id: str,
     parameter_id: str,
-) -> 'ProfilerResult':
+) -> "ProfilerResult":
     """Read HDF5 results per start.
 
     Parameters
@@ -32,25 +34,23 @@ def read_hdf5_profile(
         specifies the profile index that is read
         from the HDF5 file
     """
-    result = ProfilerResult(np.array([]), np.array([]), np.array([]))
+    result = ProfilerResult(np.empty((0, 0)), np.array([]), np.array([]))
 
     for profile_key in result.keys():
-        if profile_key in f[f'/profiling/{profile_id}/{parameter_id}']:
+        if profile_key in f[f"/profiling/{profile_id}/{parameter_id}"]:
             result[profile_key] = f[
-                f'/profiling/{profile_id}/{parameter_id}/{profile_key}'
+                f"/profiling/{profile_id}/{parameter_id}/{profile_key}"
             ][:]
-        elif profile_key in f[f'/profiling/{profile_id}/{parameter_id}'].attrs:
+        elif profile_key in f[f"/profiling/{profile_id}/{parameter_id}"].attrs:
             result[profile_key] = f[
-                f'/profiling/{profile_id}/{parameter_id}'
+                f"/profiling/{profile_id}/{parameter_id}"
             ].attrs[profile_key]
     return result
 
 
 def read_hdf5_optimization(
-    f: h5py.File,
-    file_name: str,
-    opt_id: str,
-) -> 'OptimizerResult':
+    f: h5py.File, file_name: Union[Path, str], opt_id: str, lazy: bool = False
+) -> "OptimizerResult":
     """Read HDF5 results per start.
 
     Parameters
@@ -61,23 +61,27 @@ def read_hdf5_optimization(
         The name of the HDF5 file, needed to create HDF5History
     opt_id:
         Specifies the start that is read from the HDF5 file
+    lazy:
+        Whether to use lazy loading for optimizer results
     """
-    result = OptimizerResult()
+    if lazy:
+        from ..result import LazyOptimizerResult
 
+        return LazyOptimizerResult(file_name, f"optimization/results/{opt_id}")
+
+    group = f[f"/optimization/results/{opt_id}"]
+    dset_ids = set(group)
+    attr_ids = set(group.attrs)
+
+    result = OptimizerResult()
     for optimization_key in result.keys():
-        if optimization_key == 'history':
-            if optimization_key in f:
-                result['history'] = Hdf5History(id=opt_id, file=file_name)
-                result['history'].recover_options(file_name)
-                continue
-        if optimization_key in f[f'/optimization/results/{opt_id}']:
-            result[optimization_key] = f[
-                f'/optimization/results/{opt_id}/{optimization_key}'
-            ][:]
-        elif optimization_key in f[f'/optimization/results/{opt_id}'].attrs:
-            result[optimization_key] = f[
-                f'/optimization/results/{opt_id}'
-            ].attrs[optimization_key]
+        if optimization_key == "history" and optimization_key in f:
+            result["history"] = Hdf5History(id=opt_id, file=file_name)
+            result["history"].recover_options(file_name)
+        elif optimization_key in dset_ids:
+            result[optimization_key] = group[optimization_key][:]
+        elif optimization_key in attr_ids:
+            result[optimization_key] = group.attrs[optimization_key]
     return result
 
 
@@ -91,12 +95,12 @@ class ProblemHDF5Reader:
         HDF5 problem file name
     """
 
-    def __init__(self, storage_filename: str):
+    def __init__(self, storage_filename: Union[str, Path]):
         """Initialize reader.
 
         Parameters
         ----------
-        storage_filename: str
+        storage_filename:
             HDF5 problem file name
         """
         self.storage_filename = storage_filename
@@ -118,27 +122,27 @@ class ProblemHDF5Reader:
         if objective is None:
             objective = Objective()
             # raise warning that objective is not loaded.
-            logger.info(
-                'WARNING: You are loading a problem.\nThis problem'
-                ' is not to be used without a separately created'
-                ' objective.'
+            logger.debug(
+                "You are loading a problem. This problem is not to be used "
+                "without a separately created objective."
             )
         problem = Problem(objective, [], [])
 
-        with h5py.File(self.storage_filename, 'r') as f:
-            for problem_key in f['/problem']:
-                if problem_key == 'config':
+        with h5py.File(self.storage_filename, "r") as f:
+            for problem_key in f["/problem"]:
+                if problem_key == "config":
                     continue
-                setattr(problem, problem_key, f[f'/problem/{problem_key}'][:])
-            for problem_attr in f['/problem'].attrs:
+                setattr(problem, problem_key, f[f"/problem/{problem_key}"][:])
+            for problem_attr in f["/problem"].attrs:
                 setattr(
-                    problem, problem_attr, f['/problem'].attrs[problem_attr]
+                    problem, problem_attr, f["/problem"].attrs[problem_attr]
                 )
 
         # h5 uses numpy for everything; convert to lists where necessary
         problem.x_fixed_vals = [float(val) for val in problem.x_fixed_vals]
         problem.x_fixed_indices = [int(ix) for ix in problem.x_fixed_indices]
         problem.x_names = [name.decode() for name in problem.x_names]
+        problem.x_scales = [scale.decode() for scale in problem.x_scales]
 
         return problem
 
@@ -151,29 +155,35 @@ class OptimizationResultHDF5Reader:
     ----------
     storage_filename:
         HDF5 result file name
+    lazy:
+        Whether to use lazy loading for optimizer results
     """
 
-    def __init__(self, storage_filename: str):
+    def __init__(self, storage_filename: Union[str, Path], lazy: bool = False):
         """
         Initialize reader.
 
         Parameters
         ----------
-        storage_filename: str
+        storage_filename:
             HDF5 result file name
+        lazy:
+            Whether to use lazy loading for optimizer results
         """
         self.storage_filename = storage_filename
         self.results = Result()
+        self.lazy = lazy
 
     def read(self) -> Result:
         """Read HDF5 result file and return pyPESTO result object."""
         with h5py.File(self.storage_filename, "r") as f:
-            for opt_id in f['/optimization/results']:
-                result = read_hdf5_optimization(
-                    f, self.storage_filename, opt_id
+            results = [
+                read_hdf5_optimization(
+                    f, self.storage_filename, opt_id, lazy=self.lazy
                 )
-                self.results.optimize_result.append(result)
-            self.results.optimize_result.sort()
+                for opt_id in f["/optimization/results"]
+            ]
+            self.results.optimize_result.append(results, sort=True)
         return self.results
 
 
@@ -187,12 +197,12 @@ class SamplingResultHDF5Reader:
         HDF5 result file name
     """
 
-    def __init__(self, storage_filename: str):
+    def __init__(self, storage_filename: Union[str, Path]):
         """Initialize reader.
 
         Parameters
         ----------
-        storage_filename: str
+        storage_filename:
             HDF5 result file name
         """
         self.storage_filename = storage_filename
@@ -202,15 +212,15 @@ class SamplingResultHDF5Reader:
         """Read HDF5 result file and return pyPESTO result object."""
         sample_result = {}
         with h5py.File(self.storage_filename, "r") as f:
-            for key in f['/sampling/results']:
-                sample_result[key] = f[f'/sampling/results/{key}'][:]
-            for key in f['/sampling/results'].attrs:
-                sample_result[key] = f['/sampling/results'].attrs[key]
+            for key in f["/sampling/results"]:
+                sample_result[key] = f[f"/sampling/results/{key}"][:]
+            for key in f["/sampling/results"].attrs:
+                sample_result[key] = f["/sampling/results"].attrs[key]
         try:
             self.results.sample_result = McmcPtResult(**sample_result)
         except TypeError:
             logger.warning(
-                "Warning: You tried loading a non-existent " "sampling result."
+                "Warning: You tried loading a non-existent sampling result."
             )
 
         return self.results
@@ -226,7 +236,7 @@ class ProfileResultHDF5Reader:
         HDF5 result file name
     """
 
-    def __init__(self, storage_filename: str):
+    def __init__(self, storage_filename: Union[str, Path]):
         """
         Initialize reader.
 
@@ -242,30 +252,31 @@ class ProfileResultHDF5Reader:
         """Read HDF5 result file and return pyPESTO result object."""
         profiling_list = []
         with h5py.File(self.storage_filename, "r") as f:
-            for profile_id in f['/profiling']:
+            for profile_id in f["/profiling"]:
                 profiling_list.append(
-                    [None for _ in f[f'/profiling/{profile_id}']]
+                    [None for _ in f[f"/profiling/{profile_id}"]]
                 )
-                for parameter_id in f[f'/profiling/{profile_id}']:
-                    if f[f'/profiling/{profile_id}/' f'{parameter_id}'].attrs[
-                        'IsNone'
+                for parameter_id in f[f"/profiling/{profile_id}"]:
+                    if f[f"/profiling/{profile_id}/{parameter_id}"].attrs[
+                        "IsNone"
                     ]:
                         continue
-                    profiling_list[int(profile_id)][
-                        int(parameter_id)
-                    ] = read_hdf5_profile(
-                        f, profile_id=profile_id, parameter_id=parameter_id
+                    profiling_list[int(profile_id)][int(parameter_id)] = (
+                        read_hdf5_profile(
+                            f, profile_id=profile_id, parameter_id=parameter_id
+                        )
                     )
             self.results.profile_result.list = profiling_list
         return self.results
 
 
 def read_result(
-    filename: str,
+    filename: Union[Path, str],
     problem: bool = True,
     optimize: bool = False,
     profile: bool = False,
     sample: bool = False,
+    lazy: bool = False,
 ) -> Result:
     """Save the whole pypesto.Result object in an HDF5 file.
 
@@ -284,6 +295,8 @@ def read_result(
         Read the profile result.
     sample:
         Read the sample result.
+    lazy:
+        Whether to use lazy loading for optimizer results
 
     Returns
     -------
@@ -301,15 +314,15 @@ def read_result(
         result.problem = pypesto_problem_reader.read()
 
     if optimize:
-        pypesto_opt_reader = OptimizationResultHDF5Reader(filename)
+        pypesto_opt_reader = OptimizationResultHDF5Reader(filename, lazy=lazy)
         try:
             temp_result = pypesto_opt_reader.read()
             result.optimize_result = temp_result.optimize_result
         except KeyError:
             logger.warning(
-                'Loading the optimization result failed. It is '
-                'highly likely that no optimization result exists '
-                f'within {filename}.'
+                "Loading the optimization result failed. It is "
+                "highly likely that no optimization result exists "
+                f"within {filename}."
             )
 
     if profile:
@@ -319,9 +332,9 @@ def read_result(
             result.profile_result = temp_result.profile_result
         except KeyError:
             logger.warning(
-                'Loading the profiling result failed. It is '
-                'highly likely that no profiling result exists '
-                f'within {filename}.'
+                "Loading the profiling result failed. It is "
+                "highly likely that no profiling result exists "
+                f"within {filename}."
             )
 
     if sample:
@@ -331,15 +344,15 @@ def read_result(
             result.sample_result = temp_result.sample_result
         except KeyError:
             logger.warning(
-                'Loading the sampling result failed. It is '
-                'highly likely that no sampling result exists '
-                f'within {filename}.'
+                "Loading the sampling result failed. It is "
+                "highly likely that no sampling result exists "
+                f"within {filename}."
             )
 
     return result
 
 
-def load_objective_config(filename: str):
+def load_objective_config(filename: Union[str, Path]):
     """Load the objective information stored in f.
 
     Parameters
@@ -352,7 +365,7 @@ def load_objective_config(filename: str):
         A dictionary of the information, stored instead of the
         actual objective in problem.objective.
     """
-    with h5py.File(filename, 'r') as f:
-        info_str = f['problem/config'][()].decode()
+    with h5py.File(filename, "r") as f:
+        info_str = f["problem/config"][()].decode()
         info = ast.literal_eval(info_str)
         return info
