@@ -1,12 +1,17 @@
+from __future__ import annotations
+
+import logging
 import warnings
+from collections.abc import Iterable
 from numbers import Number
-from typing import Iterable, List, Optional, Union
+from typing import TYPE_CHECKING
 
 import numpy as np
 
 from ..C import (
     ALL,
     ALL_CLUSTERED,
+    COLOR,
     FIRST_CLUSTER,
     FREE_ONLY,
     LEN_RGB,
@@ -22,34 +27,41 @@ from ..result import Result
 from ..util import assign_clusters, delete_nan_inf
 from .clust_color import assign_colors_for_list
 
+if TYPE_CHECKING:
+    from matplotlib.pyplot import Axes
+
+logger = logging.getLogger(__name__)
+
 
 def process_result_list(
-    results: Union[Result, List[Result]], colors=None, legends=None
-):
+    results: Result | list[Result],
+    colors: COLOR | list[COLOR] | np.ndarray | None = None,
+    legends: str | list[str] | None = None,
+) -> tuple[list[Result], list[COLOR], list[str]]:
     """
     Assign colors and legends to a list of results, check user provided lists.
 
     Parameters
     ----------
-    results: list or pypesto.Result
+    results:
         list of pypesto.Result objects or a single pypesto.Result
-    colors: list, optional
-        list of RGBA colors
-    legends: str or list
+    colors:
+        list of colors recognized by matplotlib, or single color
+    legends:
         labels for line plots
 
     Returns
     -------
-    results: list of pypesto.Result
+    results:
        list of pypesto.Result objects
-    colors: list of RGBA
+    colors:
         One for each element in 'results'.
-    legends: list of str
+    legends:
         labels for line plots
     """
     # check how many results were passed
     single_result = False
-    legend_error = False
+    legend_type_error = False
     if isinstance(results, list):
         if len(results) == 1:
             single_result = True
@@ -60,14 +72,18 @@ def process_result_list(
     # handle results according to their number
     if single_result:
         # assign colors and create list for later handling
-        if colors is None:
-            colors = [colors]
-        else:
+        if colors is not None and isinstance(colors, list):
             colors = [np.array(colors)]
+        else:
+            colors = [colors]
 
         # create list of legends for later handling
         if not isinstance(legends, list):
             legends = [legends]
+        try:
+            str(legends[0])
+        except TypeError:
+            legend_type_error = True
     else:
         # if more than one result is passed, we use one color per result
         colors = assign_colors_for_list(len(results), colors)
@@ -77,27 +93,28 @@ def process_result_list(
             # No legends were passed: create some custom legends
             legends = []
             for i_leg in range(len(results)):
-                legends.append('Result ' + str(i_leg))
+                legends.append("Result " + str(i_leg))
         else:
             # legends were passed by user: check length
-            if isinstance(legends, list):
+            try:
+                if isinstance(legends, str):
+                    legends = [legends]
                 if len(legends) != len(results):
-                    legend_error = True
-            else:
-                legend_error = True
+                    raise ValueError(
+                        "List of results passed and list of labels do "
+                        "not have the same length."
+                    )
+            except TypeError:
+                legend_type_error = True
 
-    # size of legend list and size of results does not match
-    if legend_error:
-        raise ValueError(
-            'List of results passed and list of labels do '
-            'not have the same length but should. Stopping.'
-        )
+    if legend_type_error:
+        raise TypeError("Unexpected legend type.")
 
     return results, colors, legends
 
 
 def process_offset_y(
-    offset_y: Optional[float], scale_y: str, min_val: float
+    offset_y: float | None, scale_y: str, min_val: float
 ) -> float:
     """
     Compute offset for y-axis, depend on user settings.
@@ -121,38 +138,42 @@ def process_offset_y(
     """
     # check whether the offset specified by the user is sufficient
     if offset_y is not None:
-        if (scale_y == 'log10') and (min_val + offset_y <= 0.0):
+        if (scale_y == "log10") and (min_val + offset_y <= 0.0):
             warnings.warn(
                 "Offset specified by user is insufficient. "
                 "Ignoring specified offset and using "
                 + str(np.abs(min_val) + 1.0)
-                + " instead."
+                + " instead.",
+                stacklevel=2,
             )
         else:
             return offset_y
     else:
         # check whether scaling is lin or log10
-        if scale_y == 'lin':
+        if scale_y == "lin":
             # linear scaling doesn't need any offset
             return 0.0
 
     return 1.0 - min_val
 
 
-def process_y_limits(ax, y_limits):
+def process_y_limits(
+    ax: Axes,
+    y_limits: None | Iterable[float] | np.ndarray,
+) -> Axes:
     """
     Apply user specified limits of y-axis.
 
     Parameters
     ----------
-    ax: matplotlib.Axes, optional
+    ax:
         Axes object to use.
-    y_limits: ndarray
+    y_limits:
        y_limits, minimum and maximum, for current axes object
 
     Returns
     -------
-    ax: matplotlib.Axes, optional
+    ax:
         Axes object to use.
     """
     # apply y-limits, if they were specified by the user
@@ -170,23 +191,25 @@ def process_y_limits(ax, y_limits):
             y_limits = [y_limits[0], y_limits[1]]
 
         # check validity of bounds if plotting in log-scale
-        if ax.get_yscale() == 'log' and y_limits[0] <= 0.0:
+        if ax.get_yscale() == "log" and y_limits[0] <= 0.0:
             tmp_y_limits = ax.get_ylim()
             if y_limits[1] <= 0.0:
                 y_limits = tmp_y_limits
                 warnings.warn(
                     "Invalid bounds for plotting in "
-                    "log-scale. Using defaults bounds."
+                    "log-scale. Using defaults bounds.",
+                    stacklevel=2,
                 )
             else:
                 y_limits = [tmp_y_limits[0], y_limits[1]]
                 warnings.warn(
                     "Invalid lower bound for plotting in "
-                    "log-scale. Using only upper bound."
+                    "log-scale. Using only upper bound.",
+                    stacklevel=2,
                 )
 
-            # set limits
-            ax.set_ylim(y_limits)
+        # set limits
+        ax.set_ylim(y_limits)
 
     else:
         # No limits passed, but if we have a result list: check the limits
@@ -194,10 +217,12 @@ def process_y_limits(ax, y_limits):
         data_limits = ax.dataLim.ymin, ax.dataLim.ymax
 
         # Check if data fits to axes and adapt limits, if necessary
-        if ax_limits[0] > data_limits[0] or ax_limits[1] < data_limits[1]:
+        if np.isfinite(data_limits).all() and (
+            ax_limits[0] > data_limits[0] or ax_limits[1] < data_limits[1]
+        ):
             # Get range of data
             data_range = data_limits[1] - data_limits[0]
-            if ax.get_yscale() == 'log':
+            if ax.get_yscale() == "log":
                 data_range = np.log10(data_range)
                 new_limits = (
                     np.power(10, np.log10(data_limits[0]) - 0.02 * data_range),
@@ -238,7 +263,7 @@ def rgba2rgb(fg: RGB_RGBA, bg: RGB_RGBA = None) -> RGB:
     else:
         if len(bg) != LEN_RGB:
             raise IndexError(
-                'A background color of unexpected length was provided: {bg}'
+                "A background color of unexpected length was provided: {bg}"
             )
         bg = (*bg, RGBA_MAX)
 
@@ -247,7 +272,7 @@ def rgba2rgb(fg: RGB_RGBA, bg: RGB_RGBA = None) -> RGB:
         return fg
     if len(fg) != LEN_RGBA:
         raise IndexError(
-            'A foreground color of unexpected length was provided: {fg}'
+            "A foreground color of unexpected length was provided: {fg}"
         )
 
     def apparent_composite_color_component(
@@ -289,13 +314,13 @@ def rgba2rgb(fg: RGB_RGBA, bg: RGB_RGBA = None) -> RGB:
 
 def process_start_indices(
     result: Result,
-    start_indices: Union[str, int, Iterable[int]] = None,
+    start_indices: str | int | Iterable[int] = None,
 ) -> np.ndarray:
     """
     Process the start_indices.
 
-    Create an array of indices if a number was provided and checks that the
-    indices do not exceed the max_index.
+    Create an array of indices if a number was provided, checks that the indices
+    do not exceed the max_index and removes starts with non-finite fval.
 
     Parameters
     ----------
@@ -314,7 +339,7 @@ def process_start_indices(
         start_indices = ALL
     if isinstance(start_indices, str):
         if start_indices == ALL:
-            return np.asarray(range(len(result.optimize_result)))
+            start_indices = np.asarray(range(len(result.optimize_result)))
         elif start_indices == ALL_CLUSTERED:
             clust_ind, clust_size = assign_clusters(
                 delete_nan_inf(result.optimize_result.fval)[1]
@@ -327,17 +352,17 @@ def process_start_indices(
             start_indices = np.concatenate(
                 [np.where(clust_ind == i_clust)[0] for i_clust in clust_gr2]
             )
-            return start_indices
+            start_indices = start_indices
         elif start_indices == FIRST_CLUSTER:
             clust_ind = assign_clusters(
                 delete_nan_inf(result.optimize_result.fval)[1]
             )[0]
-            return np.where(clust_ind == 0)[0]
+            start_indices = np.where(clust_ind == 0)[0]
         else:
             raise ValueError(
                 f"Permissible values for start_indices are {ALL}, "
                 f"{ALL_CLUSTERED}, {FIRST_CLUSTER}, an integer or a "
-                f"list of indices."
+                f"list of indices. Got {start_indices}."
             )
     # if it is an integer n, select the first n starts
     if isinstance(start_indices, Number):
@@ -350,12 +375,24 @@ def process_start_indices(
         if start_index < len(result.optimize_result)
     ]
 
-    return np.asarray(start_indices)
+    # filter out the indices that are not finite
+    start_indices_unfiltered = len(start_indices)
+    start_indices = [
+        start_index
+        for start_index in start_indices
+        if np.isfinite(result.optimize_result[start_index].fval)
+    ]
+    if len(start_indices) != start_indices_unfiltered:
+        logger.warning(
+            "Some start indices were removed due to inf or nan function values."
+        )
+
+    return np.asarray(start_indices, dtype=int)
 
 
 def process_parameter_indices(
     result: Result,
-    parameter_indices: Union[str, Iterable[int]] = FREE_ONLY,
+    parameter_indices: str | Iterable[int] = FREE_ONLY,
 ) -> list:
     """
     Process the parameter indices, always returning a valid array.
