@@ -1,12 +1,13 @@
-from numbers import Real
-from typing import Dict, Iterable, List, Optional, Sequence, Tuple, Union
+from collections.abc import Iterable, Sequence
 
 import matplotlib.axes
 import matplotlib.pyplot as plt
 import numpy as np
+from matplotlib.colors import is_color_like
 
 from pypesto.util import delete_nan_inf
 
+from ..C import COLOR
 from ..result import Result
 from .clust_color import assign_colors, assign_colors_for_list
 from .misc import process_result_list, process_start_indices
@@ -14,12 +15,12 @@ from .misc import process_result_list, process_start_indices
 
 def optimization_run_properties_one_plot(
     results: Result,
-    properties_to_plot: Optional[List[str]] = None,
-    size: Tuple[float, float] = (18.5, 10.5),
-    start_indices: Optional[Union[int, Iterable[int]]] = None,
-    colors: Optional[Union[List[float], List[List[float]]]] = None,
-    legends: Optional[Union[str, List[str]]] = None,
-    plot_type: str = 'line',
+    properties_to_plot: list[str] | None = None,
+    size: tuple[float, float] = (18.5, 10.5),
+    start_indices: int | Iterable[int] | None = None,
+    colors: COLOR | list[COLOR] | np.ndarray | None = None,
+    legends: str | list[str] | None = None,
+    plot_type: str = "line",
 ) -> matplotlib.axes.Axes:
     """
     Plot stats for allproperties specified in properties_to_plot on one plot.
@@ -37,8 +38,8 @@ def optimization_run_properties_one_plot(
         List of integers specifying the multistarts to be plotted or
         int specifying up to which start index should be plotted
     colors:
-        List of RGBA colors (one color per property in properties_to_plot),
-        or single RGBA color. If not set and one result, clustering is done
+        List of colors recognized by matplotlib colors (one color per property in properties_to_plot),
+        or single color. If not set and one result, clustering is done
         and colors are assigned automatically
     legends:
         Labels, one label per optimization property
@@ -52,35 +53,39 @@ def optimization_run_properties_one_plot(
 
     Examples
     --------
-    optimization_properties_per_multistart(
-        result1,
-        properties_to_plot=['time'],
-        colors=[.5, .9, .9, .3])
+    .. code-block:: python
 
-    optimization_properties_per_multistart(
-        result1,
-        properties_to_plot=['time', 'n_grad'],
-        colors=[[.5, .9, .9, .3], [.2, .1, .9, .5]])
+        optimization_run_properties_one_plot(
+            result1,
+            properties_to_plot=['time'],
+            colors=[.5, .9, .9, .3]
+        )
+
+        optimization_run_properties_one_plot(
+            result1,
+            properties_to_plot=['time', 'n_grad'],
+            colors=[[.5, .9, .9, .3], [.2, .1, .9, .5]]
+        )
     """
     if properties_to_plot is None:
         properties_to_plot = [
-            'time',
-            'n_fval',
-            'n_grad',
-            'n_hess',
-            'n_res',
-            'n_sres',
+            "time",
+            "n_fval",
+            "n_grad",
+            "n_hess",
+            "n_res",
+            "n_sres",
         ]
 
     if colors is None:
         colors = assign_colors_for_list(len(properties_to_plot))
-    elif len(colors) == 4 and isinstance(colors[0], Real):
+    elif is_color_like(colors):
         colors = [colors]
 
     if len(colors) != len(properties_to_plot):
         raise ValueError(
-            'Number of RGBA colors should be the same as number '
-            'of optimization properties to plot'
+            "Number of colors should be the same as number "
+            "of optimization properties to plot"
         )
 
     if legends is None:
@@ -90,8 +95,8 @@ def optimization_run_properties_one_plot(
 
     if len(legends) != len(properties_to_plot):
         raise ValueError(
-            'Number of legends should be the same as number of '
-            'optimization properties to plot'
+            "Number of legends should be the same as number of "
+            "optimization properties to plot"
         )
 
     ax = plt.subplots()[1]
@@ -116,14 +121,14 @@ def optimization_run_properties_one_plot(
 
 
 def optimization_run_properties_per_multistart(
-    results: Union[Result, Sequence[Result]],
-    properties_to_plot: Optional[List[str]] = None,
-    size: Tuple[float, float] = (18.5, 10.5),
-    start_indices: Optional[Union[int, Iterable[int]]] = None,
-    colors: Optional[Union[List[float], List[List[float]]]] = None,
-    legends: Optional[Union[str, List[str]]] = None,
-    plot_type: str = 'line',
-) -> Dict[str, plt.Subplot]:
+    results: Result | Sequence[Result],
+    properties_to_plot: list[str] | None = None,
+    size: tuple[float, float] = (18.5, 10.5),
+    start_indices: int | Iterable[int] | None = None,
+    colors: COLOR | list[COLOR] | np.ndarray | None = None,
+    legends: str | list[str] | None = None,
+    plot_type: str = "line",
+) -> dict[str, plt.Subplot]:
     """
     One plot per optimization property in properties_to_plot.
 
@@ -140,8 +145,8 @@ def optimization_run_properties_per_multistart(
         List of integers specifying the multistarts to be plotted or
         int specifying up to which start index should be plotted
     colors:
-        List of RGBA colors (one color per result in results),
-        or single RGBA color. If not set and one result, clustering is done
+        List of colors recognized by matplotlib (one color per result in results),
+        or single color. If not set and one result, clustering is done
         and colors are assigned automatically
     legends:
         Labels for line plots, one label per result object
@@ -155,33 +160,39 @@ def optimization_run_properties_per_multistart(
 
     Examples
     --------
-    optimization_properties_per_multistart(
-        result1,
-        properties_to_plot=['time'],
-        colors=[.5, .9, .9, .3])
+    .. code-block:: python
 
-    optimization_properties_per_multistart(
-        [result1, result2],
-        properties_to_plot=['time'],
-        colors=[[.5, .9, .9, .3], [.2, .1, .9, .5]])
+        optimization_run_properties_per_multistart(
+            result1,
+            properties_to_plot=['time'],
+            colors=[.5, .9, .9, .3]
+        )
 
-    optimization_properties_per_multistart(
-        result1,
-        properties_to_plot=['time', 'n_grad'],
-        colors=[.5, .9, .9, .3])
+        optimization_run_properties_per_multistart(
+            [result1, result2],
+            properties_to_plot=['time'],
+            colors=[[.5, .9, .9, .3], [.2, .1, .9, .5]]
+        )
 
-    optimization_properties_per_multistart(
-        [result1, result2], properties_to_plot=['time', 'n_fval'],
-        colors=[[.5, .9, .9, .3], [.2, .1, .9, .5]])
+        optimization_run_properties_per_multistart(
+            result1,
+            properties_to_plot=['time', 'n_grad'],
+            colors=[.5, .9, .9, .3]
+        )
+
+        optimization_run_properties_per_multistart(
+            [result1, result2], properties_to_plot=['time', 'n_fval'],
+            colors=[[.5, .9, .9, .3], [.2, .1, .9, .5]]
+        )
     """
     if properties_to_plot is None:
         properties_to_plot = [
-            'time',
-            'n_fval',
-            'n_grad',
-            'n_hess',
-            'n_res',
-            'n_sres',
+            "time",
+            "n_fval",
+            "n_grad",
+            "n_hess",
+            "n_res",
+            "n_sres",
         ]
 
     num_subplot = len(properties_to_plot)
@@ -193,7 +204,7 @@ def optimization_run_properties_per_multistart(
 
     for ax in axes.flat[num_subplot:]:
         ax.remove()
-    axes = dict(zip(range(num_subplot), axes.flat))
+    axes = dict(zip(range(num_subplot), axes.flat, strict=True))
     for idx, prop_name in enumerate(properties_to_plot):
         ax = axes[idx]
         optimization_run_property_per_multistart(
@@ -210,21 +221,21 @@ def optimization_run_properties_per_multistart(
 
 
 def optimization_run_property_per_multistart(
-    results: Union[Result, Sequence[Result]],
+    results: Result | Sequence[Result],
     opt_run_property: str,
-    axes: Optional[matplotlib.axes.Axes] = None,
-    size: Tuple[float, float] = (18.5, 10.5),
-    start_indices: Optional[Union[int, Iterable[int]]] = None,
-    colors: Optional[Union[List[float], List[List[float]]]] = None,
-    legends: Optional[Union[str, List[str]]] = None,
-    plot_type: str = 'line',
+    axes: matplotlib.axes.Axes | None = None,
+    size: tuple[float, float] = (18.5, 10.5),
+    start_indices: int | Iterable[int] | None = None,
+    colors: COLOR | list[COLOR] | np.ndarray | None = None,
+    legends: str | list[str] | None = None,
+    plot_type: str = "line",
 ) -> matplotlib.axes.Axes:
     """
     Plot stats for an optimization run property specified by opt_run_property.
 
     It is possible to plot a histogram or a line plot. In a line plot,
-    on the x axis are the numbers of the multistarts, where the multistarts are
-    ordered with respect to a function value. On the y axis of the line plot
+    on the x-axis are the numbers of the multistarts, where the multistarts are
+    ordered with respect to a function value. On the y-axis of the line plot
     the value of the corresponding parameter for each multistart is displayed.
 
     Parameters
@@ -243,8 +254,8 @@ def optimization_run_property_per_multistart(
         List of integers specifying the multistarts to be plotted or
         int specifying up to which start index should be plotted
     colors:
-        List of RGBA colors (one color per result in results),
-        or single RGBA color. If not set and one result, clustering is done
+        List of colors recognized by matplotlib (one color per result in results),
+        or single color. If not set and one result, clustering is done
         and colors are assigned automatically
     legends:
         Labels for line plots, one label per result object
@@ -257,12 +268,12 @@ def optimization_run_property_per_multistart(
         The plot axes.
     """
     supported_properties = {
-        'time': 'Wall-clock time (seconds)',
-        'n_fval': 'Number of function evaluations',
-        'n_grad': 'Number of gradient evaluations',
-        'n_hess': 'Number of Hessian evaluations',
-        'n_res': 'Number of residuals evaluations',
-        'n_sres': 'Number of residual sensitivity evaluations',
+        "time": "Wall-clock time (seconds)",
+        "n_fval": "Number of function evaluations",
+        "n_grad": "Number of gradient evaluations",
+        "n_hess": "Number of Hessian evaluations",
+        "n_res": "Number of residuals evaluations",
+        "n_sres": "Number of residual sensitivity evaluations",
     }
 
     if opt_run_property not in supported_properties:
@@ -277,20 +288,20 @@ def optimization_run_property_per_multistart(
 
     # axes
     if axes is None:
-        ncols = 2 if plot_type == 'both' else 1
+        ncols = 2 if plot_type == "both" else 1
         fig, axes = plt.subplots(1, ncols)
         fig.set_size_inches(*size)
         fig.suptitle(
-            f'{supported_properties[opt_run_property]} per optimizer run'
+            f"{supported_properties[opt_run_property]} per optimizer run"
         )
     else:
         axes.set_title(
-            f'{supported_properties[opt_run_property]} per optimizer run'
+            f"{supported_properties[opt_run_property]} per optimizer run"
         )
 
     # loop over results
     for j, result in enumerate(results):
-        if plot_type == 'both':
+        if plot_type == "both":
             axes[0] = stats_lowlevel(
                 result,
                 opt_run_property,
@@ -309,7 +320,7 @@ def optimization_run_property_per_multistart(
                 start_indices,
                 colors[j],
                 legends[j],
-                plot_type='hist',
+                plot_type="hist",
             )
         else:
             axes = stats_lowlevel(
@@ -323,8 +334,8 @@ def optimization_run_property_per_multistart(
                 plot_type,
             )
 
-    if sum((legend is not None for legend in legends)) > 0:
-        if plot_type == 'both':
+    if sum(legend is not None for legend in legends) > 0:
+        if plot_type == "both":
             for ax in axes:
                 ax.legend()
         else:
@@ -338,10 +349,10 @@ def stats_lowlevel(
     property_name: str,
     axis_label: str,
     ax: matplotlib.axes.Axes,
-    start_indices: Optional[Union[int, Iterable[int]]] = None,
-    color: Union[str, List[float], List[List[float]]] = 'C0',
-    legend: Optional[str] = None,
-    plot_type: str = 'line',
+    start_indices: int | Iterable[int] | None = None,
+    color: COLOR | list[COLOR] | np.ndarray | None = "C0",
+    legend: str | None = None,
+    plot_type: str = "line",
 ):
     """
     Plot values of the optimization run property across different multistarts.
@@ -353,15 +364,15 @@ def stats_lowlevel(
     property_name:
         name of the optimization result property which value should be plotted
     axis_label:
-        Label for the y axis of the line plot or x axis of the histogram
+        Label for the y-axis of the line plot or x-axis of the histogram
     ax:
         Axes object to use
     start_indices:
         List of integers specifying the multistarts to be plotted or
         int specifying up to which start index should be plotted
     color:
-        List of RGBA colors (length equal to the number of multistarts),
-        or single color, defined by a string or RGBA list
+        List of colors recognized by matplotlib (length equal to the number of multistarts),
+        or single color
         If not set, then for the line plot clustering is done and
         colors are assigned automatically
     legend:
@@ -375,7 +386,7 @@ def stats_lowlevel(
         The plot axes.
     """
     fvals = result.optimize_result.fval
-    values = [res[property_name] for res in result.optimize_result.list]
+    values = [[res[property_name]] for res in result.optimize_result.list]
     values, fvals = delete_nan_inf(fvals, values)
 
     if start_indices is not None:
@@ -388,11 +399,10 @@ def stats_lowlevel(
     # assign colors
     colors = assign_colors(vals=fvals, colors=color, balance_alpha=False)
 
-    # sort TODO: issue # 378
     sorted_indices = sorted(range(n_starts), key=lambda j: fvals[j])
     values = values[sorted_indices]
 
-    if plot_type == 'line':
+    if plot_type == "line":
         # plot line
         ax.plot(range(n_starts), values, color=[0.7, 0.7, 0.7, 0.6])
 
@@ -402,12 +412,12 @@ def stats_lowlevel(
                 tmp_legend = legend
             else:
                 tmp_legend = None
-            ax.scatter(i, v, color=colors[i], marker='o', label=tmp_legend)
-        ax.set_xlabel('Ordered optimizer run')
+            ax.scatter(i, v, color=colors[i], marker="o", label=tmp_legend)
+        ax.set_xlabel("Ordered optimizer run")
         ax.set_ylabel(axis_label)
     else:
-        ax.hist(values, color=color, bins='auto', label=legend)
+        ax.hist(values, color=color, bins="auto", label=legend)
         ax.set_xlabel(axis_label)
-        ax.set_ylabel('Number of multistarts')
+        ax.set_ylabel("Number of multistarts")
 
     return ax
