@@ -1,4 +1,4 @@
-from typing import Optional, Tuple
+from typing import Optional
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -10,9 +10,9 @@ from ..result import Result
 def optimizer_convergence(
     result: Result,
     ax: Optional[plt.Axes] = None,
-    xscale: str = 'symlog',
-    yscale: str = 'log',
-    size: Tuple[float] = (18.5, 10.5),
+    xscale: str = "symlog",
+    yscale: str = "log",
+    size: tuple[float] = (18.5, 10.5),
 ) -> plt.Axes:
     """
     Visualize to help spotting convergence issues.
@@ -52,22 +52,24 @@ def optimizer_convergence(
 
     fvals = result.optimize_result.fval
     grad_norms = [
-        np.linalg.norm(
-            result.problem.get_reduced_vector(
-                grad, result.problem.x_free_indices
-            ),
-            2,
+        (
+            np.linalg.norm(
+                result.problem.get_reduced_vector(
+                    grad, result.problem.x_free_indices
+                ),
+                2,
+            )
+            if grad is not None
+            else np.nan
         )
-        if grad is not None
-        else np.NaN
         for grad in result.optimize_result.grad
     ]
     msgs = result.optimize_result.message
     conv_data = pd.DataFrame(
-        {'fval': fvals, 'gradient norm': grad_norms, 'exit message': msgs}
+        {"fval": fvals, "gradient norm": grad_norms, "exit message": msgs}
     )
     sns.scatterplot(
-        x='fval', y='gradient norm', hue='exit message', data=conv_data, ax=ax
+        x="fval", y="gradient norm", hue="exit message", data=conv_data, ax=ax
     )
     ax.set_yscale(yscale)
     ax.set_xscale(xscale)
