@@ -1,12 +1,13 @@
 import logging
-from typing import Iterable, List, Optional, Tuple, Union
+import warnings
+from collections.abc import Iterable
 
 import matplotlib.pyplot as plt
 import numpy as np
 from matplotlib.ticker import MaxNLocator
 
 from ..C import (
-    RGBA,
+    COLOR,
     TRACE_X_STEPS,
     TRACE_X_TIME,
     TRACE_Y_FVAL,
@@ -22,20 +23,22 @@ logger = logging.getLogger(__name__)
 
 
 def optimizer_history(
-    results: Union[Result, List[Result]],
-    ax: Optional[plt.Axes] = None,
-    size: Tuple = (18.5, 10.5),
+    results: Result | list[Result],
+    ax: plt.Axes | None = None,
+    size: tuple = (18.5, 10.5),
     trace_x: str = TRACE_X_STEPS,
     trace_y: str = TRACE_Y_FVAL,
-    scale_y: str = 'log10',
-    offset_y: Optional[float] = None,
-    colors: Optional[Union[RGBA, List[RGBA]]] = None,
-    y_limits: Optional[Union[float, List[float], np.ndarray]] = None,
-    start_indices: Optional[Union[int, List[int]]] = None,
-    reference: Optional[
-        Union[ReferencePoint, dict, List[ReferencePoint], List[dict]]
-    ] = None,
-    legends: Optional[Union[str, List[str]]] = None,
+    scale_y: str = "log10",
+    offset_y: float | None = None,
+    colors: COLOR | list[COLOR] | np.ndarray | None = None,
+    y_limits: float | list[float] | np.ndarray | None = None,
+    start_indices: int | list[int] | None = None,
+    reference: ReferencePoint
+    | dict
+    | list[ReferencePoint]
+    | list[dict]
+    | None = None,
+    legends: str | list[str] | None = None,
 ) -> plt.Axes:
     """
     Plot history of optimizer.
@@ -49,7 +52,7 @@ def optimizer_history(
         Optimization result obtained by 'optimize.py' or list of those
     ax:
         Axes object to use.
-    size: tuple, optional
+    size:
         Figure size (width, height) in inches. Is only applied when no ax
         object is specified
     trace_x:
@@ -65,9 +68,8 @@ def optimizer_history(
     offset_y:
         Offset for the y-axis-values, as these are plotted on a log10-scale
         Will be computed automatically if necessary
-    colors: list, or RGBA, optional
-        list of colors, or single color
-        color or list of colors for plotting. If not set, clustering is done
+    colors:
+        Color recognized by matplotlib or list of colors for plotting. If not set, clustering is done
         and colors are assigned automatically
     y_limits:
         maximum value to be plotted on the y-axis, or y-limits
@@ -124,14 +126,14 @@ def optimizer_history(
 
 
 def optimizer_history_lowlevel(
-    vals: List[np.ndarray],
-    scale_y: str = 'log10',
-    colors: Optional[Union[RGBA, List[RGBA]]] = None,
-    ax: Optional[plt.Axes] = None,
-    size: Tuple = (18.5, 10.5),
-    x_label: str = 'Optimizer steps',
-    y_label: str = 'Objective value',
-    legend_text: Optional[str] = None,
+    vals: list[np.ndarray],
+    scale_y: str = "log10",
+    colors: COLOR | list[COLOR] | np.ndarray | None = None,
+    ax: plt.Axes | None = None,
+    size: tuple = (18.5, 10.5),
+    x_label: str = "Optimizer steps",
+    y_label: str = "Objective value",
+    legend_text: str | None = None,
 ) -> plt.Axes:
     """
     Plot optimizer history using list of numpy arrays.
@@ -142,9 +144,8 @@ def optimizer_history_lowlevel(
         list of 2xn-arrays (x_values and y_values of the trace)
     scale_y:
         May be logarithmic or linear ('log10' or 'lin')
-    colors: list, or RGBA, optional
-        list of colors, or single color
-        color or list of colors for plotting. If not set, clustering is done
+    colors:
+        Color recognized by matplotlib or list of colors for plotting. If not set, clustering is done
         and colors are assigned automatically
     ax:
         Axes object to use.
@@ -181,9 +182,9 @@ def optimizer_history_lowlevel(
         vals = np.asarray(vals)
         if vals.shape[0] != 2 or vals.ndim != 2:
             raise ValueError(
-                'If numpy array is passed directly to lowlevel '
-                'routine of optimizer_history, shape needs to '
-                'be 2 x n.'
+                "If numpy array is passed directly to lowlevel "
+                "routine of optimizer_history, shape needs to "
+                "be 2 x n."
             )
         fvals = [vals[1, -1]]
         vals = [vals]
@@ -209,7 +210,7 @@ def optimizer_history_lowlevel(
             tmp_legend = None
 
         # line plots
-        if scale_y == 'log10':
+        if scale_y == "log10":
             ax.semilogy(val[0, :], val[1, :], color=color, label=tmp_legend)
         else:
             ax.plot(val[0, :], val[1, :], color=color, label=tmp_legend)
@@ -217,7 +218,7 @@ def optimizer_history_lowlevel(
     # set labels
     ax.set_xlabel(x_label)
     ax.set_ylabel(y_label)
-    ax.set_title('Optimizer history')
+    ax.set_title("Optimizer history")
     if legend_text is not None:
         ax.legend()
 
@@ -225,8 +226,8 @@ def optimizer_history_lowlevel(
 
 
 def get_trace(
-    result: Result, trace_x: Optional[str], trace_y: Optional[str]
-) -> List[np.ndarray]:
+    result: Result, trace_x: str | None, trace_y: str | None
+) -> list[np.ndarray]:
     """
     Get the values of the optimizer trace from the pypesto.Result object.
 
@@ -253,7 +254,7 @@ def get_trace(
         label for y-axis to be plotted later.
     """
     # get data frames
-    histories: List[HistoryBase] = result.optimize_result.history
+    histories: list[HistoryBase] = result.optimize_result.history
 
     vals = []
 
@@ -295,6 +296,10 @@ def get_trace(
         else:  # trace_x == TRACE_X_STEPS:
             x_vals = np.array(list(range(len(indices))))
 
+        # if the trace is empty, skip
+        if len(x_vals) == 0:
+            continue
+
         # write down values
         vals.append(np.vstack([x_vals, y_vals]))
 
@@ -302,12 +307,12 @@ def get_trace(
 
 
 def get_vals(
-    vals: List[np.ndarray],
-    scale_y: Optional[str],
+    vals: list[np.ndarray],
+    scale_y: str | None,
     offset_y: float,
     trace_y: str,
     start_indices: Iterable[int],
-) -> Tuple[List[np.ndarray], float]:
+) -> tuple[list[np.ndarray], float]:
     """
     Postprocess the values of the optimization history.
 
@@ -369,7 +374,7 @@ def get_vals(
     return vals, offset_y
 
 
-def get_labels(trace_x: str, trace_y: str, offset_y: float) -> Tuple[str, str]:
+def get_labels(trace_x: str, trace_y: str, offset_y: float) -> tuple[str, str]:
     """
     Generate labels for x and y axes of the history plot.
 
@@ -385,33 +390,32 @@ def get_labels(trace_x: str, trace_y: str, offset_y: float) -> Tuple[str, str]:
     Returns
     -------
     labels for x and y axes
-
     """
-    x_label = ''
-    y_label = ''
+    x_label = ""
+    y_label = ""
 
     if trace_x == TRACE_X_TIME:
-        x_label = 'Computation time [s]'
+        x_label = "Computation time [s]"
     else:
-        x_label = 'Optimizer steps'
+        x_label = "Optimizer steps"
 
     if trace_y == TRACE_Y_GRADNORM:
-        y_label = 'Gradient norm'
+        y_label = "Gradient norm"
     else:
-        y_label = 'Objective value'
+        y_label = "Objective value"
 
     if offset_y != 0:
-        y_label = 'Offsetted ' + y_label.lower()
+        y_label = "Offsetted " + y_label.lower()
 
     return x_label, y_label
 
 
 def handle_options(
     ax: plt.Axes,
-    vals: List[np.ndarray],
+    vals: list[np.ndarray],
     trace_y: str,
-    ref: List[ReferencePoint],
-    y_limits: Union[float, np.ndarray, None],
+    ref: list[ReferencePoint],
+    y_limits: float | np.ndarray | None,
     offset_y: float,
 ) -> plt.Axes:
     """
@@ -457,7 +461,7 @@ def handle_options(
                 ax.plot(
                     [0, max_len],
                     [i_ref.fval + offset_y, i_ref.fval + offset_y],
-                    '--',
+                    "--",
                     color=i_ref.color,
                     label=i_ref.legend,
                 )
@@ -467,8 +471,131 @@ def handle_options(
                     ax.legend()
     else:
         logger.warning(
-            f'Reference point is currently only implemented for trace_y == '
-            f'{TRACE_Y_FVAL} and will not be plotted for trace_y == {trace_y}.'
+            f"Reference point is currently only implemented for trace_y == "
+            f"{TRACE_Y_FVAL} and will not be plotted for trace_y == {trace_y}."
         )
 
     return ax
+
+
+def monotonic_history(
+    histories: list[HistoryBase],
+) -> tuple[np.ndarray, np.ndarray]:
+    """
+    Get the overall monotonically decreasing history from multiple histories.
+
+    Parameters
+    ----------
+    histories:
+        List of histories to merge.
+        The histories are expected to have time and function value traces,
+        and that the function values are monotonically decreasing within each
+        history.
+
+    Returns
+    -------
+    t:
+        Time points of overall history.
+    fx:
+        Objective values of overall history.
+    """
+    if len(histories) == 0:
+        warnings.warn("No histories to process.", stacklevel=2)
+        return np.array([], dtype="float"), np.array([], dtype="float")
+
+    for history in histories:
+        fvals = history.get_fval_trace()
+        if not np.all(fvals[:-1] >= fvals[1:]):
+            raise ValueError(
+                "Each history is expected to have monotonically "
+                "decreasing function values."
+            )
+
+    # merge results
+    t = np.hstack([history.get_time_trace() for history in histories])
+    fx = np.hstack([history.get_fval_trace() for history in histories])
+    time_order = np.argsort(t)
+    t = t[time_order]
+    fx = fx[time_order]
+
+    # get the monotonously decreasing sequence
+    monotone = np.where(fx == np.fmin.accumulate(fx))[0]
+    t_mono, fx_mono = t[monotone], fx[monotone]
+
+    # remove duplicates
+    # convert to np.recarray for multi-level sorting
+    # so that we can use np.unique later to remove duplicates
+    records = np.rec.fromarrays([t_mono, fx_mono], names="t,fx")
+    records.sort(order=["t", "fx"])
+    t_mono, unique_idx = np.unique(records.t, return_index=True, sorted=True)
+    fx_mono = records.fx[unique_idx]
+
+    # extend from last decrease to last timepoint
+    if t_mono[-1] != (t_max := t.max()):
+        t_mono = np.append(t_mono, [t_max])
+        fx_mono = np.append(fx_mono, [fx_mono.min()])
+
+    return t_mono, fx_mono
+
+
+def sacess_history(
+    histories: list[HistoryBase],
+    ax: plt.Axes | None = None,
+) -> plt.Axes:
+    """Plot `SacessOptimizer` history.
+
+    Plot the history of the best objective values for each
+    :class:`pypesto.optimize.ess.sacess.SacessOptimizer`
+    worker over computation time as step splot.
+
+    Parameters
+    ----------
+    histories:
+        List of histories from different workers as obtained from
+        :attr:`pypesto.optimize.ess.sacess.SacessOptimizer.histories`.
+    ax:
+        Axes object to use.
+
+    Returns
+    -------
+    The plot axes. `ax` or a new axes if `ax` was `None`.
+    """
+    ax = ax or plt.subplot()
+    if len(histories) == 0:
+        warnings.warn("No histories to plot.", stacklevel=2)
+
+    # plot overall minimum
+    t_overall, fx_overall = monotonic_history(histories)
+    ax.step(
+        t_overall,
+        fx_overall,
+        linestyle="dotted",
+        color="grey",
+        where="post",
+        label="overall",
+        alpha=0.8,
+    )
+
+    # plot steps of individual workers
+    for worker_idx, history in enumerate(histories):
+        x, y = history.get_time_trace(), history.get_fval_trace()
+        if len(x) == 0:
+            warnings.warn(f"No trace for worker #{worker_idx}.", stacklevel=2)
+            continue
+        # extend from last decrease to last timepoint
+        x = np.append(x, [np.max(t_overall)])
+        y = np.append(y, [np.min(y)])
+        lines = ax.step(
+            x, y, ".-", where="post", label=f"worker {worker_idx}", alpha=0.8
+        )
+        # Plot last point without marker, unless we actually had an improvement there.
+        # The time point of the overall last improvement is appended to all histories,
+        # even if redundant, so we can just skip the marker for the last point.
+        for line in lines:
+            line.set_markevery([True] * (len(x) - 1) + [False])
+
+    ax.legend()
+    ax.set_xlabel("time (s)")
+    ax.set_ylabel("fval")
+    ax.set_title("SacessOptimizer convergence")
+    return ax
