@@ -1,7 +1,8 @@
 import logging
 import warnings
+from collections.abc import Sequence
 from colorsys import rgb_to_hls
-from typing import Dict, Optional, Sequence, Tuple, Union
+from typing import Optional, Union
 
 import matplotlib.axes
 import matplotlib.pyplot as plt
@@ -32,9 +33,9 @@ logger = logging.getLogger(__name__)
 
 
 prediction_errorbar_settings = {
-    'fmt': 'none',
-    'color': 'k',
-    'capsize': 10,
+    "fmt": "none",
+    "color": "k",
+    "capsize": 10,
 }
 
 
@@ -44,7 +45,7 @@ def sampling_fval_traces(
     full_trace: bool = False,
     stepsize: int = 1,
     title: str = None,
-    size: Tuple[float, float] = None,
+    size: tuple[float, float] = None,
     ax: matplotlib.axes.Axes = None,
 ):
     """
@@ -87,14 +88,14 @@ def sampling_fval_traces(
         _, ax = plt.subplots(figsize=size)
 
     sns.set(style="ticks")
-    kwargs = {'edgecolor': "w", 'linewidth': 0.3, 's': 10}  # for edge color
+    kwargs = {"edgecolor": "w", "linewidth": 0.3, "s": 10}  # for edge color
     if full_trace:
-        kwargs['hue'] = "converged"
-        if len(params_fval[kwargs['hue']].unique()) == 1:
-            kwargs['palette'] = ["#477ccd"]
-        elif len(params_fval[kwargs['hue']].unique()) == 2:
-            kwargs['palette'] = ["#868686", "#477ccd"]
-        kwargs['legend'] = False
+        kwargs["hue"] = "converged"
+        if len(params_fval[kwargs["hue"]].unique()) == 1:
+            kwargs["palette"] = ["#477ccd"]
+        elif len(params_fval[kwargs["hue"]].unique()) == 2:
+            kwargs["palette"] = ["#868686", "#477ccd"]
+        kwargs["legend"] = False
 
     sns.scatterplot(
         x="iteration", y="logPosterior", data=params_fval, ax=ax, **kwargs
@@ -106,10 +107,10 @@ def sampling_fval_traces(
         _burn_in = result.sample_result.burn_in
 
     if full_trace and _burn_in > 0:
-        ax.axvline(_burn_in, linestyle='--', linewidth=1.5, color='k')
+        ax.axvline(_burn_in, linestyle="--", linewidth=1.5, color="k")
 
-    ax.set_xlabel('iteration index')
-    ax.set_ylabel('log-posterior')
+    ax.set_xlabel("iteration index")
+    ax.set_ylabel("log-posterior")
 
     if title:
         ax.set_title(title)
@@ -119,7 +120,7 @@ def sampling_fval_traces(
     return ax
 
 
-def _get_level_percentiles(level: float) -> Tuple[float, float]:
+def _get_level_percentiles(level: float) -> tuple[float, float]:
     """Convert a credibility level to percentiles.
 
     Similar to the highest-density region of a symmetric, unimodal distribution
@@ -147,11 +148,11 @@ def _get_level_percentiles(level: float) -> Tuple[float, float]:
 
 
 def _get_statistic_data(
-    summary: Dict[str, PredictionResult],
+    summary: dict[str, PredictionResult],
     statistic: str,
     condition_id: str,
     output_id: str,
-) -> Tuple[Sequence[float], Sequence[float]]:
+) -> tuple[Sequence[float], Sequence[float]]:
     """Get statistic-, condition-, and output-specific data.
 
     Parameters
@@ -182,18 +183,18 @@ def _get_statistic_data(
 
 
 def _plot_trajectories_by_condition(
-    summary: Dict[str, PredictionResult],
+    summary: dict[str, PredictionResult],
     condition_ids: Sequence[str],
     output_ids: Sequence[str],
     axes: matplotlib.axes.Axes,
     levels: Sequence[float],
-    level_opacities: Dict[int, float],
-    labels: Dict[str, str],
+    level_opacities: dict[int, float],
+    labels: dict[str, str],
     variable_colors: Sequence[RGB],
     average: str = MEDIAN,
     add_sd: bool = False,
-    grouped_measurements: Dict[
-        Tuple[str, str], Sequence[Sequence[float]]
+    grouped_measurements: dict[
+        tuple[str, str], Sequence[Sequence[float]]
     ] = None,
 ) -> None:
     """Plot predicted trajectories, with subplots grouped by condition.
@@ -237,7 +238,7 @@ def _plot_trajectories_by_condition(
     # Each subplot has all data for a single condition.
     for condition_index, condition_id in enumerate(condition_ids):
         ax = axes.flat[condition_index]
-        ax.set_title(f'Condition: {labels[condition_id]}')
+        ax.set_title(f"Condition: {labels[condition_id]}")
         # Each subplot has all data for all condition-specific outputs.
         for output_index, output_id in enumerate(output_ids):
             facecolor0 = variable_colors[output_index]
@@ -251,7 +252,7 @@ def _plot_trajectories_by_condition(
             ax.plot(
                 t_average,
                 y_average,
-                'k-',
+                "k-",
             )
             if add_sd:
                 t_std, y_std = _get_statistic_data(
@@ -262,8 +263,8 @@ def _plot_trajectories_by_condition(
                 )
                 if (t_std != t_average).all():
                     raise ValueError(
-                        'Unknown error: timepoints for average and standard '
-                        'deviation do not match.'
+                        "Unknown error: timepoints for average and standard "
+                        "deviation do not match."
                     )
                 ax.errorbar(
                     t_average,
@@ -276,10 +277,10 @@ def _plot_trajectories_by_condition(
             for level_index, level in enumerate(levels):
                 # Get the percentiles that correspond to the credibility level,
                 # as their labels in the `summary`.
-                lower_label, upper_label = [
+                lower_label, upper_label = (
                     get_percentile_label(percentile)
                     for percentile in _get_level_percentiles(level)
-                ]
+                )
                 # Get the data for each percentile.
                 t_lower, lower_data = _get_statistic_data(
                     summary,
@@ -297,8 +298,8 @@ def _plot_trajectories_by_condition(
                 # some incorrect time points.
                 if not (np.array(t_lower) == np.array(t_upper)).all():
                     raise ValueError(
-                        'The timepoints of the data for the upper and lower '
-                        'percentiles do not match.'
+                        "The timepoints of the data for the upper and lower "
+                        "percentiles do not match."
                     )
                 # Plot a shaded region between the data that correspond to the
                 # lower and upper percentiles.
@@ -318,29 +319,29 @@ def _plot_trajectories_by_condition(
                 ax.scatter(
                     measurements[0],
                     measurements[1],
-                    marker='o',
+                    marker="o",
                     facecolor=facecolor0,
                     edgecolor=(
-                        'white'
+                        "white"
                         if rgb_to_hls(*facecolor0)[1] < 0.5
-                        else 'black'
+                        else "black"
                     ),
                 )
 
 
 def _plot_trajectories_by_output(
-    summary: Dict[str, PredictionResult],
+    summary: dict[str, PredictionResult],
     condition_ids: Sequence[str],
     output_ids: Sequence[str],
     axes: matplotlib.axes.Axes,
     levels: Sequence[float],
-    level_opacities: Dict[int, float],
-    labels: Dict[str, str],
+    level_opacities: dict[int, float],
+    labels: dict[str, str],
     variable_colors: Sequence[RGB],
     average: str = MEDIAN,
     add_sd: bool = False,
-    grouped_measurements: Dict[
-        Tuple[str, str], Sequence[Sequence[float]]
+    grouped_measurements: dict[
+        tuple[str, str], Sequence[Sequence[float]]
     ] = None,
 ) -> None:
     """Plot predicted trajectories, with subplots grouped by output.
@@ -360,7 +361,7 @@ def _plot_trajectories_by_output(
         # next condition plot starts at the end of the previous condition plot.
         t0 = 0
         ax = axes.flat[output_index]
-        ax.set_title(f'Trajectory: {labels[output_id]}')
+        ax.set_title(f"Trajectory: {labels[output_id]}")
         # Each subplot is divided by conditions, with vertical lines.
         for condition_index, condition_id in enumerate(condition_ids):
             facecolor0 = variable_colors[condition_index]
@@ -368,7 +369,7 @@ def _plot_trajectories_by_output(
                 ax.axvline(
                     t0,
                     linewidth=2,
-                    color='k',
+                    color="k",
                 )
 
             t_max = t0
@@ -384,7 +385,7 @@ def _plot_trajectories_by_output(
             ax.plot(
                 t_average_shifted,
                 y_average,
-                'k-',
+                "k-",
             )
             if add_sd:
                 t_std, y_std = _get_statistic_data(
@@ -395,8 +396,8 @@ def _plot_trajectories_by_output(
                 )
                 if (t_std != t_average).all():
                     raise ValueError(
-                        'Unknown error: timepoints for average and standard '
-                        'deviation do not match.'
+                        "Unknown error: timepoints for average and standard "
+                        "deviation do not match."
                     )
                 ax.errorbar(
                     t_average_shifted,
@@ -408,10 +409,10 @@ def _plot_trajectories_by_output(
             for level_index, level in enumerate(levels):
                 # Get the percentiles that correspond to the credibility level,
                 # as their labels in the `summary`.
-                lower_label, upper_label = [
+                lower_label, upper_label = (
                     get_percentile_label(percentile)
                     for percentile in _get_level_percentiles(level)
-                ]
+                )
                 # Get the data for each percentile.
                 t_lower, lower_data = _get_statistic_data(
                     summary,
@@ -433,8 +434,8 @@ def _plot_trajectories_by_output(
                 # some incorrect time points.
                 if not (np.array(t_lower) == np.array(t_upper)).all():
                     raise ValueError(
-                        'The timepoints of the data for the upper and lower '
-                        'percentiles do not match.'
+                        "The timepoints of the data for the upper and lower "
+                        "percentiles do not match."
                     )
                 # Plot a shaded region between the data that correspond to the
                 # lower and upper percentiles.
@@ -454,12 +455,12 @@ def _plot_trajectories_by_output(
                 ax.scatter(
                     [t0 + _t for _t in measurements[0]],
                     measurements[1],
-                    marker='o',
+                    marker="o",
                     facecolor=facecolor0,
                     edgecolor=(
-                        'white'
+                        "white"
                         if rgb_to_hls(*facecolor0)[1] < 0.5
-                        else 'black'
+                        else "black"
                     ),
                 )
             # Set t0 to the last plotted timepoint of the current condition
@@ -468,8 +469,8 @@ def _plot_trajectories_by_output(
 
 
 def _get_condition_and_output_ids(
-    summary: Dict[str, PredictionResult]
-) -> Tuple[Sequence[str], Sequence[str]]:
+    summary: dict[str, PredictionResult],
+) -> tuple[Sequence[str], Sequence[str]]:
     """Get all condition and output IDs in a prediction summary.
 
     Parameters
@@ -496,7 +497,7 @@ def _get_condition_and_output_ids(
             ]
         ).all()
     ):
-        raise KeyError('All predictions must have the same set of conditions.')
+        raise KeyError("All predictions must have the same set of conditions.")
     condition_ids = all_condition_ids[0]
 
     output_ids = sorted(
@@ -515,7 +516,7 @@ def _handle_legends(
     fig: matplotlib.figure.Figure,
     axes: matplotlib.axes.Axes,
     levels: Union[float, Sequence[float]],
-    labels: Dict[str, str],
+    labels: dict[str, str],
     level_opacities: Sequence[float],
     variable_names: Sequence[str],
     variable_colors: Sequence[RGB],
@@ -525,7 +526,7 @@ def _handle_legends(
     average: str,
     add_sd: bool,
     grouped_measurements: Optional[
-        Dict[Tuple[str, str], Sequence[Sequence[float]]]
+        dict[tuple[str, str], Sequence[Sequence[float]]]
     ],
 ) -> None:
     """Add legends to a sampling prediction trajectories plot.
@@ -591,7 +592,7 @@ def _handle_legends(
     for index, level in enumerate(levels):
         ci_lines.append(
             [
-                f'{level}% CI',
+                f"{level}% CI",
                 Line2D(
                     *fake_data,
                     color=rgba2rgb(
@@ -608,16 +609,16 @@ def _handle_legends(
     if add_sd:
         capline = Line2D(
             *fake_data,
-            color=prediction_errorbar_settings['color'],
+            color=prediction_errorbar_settings["color"],
             # https://github.com/matplotlib/matplotlib/blob
             # /710fce3df95e22701bd68bf6af2c8adbc9d67a79/lib/matplotlib/
             # axes/_axes.py#L3424=
-            markersize=2.0 * prediction_errorbar_settings['capsize'],
+            markersize=2.0 * prediction_errorbar_settings["capsize"],
         )
-        average_title += ' + SD'
+        average_title += " + SD"
         barline = LineCollection(
             np.empty((2, 2, 2)),
-            color=prediction_errorbar_settings['color'],
+            color=prediction_errorbar_settings["color"],
         )
         average_line_object = ErrorbarContainer(
             (
@@ -636,13 +637,13 @@ def _handle_legends(
     if grouped_measurements:
         data_line = [
             [
-                'Data',
+                "Data",
                 Line2D(
                     *fake_data,
                     linewidth=0,
-                    marker='o',
-                    markerfacecolor='grey',
-                    markeredgecolor='white',
+                    marker="o",
+                    markerfacecolor="grey",
+                    markeredgecolor="white",
                 ),
             ]
         ]
@@ -651,16 +652,16 @@ def _handle_legends(
 
     # CI level, and variable name, legends.
     legend_options_top_right = {
-        'bbox_to_anchor': (1 + artist_padding, 1),
-        'loc': 'upper left',
+        "bbox_to_anchor": (1 + artist_padding, 1),
+        "loc": "upper left",
     }
     legend_options_bottom_right = {
-        'bbox_to_anchor': (1 + artist_padding, 0),
-        'loc': 'lower left',
+        "bbox_to_anchor": (1 + artist_padding, 0),
+        "loc": "lower left",
     }
     legend_titles = {
-        OUTPUT: 'Conditions',
-        CONDITION: 'Trajectories',
+        OUTPUT: "Conditions",
+        CONDITION: "Trajectories",
     }
     legend_variables = axes.flat[n_col - 1].legend(
         variable_lines[:, 1],
@@ -673,7 +674,7 @@ def _handle_legends(
         level_lines[:, 1],
         level_lines[:, 0],
         **legend_options_bottom_right,
-        title='Prediction',
+        title="Prediction",
     )
     fig.add_artist(legend_variables)
 
@@ -682,7 +683,7 @@ def _handle_colors(
     levels: Union[float, Sequence[float]],
     n_variables: int,
     reverse: bool = False,
-) -> Tuple[Sequence[float], Sequence[RGB]]:
+) -> tuple[Sequence[float], Sequence[RGB]]:
     """Calculate the colors for the prediction trajectories plot.
 
     Parameters
@@ -719,9 +720,9 @@ def sampling_prediction_trajectories(
     ensemble_prediction: EnsemblePrediction,
     levels: Union[float, Sequence[float]],
     title: str = None,
-    size: Tuple[float, float] = None,
+    size: tuple[float, float] = None,
     axes: matplotlib.axes.Axes = None,
-    labels: Dict[str, str] = None,
+    labels: dict[str, str] = None,
     axis_label_padding: int = 50,
     groupby: str = CONDITION,
     condition_gap: float = 0.01,
@@ -815,7 +816,7 @@ def sampling_prediction_trajectories(
     # Handle data
     grouped_measurements = {}
     if measurement_df is not None:
-        import petab
+        import petab.v1 as petab
 
         for condition_id in condition_ids:
             if petab.PARAMETER_SEPARATOR in condition_id:
@@ -825,16 +826,16 @@ def sampling_prediction_trajectories(
                 ) = condition_id.split(petab.PARAMETER_SEPARATOR)
             else:
                 preequilibration_condition_id, simulation_condition_id = (
-                    '',
+                    "",
                     condition_id,
                 )
             condition = {
                 petab.SIMULATION_CONDITION_ID: simulation_condition_id,
             }
             if preequilibration_condition_id:
-                condition[
-                    petab.PREEQUILIBRATION_CONDITION_ID
-                ] = preequilibration_condition_id
+                condition[petab.PREEQUILIBRATION_CONDITION_ID] = (
+                    preequilibration_condition_id
+                )
             for output_id in output_ids:
                 _df = petab.get_rows_for_condition(
                     measurement_df=measurement_df,
@@ -858,7 +859,7 @@ def sampling_prediction_trajectories(
         variable_names = condition_ids
         n_subplots = len(output_ids)
     else:
-        raise ValueError(f'Unsupported groupby value: {groupby}')
+        raise ValueError(f"Unsupported groupby value: {groupby}")
 
     level_opacities, variable_colors = _handle_colors(
         levels=levels,
@@ -878,8 +879,8 @@ def sampling_prediction_trajectories(
             axes = np.array([[axes]])
         if len(axes.flat) < n_subplots:
             raise ValueError(
-                'Provided `axes` contains insufficient subplots. At least '
-                f'{n_subplots} are required.'
+                "Provided `axes` contains insufficient subplots. At least "
+                f"{n_subplots} are required."
             )
     artist_padding = axis_label_padding / (fig.get_size_inches() * fig.dpi)[0]
 
@@ -935,26 +936,26 @@ def sampling_prediction_trajectories(
     xmin = min(ax.get_position().xmin for ax in axes.flat)
     ymin = min(ax.get_position().ymin for ax in axes.flat)
     xlabel = (
-        'Cumulative time across all conditions'
+        "Cumulative time across all conditions"
         if groupby == OUTPUT
-        else 'Time'
+        else "Time"
     )
     fig.text(
         0.5,
         ymin - artist_padding,
         xlabel,
-        ha='center',
-        va='center',
+        ha="center",
+        va="center",
         transform=fig.transFigure,
     )
     fig.text(
         xmin - artist_padding,
         0.5,
-        'Simulated values',
-        ha='center',
-        va='center',
+        "Simulated values",
+        ha="center",
+        va="center",
         transform=fig.transFigure,
-        rotation='vertical',
+        rotation="vertical",
     )
 
     # plt.tight_layout()  # Ruins layout for `groupby == OUTPUT`.
@@ -967,7 +968,7 @@ def sampling_parameter_cis(
     step: float = 0.05,
     show_median: bool = True,
     title: str = None,
-    size: Tuple[float, float] = None,
+    size: tuple[float, float] = None,
     ax: matplotlib.axes.Axes = None,
 ) -> matplotlib.axes.Axes:
     """
@@ -1031,7 +1032,7 @@ def sampling_parameter_cis(
                 np.append(x1, x1[::-1]),
                 np.append(y1, y2[::-1]),
                 color=colors[n],
-                label=str(level) + '% CI',
+                label=str(level) + "% CI",
             )
 
             if show_median:
@@ -1042,8 +1043,8 @@ def sampling_parameter_cis(
                     ax.plot(
                         [_median, _median],
                         [npar - _step, npar + _step],
-                        'k-',
-                        label='MCMC median',
+                        "k-",
+                        label="MCMC median",
                     )
 
             # increment height of boxes
@@ -1053,8 +1054,8 @@ def sampling_parameter_cis(
     ax.set_yticklabels(
         result.problem.get_reduced_vector(result.problem.x_names)
     )
-    ax.set_xlabel('Parameter value')
-    ax.set_ylabel('Parameter name')
+    ax.set_xlabel("Parameter value")
+    ax.set_ylabel("Parameter name")
 
     if title:
         ax.set_title(title)
@@ -1076,7 +1077,7 @@ def sampling_parameter_traces(
     stepsize: int = 1,
     use_problem_bounds: bool = True,
     suptitle: str = None,
-    size: Tuple[float, float] = None,
+    size: tuple[float, float] = None,
     ax: matplotlib.axes.Axes = None,
 ):
     """
@@ -1134,15 +1135,15 @@ def sampling_parameter_traces(
     par_ax = dict(zip(param_names, ax.flat))
 
     sns.set(style="ticks")
-    kwargs = {'edgecolor': "w", 'linewidth': 0.3, 's': 10}  # for edge color
+    kwargs = {"edgecolor": "w", "linewidth": 0.3, "s": 10}  # for edge color
 
     if full_trace:
-        kwargs['hue'] = "converged"
-        if len(params_fval[kwargs['hue']].unique()) == 1:
-            kwargs['palette'] = ["#477ccd"]
-        elif len(params_fval[kwargs['hue']].unique()) == 2:
-            kwargs['palette'] = ["#868686", "#477ccd"]
-        kwargs['legend'] = False
+        kwargs["hue"] = "converged"
+        if len(params_fval[kwargs["hue"]].unique()) == 1:
+            kwargs["palette"] = ["#477ccd"]
+        elif len(params_fval[kwargs["hue"]].unique()) == 2:
+            kwargs["palette"] = ["#868686", "#477ccd"]
+        kwargs["legend"] = False
 
     if result.sample_result.burn_in is None:
         _burn_in = 0
@@ -1163,12 +1164,12 @@ def sampling_parameter_traces(
         if full_trace and _burn_in > 0:
             _ax.axvline(
                 _burn_in,
-                linestyle='--',
+                linestyle="--",
                 linewidth=1.5,
-                color='k',
+                color="k",
             )
 
-        _ax.set_xlabel('iteration index')
+        _ax.set_xlabel("iteration index")
         _ax.set_ylabel(param_names[idx])
         if use_problem_bounds:
             _ax.set_ylim([theta_lb[idx], theta_ub[idx]])
@@ -1188,7 +1189,7 @@ def sampling_scatter(
     stepsize: int = 1,
     suptitle: str = None,
     diag_kind: str = "kde",
-    size: Tuple[float, float] = None,
+    size: tuple[float, float] = None,
     show_bounds: bool = True,
 ):
     """
@@ -1227,7 +1228,7 @@ def sampling_scatter(
 
     # TODO: Think this throws the axis errors in seaborn.
     ax = sns.pairplot(
-        params_fval.drop(['logPosterior', 'iteration'], axis=1),
+        params_fval.drop(["logPosterior", "iteration"], axis=1),
         diag_kind=diag_kind,
     )
 
@@ -1252,10 +1253,10 @@ def sampling_1d_marginals(
     i_chain: int = 0,
     par_indices: Sequence[int] = None,
     stepsize: int = 1,
-    plot_type: str = 'both',
-    bw_method: str = 'scott',
+    plot_type: str = "both",
+    bw_method: str = "scott",
     suptitle: str = None,
-    size: Tuple[float, float] = None,
+    size: tuple[float, float] = None,
 ):
     """
     Plot marginals.
@@ -1307,31 +1308,28 @@ def sampling_1d_marginals(
 
     # fig, ax = plt.subplots(nr_params, figsize=size)[1]
     for idx, par_id in enumerate(param_names):
-        if plot_type == 'kde':
-            # bw_method argument is deprecated... FutureWarning:
-            # The `bw_method` parameter is deprecated in favor of
-            # `bw_method` and `bw_adjust`.
+        if plot_type == "kde":
             # TODO: add bw_adjust as option?
             sns.kdeplot(
                 params_fval[par_id], bw_method=bw_method, ax=par_ax[par_id]
             )
-        elif plot_type == 'hist':
+        elif plot_type == "hist":
             # fixes usage of sns distplot which throws a future warning
             sns.histplot(
-                x=params_fval[par_id], ax=par_ax[par_id], stat='density'
+                x=params_fval[par_id], ax=par_ax[par_id], stat="density"
             )
             sns.rugplot(x=params_fval[par_id], ax=par_ax[par_id])
-        elif plot_type == 'both':
+        elif plot_type == "both":
             sns.histplot(
                 x=params_fval[par_id],
                 kde=True,
                 ax=par_ax[par_id],
-                stat='density',
+                stat="density",
             )
             sns.rugplot(x=params_fval[par_id], ax=par_ax[par_id])
 
         par_ax[par_id].set_xlabel(param_names[idx])
-        par_ax[par_id].set_ylabel('Density')
+        par_ax[par_id].set_ylabel("Density")
 
     sns.despine()
 
@@ -1386,7 +1384,8 @@ def get_data_to_plot(
         warnings.warn(
             "Burn in index not found in the results, the full chain "
             "will be shown.\nYou may want to use, e.g., "
-            "`pypesto.sample.geweke_test`."
+            "`pypesto.sample.geweke_test`.",
+            stacklevel=2,
         )
         _burn_in = 0
     else:
@@ -1415,14 +1414,14 @@ def get_data_to_plot(
 
     # transform ndarray to pandas for the use of seaborn
     pd_params = pd.DataFrame(arr_param, columns=param_names)
-    pd_fval = pd.DataFrame(data=arr_fval, columns=['logPosterior'])
+    pd_fval = pd.DataFrame(data=arr_fval, columns=["logPosterior"])
 
-    pd_iter = pd.DataFrame(data=indices, columns=['iteration'])
+    pd_iter = pd.DataFrame(data=indices, columns=["iteration"])
 
     if full_trace:
-        converged = np.zeros((len(arr_fval)))
+        converged = np.zeros(len(arr_fval))
         converged[_burn_in:] = 1
-        pd_conv = pd.DataFrame(data=converged, columns=['converged'])
+        pd_conv = pd.DataFrame(data=converged, columns=["converged"])
 
         params_fval = pd.concat(
             [pd_params, pd_fval, pd_iter, pd_conv], axis=1, ignore_index=False
