@@ -1,4 +1,4 @@
-from typing import List, Optional, Sequence, Tuple, Union
+from collections.abc import Sequence
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -7,8 +7,9 @@ from mpl_toolkits.axes_grid1 import inset_locator
 
 from pypesto.util import delete_nan_inf
 
+from ..C import ALL, COLOR, WATERFALL_MAX_VALUE
 from ..result import Result
-from .clust_color import RGBA, assign_colors
+from .clust_color import assign_colors
 from .misc import (
     process_offset_y,
     process_result_list,
@@ -19,17 +20,17 @@ from .reference_points import ReferencePoint, create_references
 
 
 def waterfall(
-    results: Union[Result, Sequence[Result]],
-    ax: Optional[plt.Axes] = None,
-    size: Optional[Tuple[float]] = (18.5, 10.5),
-    y_limits: Optional[Tuple[float]] = None,
-    scale_y: Optional[str] = 'log10',
-    offset_y: Optional[float] = None,
-    start_indices: Optional[Union[Sequence[int], int]] = None,
+    results: Result | Sequence[Result],
+    ax: plt.Axes | None = None,
+    size: tuple[float, float] | None = (18.5, 10.5),
+    y_limits: tuple[float] | None = None,
+    scale_y: str | None = "log10",
+    offset_y: float | None = None,
+    start_indices: Sequence[int] | int | None = None,
     n_starts_to_zoom: int = 0,
-    reference: Optional[Sequence[ReferencePoint]] = None,
-    colors: Optional[Union[RGBA, Sequence[RGBA]]] = None,
-    legends: Optional[Union[Sequence[str], str]] = None,
+    reference: Sequence[ReferencePoint] | None = None,
+    colors: COLOR | list[COLOR] | np.ndarray | None = None,
+    legends: Sequence[str] | str | None = None,
     order_by_id: bool = False,
 ):
     """
@@ -60,7 +61,7 @@ def waterfall(
         Reference points for optimization results, containing at least a
         function value fval
     colors:
-        Colors or single color  for plotting. If not set, clustering is done
+        List of colors or single color for plotting. If not set, clustering is done
         and colors are assigned automatically
     legends:
         Labels for line plots, one label per result object
@@ -84,7 +85,7 @@ def waterfall(
     if n_starts_to_zoom:
         # create zoom in
         inset_axes = inset_locator.inset_axes(
-            ax, width="30%", height="30%", loc='center right'
+            ax, width="30%", height="30%", loc="center right"
         )
         inset_locator.mark_inset(ax, inset_axes, loc1=2, loc2=4)
     else:
@@ -96,6 +97,13 @@ def waterfall(
     # handle `order_by_id`
     if order_by_id:
         start_id_ordering = get_ordering_by_start_id(results)
+        # Set start indices to all, and save actual start indices for later,
+        # so that all fvals are retrieved by `process_offset_for_list`.
+        # This enables use of `order_by_id` with `start_indices`.
+        ordered_start_indices = process_start_indices(
+            result=results[0], start_indices=start_indices
+        )
+        start_indices = ALL
 
     refs = create_references(references=reference)
 
@@ -113,7 +121,11 @@ def waterfall(
         max_len_fvals = np.max([max_len_fvals, *fvals_raw.shape])
 
         # remove colors where value is infinite if colors were passed on
-        if colors[j] is not None and fvals_raw.size == colors[j].shape[0]:
+        if (
+            colors[j] is not None
+            and isinstance(colors[j], np.ndarray)
+            and fvals_raw.size == colors[j].shape[0]
+        ):
             colors[j] = colors[j][
                 np.isfinite(np.transpose(fvals_raw)).flatten()
             ]
@@ -121,7 +133,9 @@ def waterfall(
         # parse input
         if order_by_id:
             start_ids = [s.id for s in results[j].optimize_result.list]
-            fvals_raw_is_finite = np.isfinite(fvals_raw)
+            fvals_raw_is_finite = np.isfinite(fvals_raw) & (
+                np.absolute(fvals_raw) < WATERFALL_MAX_VALUE
+            )
 
             fvals = []
             for start_id in start_id_ordering:
@@ -130,12 +144,15 @@ def waterfall(
                     fvals.append(fvals_raw[start_index])
                 else:
                     fvals.append(None)
+            fvals = np.array(fvals)[ordered_start_indices]
         else:
             # remove nan or inf values in fvals
             # also remove extremely large values. These values result in `inf`
             # values in the output of `scipy.cluster.hierarchy.linkage` in the
             # method `pypesto.util.assign_clusters`, which raises errors.
-            _, fvals = delete_nan_inf(fvals=fvals_raw, magnitude_bound=1e100)
+            _, fvals = delete_nan_inf(
+                fvals=fvals_raw, magnitude_bound=WATERFALL_MAX_VALUE
+            )
             fvals.sort()
 
         # assign colors
@@ -159,55 +176,56 @@ def waterfall(
                 ax=inset_axes,
                 colors=coloring[:n_starts_to_zoom],
             )
+            # remove the title and axes labels for the zoom in subplot
+            inset_axes.set(title=None, xlabel=None, ylabel=None)
 
     # apply changes specified be the user to the axis object
     ax = handle_options(ax, max_len_fvals, refs, y_limits, offset_y)
     if inset_axes is not None:
         inset_axes = handle_options(
-            inset_axes, n_starts_to_zoom, refs, y_limits, offset_y
+            inset_axes, n_starts_to_zoom, refs, None, offset_y
         )
 
     if any(legends):
         ax.legend()
     # labels
-    ax.set_xlabel('Ordered optimizer run')
+    ax.set_xlabel("Ordered optimizer run")
     if offset_y == 0.0:
-        ax.set_ylabel('Function value')
+        ax.set_ylabel("Function value")
     else:
-        ax.set_ylabel(f'Objective value (offset={offset_y:0.3e})')
-    ax.set_title('Waterfall plot')
+        ax.set_ylabel(f"Objective value (offset={offset_y:0.3e})")
+    ax.set_title("Waterfall plot")
     return ax
 
 
 def waterfall_lowlevel(
     fvals,
-    ax: Optional[plt.Axes] = None,
-    size: Optional[Tuple[float]] = (18.5, 10.5),
-    scale_y: str = 'log10',
+    ax: plt.Axes | None = None,
+    size: tuple[float] | None = (18.5, 10.5),
+    scale_y: str = "log10",
     offset_y: float = 0.0,
-    colors: Optional[Union[RGBA, Sequence[RGBA]]] = None,
-    legend_text: Optional[str] = None,
+    colors: COLOR | list[COLOR] | np.ndarray | None = None,
+    legend_text: str | None = None,
 ):
     """
     Plot waterfall plot using list of function values.
 
     Parameters
     ----------
-    fvals: numeric list or array
+    fvals:
         Including values need to be plotted. `None` values indicate that the
         corresponding start index should be skipped.
-    ax: matplotlib.Axes
+    ax:
         Axes object to use.
     size:
         Figure size (width, height) in inches. Is only applied when no ax
         object is specified
-    scale_y: str, optional
+    scale_y:
         May be logarithmic or linear ('log10' or 'lin')
     offset_y:
         offset for the y-axis, if it is supposed to be in log10-scale
-    colors: list, or RGBA, optional
-        list of colors, or single color
-        color or list of colors for plotting. If not set, clustering is done
+    colors:
+        Color recognized by matplotlib or list of colors for plotting. If not set, clustering is done
         and colors are assigned automatically
     legend_text:
         Label for line plots
@@ -234,34 +252,27 @@ def waterfall_lowlevel(
     # plot
     ax.xaxis.set_major_locator(MaxNLocator(integer=True))
     # plot line
-    if scale_y == 'log10':
+    if scale_y == "log10":
         ax.semilogy(start_indices, fvals, color=[0.7, 0.7, 0.7, 0.6])
+        ax.set_yscale("log")
     else:
         ax.plot(start_indices, fvals, color=[0.7, 0.7, 0.7, 0.6])
 
-    # plot points
-    for j in start_indices:
-        # parse data for plotting
-        color = colors[start_indices.index(j)]
-        fval = fvals[start_indices.index(j)]
-        if j == start_indices[0]:
-            tmp_legend = legend_text
-        else:
-            tmp_legend = None
-
-        # line plot (linear or logarithmic)
-        if scale_y == 'log10':
-            ax.semilogy(
-                j, fval, color=color, marker='o', label=tmp_legend, alpha=1.0
-            )
-        else:
-            ax.plot(
-                j, fval, color=color, marker='o', label=tmp_legend, alpha=1.0
-            )
+    # Overlay with scatter points with individual colors
+    ax.scatter(
+        start_indices,
+        fvals,
+        c=colors,
+        marker="o",
+        linewidth=1.0,
+        label=legend_text,
+        zorder=2.0,
+        alpha=1.0,
+    )
 
     # check if y-axis has a reasonable scale
     y_min, y_max = ax.get_ylim()
-    if scale_y == 'log10':
+    if scale_y == "log10":
         if np.log10(y_max) - np.log10(y_min) < 1.0:
             ax.set_ylim(
                 ax.dataLim.y0 - 0.001 * abs(ax.dataLim.y0),
@@ -273,12 +284,12 @@ def waterfall_lowlevel(
             ax.set_ylim(y_mean - 0.5, y_mean + 0.5)
 
     # labels
-    ax.set_xlabel('Ordered optimizer run')
+    ax.set_xlabel("Ordered optimizer run")
     if offset_y == 0.0:
-        ax.set_ylabel('Function value')
+        ax.set_ylabel("Function value")
     else:
-        ax.set_ylabel('Objective value (offset={offset_y:0.3e})')
-    ax.set_title('Waterfall plot')
+        ax.set_ylabel("Objective value (offset={offset_y:0.3e})")
+    ax.set_title("Waterfall plot")
     if legend_text is not None:
         ax.legend()
 
@@ -288,10 +299,10 @@ def waterfall_lowlevel(
 def process_offset_for_list(
     offset_y: float,
     results: Sequence[Result],
-    scale_y: Optional[str],
-    start_indices: Optional[Sequence[int]] = None,
-    references: Optional[Sequence[ReferencePoint]] = None,
-) -> Tuple[List[np.ndarray], float]:
+    scale_y: str | None,
+    start_indices: Sequence[int] | None = None,
+    references: Sequence[ReferencePoint] | None = None,
+) -> tuple[list[np.ndarray], float]:
     """
     Compute common offset_y and add it to `fvals` of results.
 
@@ -320,9 +331,9 @@ def process_offset_for_list(
     fvals_all = []
     for result in results:
         fvals = np.array(result.optimize_result.fval)
-        # todo: order of results plays a role
-        start_indices = process_start_indices(result, start_indices)
-        fvals = fvals[start_indices]
+
+        result_start_indices = process_start_indices(result, start_indices)
+        fvals = fvals[result_start_indices]
         # if none of the fvals are finite, set default value to zero as
         # np.nanmin will error for an empty array
         if np.isfinite(fvals).any():
@@ -332,7 +343,7 @@ def process_offset_for_list(
 
     # if there are references, also account for those
     if references:
-        min_val = min(min_val, np.nanmin([r['fval'] for r in references]))
+        min_val = min(min_val, np.nanmin([r["fval"] for r in references]))
 
     offset_y = process_offset_y(offset_y, scale_y, float(min_val))
 
@@ -340,7 +351,7 @@ def process_offset_for_list(
     return [fvals + offset_y for fvals in fvals_all], offset_y
 
 
-def get_ordering_by_start_id(results: Sequence[Result]) -> List[int]:
+def get_ordering_by_start_id(results: Sequence[Result]) -> list[int]:
     """Get an ordering of start IDs.
 
     The ordering is generated by taking the best function value for each
@@ -425,7 +436,7 @@ def handle_options(ax, max_len_fvals, ref, y_limits, offset_y):
         ax.plot(
             [0, max_len_fvals - 1],
             [i_ref.fval + offset_y, i_ref.fval + offset_y],
-            '--',
+            "--",
             color=i_ref.color,
             label=i_ref.legend,
         )
